/* ************************************************************************
 *   File: act.other.c                                   Part of CircleMUD *
 *  Usage: Miscellaneous player-level commands                             *
 *                                                                         *
 *  All rights reserved.  See license.doc for complete information.        *
 *                                                                         *
 *  Copyright (C) 1993, 94 by the Trustees of the Johns Hopkins University *
 *  CircleMUD is based on DikuMUD, Copyright (C) 1990, 1991.               *
 ************************************************************************ */

#define __ACT_OTHER_C__

#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"

#include "coins.h"
#include "comm.h"
#include "constants.h"
#include "db.h"
#include "handler.h"
#include "house.h"
#include "ident.h"
#include "interpreter.h"
#include "screen.h"
#include "spells.h"


/* extern variables */
extern const char *class_abbrevs[];
extern const char *race_abbrevs[];
extern int free_rent;
extern int max_filesize;
extern int nameserver_is_slow;
extern int auto_save;
extern int track_through_doors;
extern int allow_stat_edit;
extern int two_hand_limit;

/* extern procedures */
void list_skills_perct(struct char_data *ch, struct char_data *vict);
void list_skills(struct char_data *ch);
void appear(struct char_data *ch);
void write_aliases(struct char_data *ch);
void perform_immort_vis(struct char_data *ch);
ACMD(do_gen_comm);
void die(struct char_data *ch, struct char_data *killer);
void Crash_rentsave(struct char_data *ch, int cost);
int is_night(void);

/* local functions */
int perform_group(struct char_data *ch, struct char_data *vict);
void print_group(struct char_data *ch);

ACMD(do_bury);
ACMD(do_dig);
ACMD(do_display);
ACMD(do_fly);
ACMD(do_gen_tog);
ACMD(do_gen_write);
ACMD(do_group);
ACMD(do_land);
ACMD(do_not_here);
ACMD(do_practice);
ACMD(do_quit);
ACMD(do_report);
ACMD(do_sac);
ACMD(do_save);
ACMD(do_search);
ACMD(do_split);
ACMD(do_title);
ACMD(do_ungroup);
ACMD(do_use);
ACMD(do_visible);
ACMD(do_wimpy);


ACMD(do_quit)
{
	if (IS_NPC(ch) || !ch->desc)
		return;

	if (subcmd != SCMD_QUIT && GET_LEVEL(ch) < LVL_IMMORT)
		send_to_char(ch, "You have to type quit--no less, to quit!\r\n");
	else if (GET_POS(ch) == POS_FIGHTING)
		send_to_char(ch, "No way!  You're fighting for your life!\r\n");
	else if (GET_POS(ch) < POS_STUNNED) {
		send_to_char(ch, "You die before your time...\r\n");
		die(ch, NULL);
	} else {
		act("$n has left the game.", TRUE, ch, 0, 0, TO_ROOM);
		mudlog(NRM, MAX(LVL_IMMORT, GET_INVIS_LEV(ch)), TRUE, 
				 "%s has quit the game.", GET_NAME(ch));
		send_to_char(ch, "Goodbye, friend.. Come back soon!\r\n");

		/*  We used to check here for duping attempts, but we may as well
		 *  do it right in extract_char(), since there is no check if a
		 *  player rents out and it can leave them in an equally screwy
		 *  situation.
		 */

		if (free_rent)
			Crash_rentsave(ch, 0);

		/* If someone is quitting in their house, let them load back here. */
		if (!PLR_FLAGGED(ch, PLR_LOADROOM) && 
			 ROOM_FLAGGED(IN_ROOM(ch), ROOM_HOUSE))
			GET_LOADROOM(ch) = IN_ROOM(ch);

		extract_char(ch);		/* Char is saved before extracting. */
	}
}



ACMD(do_save)
{
	if (IS_NPC(ch) || !ch->desc)
		return;

	/* Only tell the char we're saving if they actually typed "save" */
	if (cmd) {
		/*
		 * This prevents item duplication by two PC's using coordinated saves
		 * (or one PC with a house) and system crashes. Note that houses are
		 * still automatically saved without this enabled. This code assumes
		 * that guest immortals aren't trustworthy. If you've disabled guest
		 * immortal advances from mortality, you may want < instead of <=.
		 */
		if (auto_save && GET_LEVEL(ch) <= LVL_IMMORT) {
			send_to_char(ch, "Saving aliases.\r\n");
			write_aliases(ch);
			return;
		}
		send_to_char(ch, "Saving %s and aliases.\r\n", GET_NAME(ch));
	}

	write_aliases(ch);
	save_char(ch);
	Crash_crashsave(ch);
	if (ROOM_FLAGGED(IN_ROOM(ch), ROOM_HOUSE_CRASH))
		House_crashsave(GET_ROOM_VNUM(IN_ROOM(ch)));
}


/* generic function for commands which are normally overridden by
   special procedures - i.e., shop commands, mail commands, etc. */
ACMD(do_not_here)
{
	send_to_char(ch, "Sorry, but you cannot do that here!\r\n");
}


ACMD(do_practice)
{
	struct char_data *vict;
	char arg[MAX_INPUT_LENGTH];

	one_argument(argument, arg);

	if (*arg) {
		if (GET_LEVEL(ch) >= LVL_GOD) {
			if(!(vict = get_char_vis(ch, arg, NULL, FIND_CHAR_WORLD))) {
				send_to_char(ch, NOPERSON);
				return;
			}
			list_skills_perct(ch, vict);
			return;
		}
		send_to_char(ch, "You can only practice skills in your guild.\r\n");
	}
	else
		list_skills(ch);
}

#if 0
/* The above do_practice allows immorts to check on the skill level of
 * a specific player.  -ejg  */
ACMD(do_practice)
{
	char arg[MAX_INPUT_LENGTH];

	if (IS_NPC(ch))
		return;

	one_argument(argument, arg);

	if (*arg)
		send_to_char(ch, "You can only practice skills in your guild.\r\n");
	else
		list_skills(ch);
}
#endif


ACMD(do_visible)
{
	if (GET_LEVEL(ch) >= LVL_IMMORT) {
		perform_immort_vis(ch);
		return;
	}

	if AFF_FLAGGED(ch, AFF_INVISIBLE) {
		appear(ch);
		send_to_char(ch, "You break the spell of invisibility.\r\n");
	} else
		send_to_char(ch, "You are already visible.\r\n");
}



ACMD(do_title)
{
	skip_spaces(&argument);
	delete_doubledollar(argument);

	if (IS_NPC(ch))
		send_to_char(ch, "Your title is fine... go away.\r\n");
	else if (PLR_FLAGGED(ch, PLR_NOTITLE))
		send_to_char(ch, "You can't title yourself -- you shouldn't have abused it!\r\n");
	else if (strstr(argument, "(") || strstr(argument, ")"))
		send_to_char(ch, "Titles can't contain the ( or ) characters.\r\n");
	else if (strlen(argument) > MAX_TITLE_LENGTH)
		send_to_char(ch, "Sorry, titles can't be longer than %d characters.\r\n", MAX_TITLE_LENGTH);
	else {
		set_title(ch, argument);
		send_to_char(ch, "Okay, you're now %s %s.\r\n", GET_NAME(ch), GET_TITLE(ch));
	}
}


int perform_group(struct char_data *ch, struct char_data *vict)
{
	if (AFF_FLAGGED(vict, AFF_GROUP) || !CAN_SEE(ch, vict))
		return (0);

	AFF_FLAGS(vict).AFF_GROUP = 1;
	if (ch != vict)
		act("$N is now a member of your group.", FALSE, ch, 0, vict, TO_CHAR);
	act("You are now a member of $n's group.", FALSE, ch, 0, vict, TO_VICT);
	act("$N is now a member of $n's group.", FALSE, ch, 0, vict, TO_NOTVICT);
	return (1);
}


void print_group(struct char_data *ch)
{
	struct char_data *k;
	struct follow_type *f;

	if (!AFF_FLAGGED(ch, AFF_GROUP))
		send_to_char(ch, "But you are not the member of a group!\r\n");
	else {
		char buf[MAX_STRING_LENGTH];

		send_to_char(ch, "Your group consists of:\r\n");

		k = (ch->master ? ch->master : ch);

		if (AFF_FLAGGED(k, AFF_GROUP)) {
			snprintf(buf, sizeof(buf),
						"     [%3dH %3dM %3dV] [%2d %s %s] $N (Head of group)",
						GET_HIT(k), GET_MANA(k), GET_MOVE(k), GET_LEVEL(k), 
						RACE_ABBR(k), CLASS_ABBR(k));
			act(buf, FALSE, ch, 0, k, TO_CHAR);
		}

		for (f = k->followers; f; f = f->next) {
			if (!AFF_FLAGGED(f->follower, AFF_GROUP))
				continue;

			snprintf(buf, sizeof(buf), 
						"     [%3dH %3dM %3dV] [%2d %s %s] $N", 
						GET_HIT(f->follower), GET_MANA(f->follower), 
						GET_MOVE(f->follower), GET_LEVEL(f->follower),
						RACE_ABBR(f->follower), CLASS_ABBR(f->follower));
			act(buf, FALSE, ch, 0, f->follower, TO_CHAR);
		}
	}
}



ACMD(do_group)
{
	char buf[MAX_STRING_LENGTH];
	struct char_data *vict;
	struct follow_type *f;
	int found;

	one_argument(argument, buf);

	if (!*buf) {
		print_group(ch);
		return;
	}

	if (ch->master) {
		act("You can not enroll group members without being head of a group.",
			 FALSE, ch, 0, 0, TO_CHAR);
		return;
	}

	if (!str_cmp(buf, "all")) {
		perform_group(ch, ch);
		for (found = 0, f = ch->followers; f; f = f->next)
			found += perform_group(ch, f->follower);
		if (!found)
			send_to_char(ch, "Everyone following you is already in your group.\r\n");
		return;
	}

	if (!(vict = get_char_vis(ch, buf, NULL, FIND_CHAR_ROOM)))
		send_to_char(ch, "%s", NOPERSON);
	else if ((vict->master != ch) && (vict != ch))
		act("$N must follow you to enter your group.", FALSE, ch, 0, vict, TO_CHAR);
	else {
		if (!AFF_FLAGGED(vict, AFF_GROUP))
			perform_group(ch, vict);
		else {
			if (ch != vict)
				act("$N is no longer a member of your group.", FALSE, ch, 0, vict, TO_CHAR);
			act("You have been kicked out of $n's group!", FALSE, ch, 0, vict, TO_VICT);
			act("$N has been kicked out of $n's group!", FALSE, ch, 0, vict, TO_NOTVICT);
			AFF_FLAGS(vict).AFF_GROUP = 0;
		}
	}
}



ACMD(do_ungroup)
{
	char buf[MAX_INPUT_LENGTH];
	struct follow_type *f, *next_fol;
	struct char_data *tch;

	one_argument(argument, buf);

	if (!*buf) {
		if (ch->master || !(AFF_FLAGGED(ch, AFF_GROUP))) {
			send_to_char(ch, "But you lead no group!\r\n");
			return;
		}

		for (f = ch->followers; f; f = next_fol) {
			next_fol = f->next;
			if (AFF_FLAGGED(f->follower, AFF_GROUP)) {
				AFF_FLAGS(f->follower).AFF_GROUP = 0;
				act("$N has disbanded the group.", TRUE, f->follower, NULL, ch, TO_CHAR);
				if (!AFF_FLAGGED(f->follower, AFF_CHARM))
					stop_follower(f->follower);
			}
		}

		AFF_FLAGS(ch).AFF_GROUP = 0;
		send_to_char(ch, "You disband the group.\r\n");
		return;
	}
	if (!(tch = get_char_vis(ch, buf, NULL, FIND_CHAR_ROOM))) {
		send_to_char(ch, "There is no such person!\r\n");
		return;
	}
	if (tch->master != ch) {
		send_to_char(ch, "That person is not following you!\r\n");
		return;
	}

	if (!AFF_FLAGGED(tch, AFF_GROUP)) {
		send_to_char(ch, "That person isn't in your group.\r\n");
		return;
	}

	AFF_FLAGS(tch).AFF_GROUP = 0;

	act("$N is no longer a member of your group.", FALSE, ch, 0, tch, TO_CHAR);
	act("You have been kicked out of $n's group!", FALSE, ch, 0, tch, TO_VICT);
	act("$N has been kicked out of $n's group!", FALSE, ch, 0, tch, TO_NOTVICT);
 
	if (!AFF_FLAGGED(tch, AFF_CHARM))
		stop_follower(tch);
}


ACMD(do_report)
{
	char buf[MAX_STRING_LENGTH];
	struct char_data *k;
	struct follow_type *f;

	if (!AFF_FLAGGED(ch, AFF_GROUP)) {
		send_to_char(ch, "But you are not a member of any group!\r\n");
		return;
	}

	snprintf(buf, sizeof(buf), "$n reports: %d/%dH, %d/%dM, %d/%dV\r\n",
				GET_HIT(ch), GET_MAX_HIT(ch),
				GET_MANA(ch), GET_MAX_MANA(ch),
				GET_MOVE(ch), GET_MAX_MOVE(ch));

	k = (ch->master ? ch->master : ch);

	for (f = k->followers; f; f = f->next)
		if (AFF_FLAGGED(f->follower, AFF_GROUP) && f->follower != ch)
			act(buf, TRUE, ch, NULL, f->follower, TO_VICT);

	if (k != ch)
		act(buf, TRUE, ch, NULL, k, TO_VICT);

	send_to_char(ch, "You report to the group.\r\n");
}



ACMD(do_split)
{
	char buf[MAX_INPUT_LENGTH], cointype[10];
	int amount, num, share, rest, type;
	size_t len;
	struct char_data *k;
	struct follow_type *f;

	if (IS_NPC(ch))
		return;

	two_arguments(argument, buf, cointype);

	if (is_number(buf)) {
		amount = atoi(buf);

		if (!strcmp(cointype, "platinum") || is_abbrev(cointype, "platinum"))
			type = PLAT_COINS;
		else if (!strcmp(cointype, "gold") || is_abbrev(cointype, "gold"))
			type = GOLD_COINS;
		else if (!strcmp(cointype, "silver") || is_abbrev(cointype, "silver"))
			type = SILVER_COINS;
		else if (!strcmp(cointype, "copper") || is_abbrev(cointype, "copper") || 
					!strcmp(cointype, "coins") || is_abbrev(cointype, "coins"))
			type = COPPER_COINS;
		else {
			send_to_char(ch, "Split how many of what?\r\n");
			return;
		}
		
		if (amount > GET_MONEY(ch, type)) {
			send_to_char(ch, "You don't seem to have that many %s coins to "
							 "split.\r\n", coin_types[type]);
			return;
		}

		k = (ch->master ? ch->master : ch);

		if (AFF_FLAGGED(k, AFF_GROUP) && (IN_ROOM(k) == IN_ROOM(ch)))
			num = 1;
		else
			num = 0;

		for (f = k->followers; f; f = f->next)
			if (AFF_FLAGGED(f->follower, AFF_GROUP) &&
				 (!IS_NPC(f->follower)) &&
				 (IN_ROOM(f->follower) == IN_ROOM(ch)))
				num++;

		if (num && AFF_FLAGGED(ch, AFF_GROUP)) {
			share = amount / num;
			rest = amount % num;
		} else {
			send_to_char(ch, "With whom do you wish to share your %s coins?\r\n",
							 coin_types[type]);
			return;
		}

		remove_money_from_char(ch, (share * (num - 1)), type);

		/* Abusing signed/unsigned to make sizeof work. */
		len = snprintf(buf, sizeof(buf), "%s splits %d %s coins; you "
							"receive %d.\r\n", GET_NAME(ch), amount, 
							coin_types[type], share);
		if (rest && len < sizeof(buf)) {
			snprintf(buf + len, sizeof(buf) - len, 
						"%d %s coin%s %s not splitable, so %s keeps the money.\r\n",
						rest, coin_types[type], rest, (rest == 1) ? "" : "s", 
						(rest == 1) ? "was" : "were", GET_NAME(ch));
		}
		if (AFF_FLAGGED(k, AFF_GROUP) && IN_ROOM(k) == IN_ROOM(ch) &&
			 !IS_NPC(k) && k != ch) {
			add_money_to_char(k, share, type);
			send_to_char(k, "%s", buf);
		}

		for (f = k->followers; f; f = f->next) {
			if (AFF_FLAGGED(f->follower, AFF_GROUP) &&
				 (!IS_NPC(f->follower)) &&
				 (IN_ROOM(f->follower) == IN_ROOM(ch)) &&
				 f->follower != ch) {

				add_money_to_char(f->follower, share, type);
				send_to_char(f->follower, "%s", buf);
			}
		}
		send_to_char(ch, "You split %d coins among %d members -- %d coins "
						 "each.\r\n", amount, num, share);

		if (rest) {
			send_to_char(ch, "%d coin%s %s not splitable, so you keep the "
							 "money.\r\n", rest, (rest == 1) ? "" : "s", 
							 (rest == 1) ? "was" : "were");
			add_money_to_char(ch, rest, type);
		}
	} else {
		send_to_char(ch, "How many coins do you wish to split with your group?\r\n");
		return;
	}
}



ACMD(do_use)
{
	char buf[MAX_INPUT_LENGTH], arg[MAX_INPUT_LENGTH];
	struct obj_data *mag_item;

	half_chop(argument, arg, buf);
	if (!*arg) {
		send_to_char(ch, "What do you want to %s?\r\n", CMD_NAME);
		return;
	}
	mag_item = GET_EQ(ch, WEAR_HOLD);

	if (!mag_item || !isname(arg, mag_item->name)) {
		switch (subcmd) {
			case SCMD_RECITE:
			case SCMD_QUAFF:
				if (!(mag_item = get_obj_in_list_vis(ch, arg, NULL, ch->carrying))) {
					send_to_char(ch, "You don't seem to have %s %s.\r\n", AN(arg), arg);
					return;
				}
				break;
			case SCMD_USE:
				send_to_char(ch, "You don't seem to be holding %s %s.\r\n", AN(arg), arg);
				return;
			default:
				log("SYSERR: Unknown subcmd %d passed to do_use.", subcmd);
				return;
		}
	}
	switch (subcmd) {
		case SCMD_QUAFF:
			if (GET_OBJ_TYPE(mag_item) != ITEM_POTION) {
				send_to_char(ch, "You can only quaff potions.\r\n");
				return;
			}
			break;
		case SCMD_RECITE:
			if (GET_OBJ_TYPE(mag_item) != ITEM_SCROLL) {
				send_to_char(ch, "You can only recite scrolls.\r\n");
				return;
			}
			break;
		case SCMD_USE:
			if ((GET_OBJ_TYPE(mag_item) != ITEM_WAND) &&
				 (GET_OBJ_TYPE(mag_item) != ITEM_STAFF)) {
				send_to_char(ch, "You can't seem to figure out how to use it.\r\n");
				return;
			}
			break;
	}

	mag_objectmagic(ch, mag_item, buf);
}



ACMD(do_wimpy)
{
	char arg[MAX_INPUT_LENGTH];
	int wimp_lev;

	/* 'wimp_level' is a player_special. -gg 2/25/98 */
	if (IS_NPC(ch))
		return;

	one_argument(argument, arg);

	if (!*arg) {
		if (GET_WIMP_LEV(ch)) {
			send_to_char(ch, "Your current wimp level is %d hit points.\r\n", GET_WIMP_LEV(ch));
			return;
		} else {
			send_to_char(ch, "At the moment, you're not a wimp.  (sure, sure...)\r\n");
			return;
		}
	}
	if (isdigit(*arg)) {
		if ((wimp_lev = atoi(arg)) != 0) {
			if (wimp_lev < 0)
				send_to_char(ch, "Heh, heh, heh.. we are jolly funny today, eh?\r\n");
			else if (wimp_lev > GET_MAX_HIT(ch))
				send_to_char(ch, "That doesn't make much sense, now does it?\r\n");
			else if (wimp_lev > (GET_MAX_HIT(ch) / 2))
				send_to_char(ch, "You can't set your wimp level above half your hit points.\r\n");
			else {
				send_to_char(ch, "Okay, you'll wimp out if you drop below %d hit points.\r\n", wimp_lev);
				GET_WIMP_LEV(ch) = wimp_lev;
			}
		} else {
			send_to_char(ch, "Okay, you'll now tough out fights to the bitter end.\r\n");
			GET_WIMP_LEV(ch) = 0;
		}
	} else
		send_to_char(ch, "Specify at how many hit points you want to wimp out at.  (0 to disable)\r\n");
}


ACMD(do_display)
{
	size_t i;

	if (IS_NPC(ch)) {
		send_to_char(ch, "Mosters don't need displays.  Go away.\r\n");
		return;
	}
	skip_spaces(&argument);

	if (!*argument) {
		send_to_char(ch, "Usage: prompt { { H | M | V | G | X } | all | none }\r\n");
		return;
	}
	if (!str_cmp(argument, "on") || !str_cmp(argument, "all")) {
		PRF_FLAGS(ch).PRF_DISPHP   = 1;
		PRF_FLAGS(ch).PRF_DISPMANA = 1;
		PRF_FLAGS(ch).PRF_DISPMOVE = 1;
		PRF_FLAGS(ch).PRF_DISPGOLD = 1;
		PRF_FLAGS(ch).PRF_DISPEXP  = 1;
	} else if (!str_cmp(argument, "off") || !str_cmp(argument, "none"))  {
		PRF_FLAGS(ch).PRF_DISPHP   = 0;
		PRF_FLAGS(ch).PRF_DISPMANA = 0;
		PRF_FLAGS(ch).PRF_DISPMOVE = 0;
		PRF_FLAGS(ch).PRF_DISPGOLD = 0;
		PRF_FLAGS(ch).PRF_DISPEXP  = 0;
	} else {
		PRF_FLAGS(ch).PRF_DISPHP   = 0;
		PRF_FLAGS(ch).PRF_DISPMANA = 0;
		PRF_FLAGS(ch).PRF_DISPMOVE = 0;
		PRF_FLAGS(ch).PRF_DISPGOLD = 0;
		PRF_FLAGS(ch).PRF_DISPEXP  = 0;

		for (i = 0; i < strlen(argument); i++) {
			switch (LOWER(argument[i])) {
				case 'h':
					PRF_FLAGS(ch).PRF_DISPHP   = 1;
					break;
				case 'm':
					PRF_FLAGS(ch).PRF_DISPMANA = 1;
					break;
				case 'v':
					PRF_FLAGS(ch).PRF_DISPMOVE = 1;
					break;
				case 'g':
					PRF_FLAGS(ch).PRF_DISPGOLD  = 1;
					break;
				case 'x':
					PRF_FLAGS(ch).PRF_DISPEXP  = 1;
					break;
				default:
					send_to_char(ch, "Usage: prompt { { H | M | V | G | X } | all | none }\r\n");
					return;
			}
		}
	}

	send_to_char(ch, "%s", OK);
}



ACMD(do_gen_write)
{
	FILE *fl;
	char *tmp;
	const char *filename;
	struct stat fbuf;
	time_t ct;

	switch (subcmd) {
		case SCMD_BUG:
			filename = BUG_FILE;
			break;
		case SCMD_TYPO:
			filename = TYPO_FILE;
			break;
		case SCMD_IDEA:
			filename = IDEA_FILE;
			break;
		default:
			return;
	}

	ct = time(0);
	tmp = asctime(localtime(&ct));

	if (IS_NPC(ch)) {
		send_to_char(ch, "Monsters can't have ideas - Go away.\r\n");
		return;
	}

	skip_spaces(&argument);
	delete_doubledollar(argument);

	if (!*argument) {
		send_to_char(ch, "That must be a mistake...\r\n");
		return;
	}
	mudlog(CMP, LVL_IMMORT, FALSE, "%s %s: %s", GET_NAME(ch), CMD_NAME, argument);

	if (stat(filename, &fbuf) < 0) {
		perror("SYSERR: Can't stat() file");
		return;
	}
	if (fbuf.st_size >= max_filesize) {
		send_to_char(ch, "Sorry, the file is full right now.. try again later.\r\n");
		return;
	}
	if (!(fl = fopen(filename, "a"))) {
		perror("SYSERR: do_gen_write");
		send_to_char(ch, "Could not open the file.  Sorry.\r\n");
		return;
	}
	fprintf(fl, "%-8s (%6.6s) [%5d] %s\n", GET_NAME(ch), (tmp + 4),
			  GET_ROOM_VNUM(IN_ROOM(ch)), argument);
	fclose(fl);
	send_to_char(ch, "Okay.  Thanks!\r\n");
}



#define TOG_OFF 0
#define TOG_ON  1

ACMD(do_gen_tog)
{
	long result;

	const char *tog_messages[][2] = {
		{"You are now safe from summoning by other players.\r\n",
		 "You may now be summoned by other players.\r\n"},
		{"Nohassle disabled.\r\n",
		 "Nohassle enabled.\r\n"},
		{"Brief mode off.\r\n",
		 "Brief mode on.\r\n"},
		{"Compact mode off.\r\n",
		 "Compact mode on.\r\n"},
		{"You can now hear tells.\r\n",
		 "You are now deaf to tells.\r\n"},
		{"You can now hear auctions.\r\n",
		 "You are now deaf to auctions.\r\n"},
		{"You can now hear shouts.\r\n",
		 "You are now deaf to shouts.\r\n"},
		{"You can now hear gossip.\r\n",
		 "You are now deaf to gossip.\r\n"},
		{"You can now hear the congratulation messages.\r\n",
		 "You are now deaf to the congratulation messages.\r\n"},
		{"You can now hear the Wiz-channel.\r\n",
		 "You are now deaf to the Wiz-channel.\r\n"},
		{"You are no longer part of the Quest.\r\n",
		 "Okay, you are part of the Quest!\r\n"},
		{"You will no longer see the room flags.\r\n",
		 "You will now see the room flags.\r\n"},
		{"You will now have your communication repeated.\r\n",
		 "You will no longer have your communication repeated.\r\n"},
		{"HolyLight mode off.\r\n",
		 "HolyLight mode on.\r\n"},
		{"Nameserver_is_slow changed to NO; IP addresses will now be resolved.\r\n",
		 "Nameserver_is_slow changed to YES; sitenames will no longer be resolved.\r\n"},
		{"Autoexits disabled.\r\n",
		 "Autoexits enabled.\r\n"},
		{"Will no longer track through doors.\r\n",
		 "Will now track through doors.\r\n"},
		{"Autosplit disabled.\r\n",
		 "Autosplit enabled.\r\n"},
		{"Autoloot disabled.\r\n",
		 "Autoloot enabled.\r\n"},
		{"You will no longer Autoassist.\r\n",
		 "You will now Autoassist.\r\n"},
		{"Ident changed to NO; remote username lookups will not be attempted.\r\n",
		 "Ident changed to YES; remote usernames lookups will be attempted.\r\n"},
		{"Autosacrifice disabled\r\n",
		 "Autosacrifice enabled\r\n"},
		{"StatEdit disabled\r\n",
		 "StatEdit enabled\r\n"},
		{"Two handed limit disabled\r\n",
		 "Two handed limit enabled\r\n"},
		{"Ascii objects turned off.\r\n",
		 "Ascii objects turned on.\r\n"},
	};


	if (IS_NPC(ch))
		return;

	switch (subcmd) {
		case SCMD_NOSUMMON:
			result = PRF_TOG_CHK(ch, PRF_SUMMONABLE);
			break;
		case SCMD_NOHASSLE:
			result = PRF_TOG_CHK(ch, PRF_NOHASSLE);
			break;
		case SCMD_BRIEF:
			result = PRF_TOG_CHK(ch, PRF_BRIEF);
			break;
		case SCMD_COMPACT:
			result = PRF_TOG_CHK(ch, PRF_COMPACT);
			break;
		case SCMD_NOTELL:
			result = PRF_TOG_CHK(ch, PRF_NOTELL);
			break;
		case SCMD_NOAUCTION:
			result = PRF_TOG_CHK(ch, PRF_NOAUCT);
			break;
		case SCMD_DEAF:
			result = PRF_TOG_CHK(ch, PRF_DEAF);
			break;
		case SCMD_NOGOSSIP:
			result = PRF_TOG_CHK(ch, PRF_NOGOSS);
			break;
		case SCMD_NOGRATZ:
			result = PRF_TOG_CHK(ch, PRF_NOGRATZ);
			break;
		case SCMD_NOWIZ:
			result = PRF_TOG_CHK(ch, PRF_NOWIZ);
			break;
		case SCMD_QUEST:
			result = PRF_TOG_CHK(ch, PRF_QUEST);
			break;
		case SCMD_ROOMFLAGS:
			result = PRF_TOG_CHK(ch, PRF_ROOMFLAGS);
			break;
		case SCMD_NOREPEAT:
			result = PRF_TOG_CHK(ch, PRF_NOREPEAT);
			break;
		case SCMD_HOLYLIGHT:
			result = PRF_TOG_CHK(ch, PRF_HOLYLIGHT);
			break;
		case SCMD_SLOWNS:
			result = (nameserver_is_slow = !nameserver_is_slow);
			break;
		case SCMD_AUTOEXIT:
			result = PRF_TOG_CHK(ch, PRF_AUTOEXIT);
			break;
		case SCMD_TRACK:
			result = (track_through_doors = !track_through_doors);
			break;
		case SCMD_AUTOSPLIT:
			result = PRF_TOG_CHK(ch, PRF_AUTOSPLIT);
			break;
		case SCMD_AUTOLOOT:
			result = PRF_TOG_CHK(ch, PRF_AUTOLOOT);
			break;
		case SCMD_AUTOASSIST:
			result = PRF_TOG_CHK(ch, PRF_AUTOASSIST);
			break;
		case SCMD_IDENT:
			result = (ident = !ident);
			break;
		case SCMD_AUTOSAC:
			result = PRF_TOG_CHK(ch, PRF_AUTOSAC);
			break;
		case SCMD_STATEDIT:
			result = (allow_stat_edit = !allow_stat_edit);
			break;
		case SCMD_TWOHANDS:
			result = (two_hand_limit = !two_hand_limit);
			break;
		default:
			log("SYSERR: Unknown subcmd %d in do_gen_toggle.", subcmd);
			return;
	}

	if (result)
		send_to_char(ch, "%s", tog_messages[subcmd][TOG_ON]);
	else
		send_to_char(ch, "%s", tog_messages[subcmd][TOG_OFF]);

	return;
}


ACMD(do_afk)
{
	char roommsg[MAX_STRING_LENGTH], charmsg[MAX_STRING_LENGTH];

	if (IS_NPC(ch))
		return;

	if (PRF_TOG_CHK(ch, PRF_AFK)) {
		snprintf(charmsg, sizeof(charmsg), "You begin %s, ignoring everything "
					"else around you.", afk_aliases[GET_CLASS(ch)]);
		snprintf(roommsg, sizeof(roommsg), "$n begins %s, oblivious to the "
					"world around $mself.", afk_aliases[GET_CLASS(ch)]);
	} else {
		snprintf(charmsg, sizeof(charmsg), "You stop %s.", 
					afk_aliases[GET_CLASS(ch)]);
		snprintf(roommsg, sizeof(roommsg), "$n stops %s.", 
					afk_aliases[GET_CLASS(ch)]);
	}

	act(charmsg, TRUE, ch, 0, 0, TO_CHAR);
	act(roommsg, FALSE, ch, 0, 0, TO_ROOM);
}


ACMD(do_fly)
{
	int goal = GET_SKILL(ch, SKILL_FLY);
	int roll = rand_number(1, 101);
	int move = (GET_LEVEL(ch) < LVL_IMMORT ? 10 : 0);

	if (GET_POS(ch) == POS_FLYING) {
		send_to_char(ch, "You are already flying!\r\n");
		return;
	}

	if (!AFF_FLAGGED(ch, AFF_FLY) && !goal) {
		act("$n leaps into the air but quickly hits the ground.", 
			 TRUE, ch, 0, 0, TO_ROOM);
		act("You leap into the air but unfortunately hit the ground as you "
			 "fail to fly.", TRUE, ch, 0, 0, TO_CHAR);
	} else {
		if (AFF_FLAGGED(ch, AFF_FLY)) { /* We're using a spell... */
			act("$n leaps into the air and starts to fly.", 
				 TRUE, ch, 0, 0, TO_ROOM);
			act("You leap into the air and start to fly.", 
				 TRUE, ch, 0, 0, TO_CHAR);
			GET_POS(ch) = POS_FLYING;
		} else { /* ...we're using a skill */
			if (GET_MOVE(ch) < move) {
				send_to_char(ch, "You don't have the energy to fly!\r\n");
				return;
			}

			/* Harder to fly in bad weather */
			if (zone_table[IN_ZONE(ch)].sky == SKY_RAINING)
				goal -= 10;
			else if (zone_table[IN_ZONE(ch)].sky == SKY_LIGHTNING)
				goal -= 25;

			if (goal < 1)
				goal = 1;
			if (GET_LEVEL(ch) >= LVL_IMMORT)
				roll = 0;

			if (roll > goal) { /* Failure :O  */
				GET_MOVE(ch) -= move;
				act("$n leaps into the air but quickly hits the ground.", 
					 TRUE, ch, 0, 0, TO_ROOM);
				act("You leap into the air but unfortunately hit the ground "
					 "as you fail to fly.", TRUE, ch, 0, 0, TO_CHAR);
			} else { /* Success! */
				GET_MOVE(ch) -= move;
				act("$n leaps into the air and starts to fly.", 
					 TRUE, ch, 0, 0, TO_ROOM);
				act("You leap into the air and start to fly.", 
					 TRUE, ch, 0, 0, TO_CHAR);
				GET_POS(ch) = POS_FLYING;
			} /* if (roll > goal) { */
		} /* skill use attempt */
	} /* automatic failure */
}


ACMD(do_land)
{
	if (GET_POS(ch) != POS_FLYING)
		send_to_char(ch, "You are already on the ground.\r\n");
	else {
		send_to_char(ch, "You gently float to the ground and land.\r\n");
		act("$n gently floats to the ground and lands.", TRUE, ch, 0, 0, TO_ROOM);
		GET_POS(ch) = POS_STANDING;
	}
}


ACMD(do_search)
{
	int door, chance = 1;
	char buf[MAX_STRING_LENGTH];
	*buf = '\0';

	if (AFF_FLAGGED(ch, AFF_BLIND)) {
		send_to_char(ch, "You're blind, you can't see a damned thing!");
		return;
	} else if (ROOM_AFFECTED(IN_ROOM(ch), RAFF_FOG) && GET_LEVEL(ch) < LVL_IMMORT) {
		send_to_char(ch, "Your view is obscured by a mysterious fog.\r\n");
		return;
	}

	if (GET_LEVEL(ch) < LVL_IMMORT && GET_MOVE(ch) < 5) {
		send_to_char(ch, "You are too tired to search.\r\n");
		return;
	} 
	GET_MOVE(ch) -= (GET_LEVEL(ch) < LVL_IMMORT ? 5 : 0);
	
	send_to_char(ch, "\r\nYou begin to search the room...\r\n");

	for (door = 0; door < NUM_OF_DIRS; door++) {
		if (EXIT(ch, door) && EXIT(ch, door)->to_room != NOWHERE) {
			if (EXIT_FLAGGED(EXIT(ch, door), EX_HIDDEN)) {

				if (IS_ELF(ch))
					chance += 1;

				if (IS_THIEF(ch))
					chance += 1;

				if (GET_LEVEL(ch) > LVL_IMMORT)
					chance = 7;
				
				if (rand_number(1, 6) <= chance) {
					send_to_char(ch, "\r\nYou have found a secret door %s!\r\n", 
									 dirs[door]);
					EXIT(ch, door)->exit_info.EX_HIDDEN = 0;
				}
			}
		}
	}
}


/* This is currently an active command (you'll need to add it to the
 * command table.  I don't think it is appropriate as a general
 * command - you'll want to add it as a skill for some special class
 * or race, or else as a spell.  Still, I leave it here as reference.
 * You'll have to add an appropriate entry to the command table.
 * Note that this also means this hasn't been tested....  -ejg */
ACMD(do_drain)
{
	char arg[MAX_INPUT_LENGTH];
	int HIT = 0, MANA = 0, MOVE = 0;
	int level = GET_LEVEL(ch);
	struct obj_data *obj;

	one_argument(argument, arg);

	if (!*arg) {
		send_to_char(ch, "Drain what?\n\r");
		return;
	}
  
	if (!(obj = get_obj_in_list_vis(ch, arg, NULL, world[IN_ROOM(ch)].contents))) {
		send_to_char(ch, "That object is not here.\r\n");
		return;
	}

	act("$n bends down and touches $p which slowly dissapears.\r\n", 
		 FALSE, ch, obj, 0, TO_ROOM);
	act("You bend down and drain $p.\r\n", FALSE, ch, obj, 0, TO_ROOM);

	// Now let's add the energy to the players hp, mana, and move

	HIT  = rand_number(1, level * 2);
	MANA = rand_number(1, level);
	MOVE = rand_number(1, level / 2);

	GET_HIT(ch)  += HIT;
	GET_MANA(ch) += MANA;
	GET_MOVE(ch) += MOVE;

	extract_obj(obj);
}


/* Ditto as with the do_drain command - it's here, but not implemented. */
ACMD(do_transfer)
{
	int amount, loss, percent;
	struct char_data *victim = NULL;
	char arg1[MAX_INPUT_LENGTH];
	char arg2[MAX_INPUT_LENGTH];
	char arg3[MAX_INPUT_LENGTH];
	const char *usage = "Usage: transfer <hit/mana/move> <player> <amount>\r\n";

	argument = two_arguments(argument, arg1, arg2);
	one_argument(argument, arg3);

	if (IS_NPC(ch)) {
		send_to_char(ch, "Go Away!\r\n");
		return; 
	}

	if (!(victim = get_char_vis(ch, arg2, NULL, FIND_CHAR_ROOM))) {
		send_to_char(ch, "There is no such person.\r\n");
		return;
	}

	if (IS_NPC(victim)) {
		send_to_char(ch, "There is no such person.\r\n");
		return;
	}

	if (!*argument || (!*arg1 || (!*arg2 || (!*arg3)))) {
		send_to_char(ch, usage);
		return; 
	}

	if (victim == ch) {
		send_to_char(ch, "Now, that doesn't make sense, does it?\r\n");
		return;
	}

	if (!str_cmp("hit", arg1) || !str_cmp("hitpoints", arg1)) {
		if ((amount = atoi(arg3)) <= 0) {
			send_to_char(ch, "How many hitpoints do you wish to transfer?\r\n");
			return;
		}
		if (GET_HIT(ch) < amount) {
			send_to_char(ch, "You don't have that many hitpoints.\r\n");
			return; 
		}
		if (GET_HIT(victim) >= GET_MAX_HIT(victim)) {
			send_to_char(ch, "%s is already at maxhealth.\r\n", GET_NAME(victim));
			return; 
		}

		if (amount + GET_HIT(victim) > GET_MAX_HIT(victim))
			amount = GET_MAX_HIT(victim) - GET_HIT(victim);

		if (GET_WIS(ch) < rand_number(0, 40))
			loss = rand_number(0, amount);
		else
			loss = 0;

		if (FIGHTING(victim))
			loss = amount;

		GET_HIT(ch) -= amount;
		GET_HIT(victim) += amount - loss;

		percent = (loss*100) / amount;
		send_to_char(ch, "You transfer %d hitpoint%s to %s (%d%% loss).\r\n",  
						 amount, (amount == 1 ? "" : "s"), 
						 GET_NAME(victim), percent);

		send_to_char(victim, "%s transfers %d hitpoint%s to you (%d%% "
						 "loss).\r\n", GET_NAME(ch), amount, 
						 (amount == 1 ? "" : "s"), percent);

		if (GET_HIT(victim) >= GET_MAX_HIT(victim))
			send_to_char(ch, "%s is now at full health.\r\n", GET_NAME(victim));

	} else if (!str_cmp("mana", arg1) || !str_cmp("magicpoints", arg1)) {
		if ((amount = atoi(arg3)) <= 0) {
			send_to_char(ch, "How many magic points do you wish to transfer?\r\n");
			return;
		}
		if (GET_MANA(ch) < amount) {
			send_to_char(ch, "You don't have that many magic points.\r\n");
			return;
		}
		if (GET_MANA(victim) >= GET_MAX_MANA(victim)) {
			send_to_char(ch, "%s is already at maxmana.\r\n", GET_NAME(victim));
			return; 
		}

		if (amount + GET_MANA(victim) > GET_MAX_MANA(victim))
			amount = GET_MAX_MANA(victim) - GET_MANA(victim);

		if (GET_WIS(ch) < rand_number(0, 40))
			loss = rand_number(0, amount);
		else
			loss = 0;

		if (FIGHTING(victim))
			loss = amount;

		GET_MANA(ch) -= amount;
		GET_MANA(victim) += amount - loss;

		percent = (loss*100) / amount;

		send_to_char(ch, "You transfer %d magic point%s to %s (%d%% "
						 "loss).\r\n",  amount, (amount == 1 ? "" : "s"), 
						 GET_NAME(victim), percent);

		send_to_char(victim, "%s transfers %d magic point%s to you (%d%% "
						 "loss).\r\n", GET_NAME(ch), amount, 
						 (amount == 1 ? "" : "s"), percent);

		if (GET_MANA(victim) >= GET_MAX_MANA(victim))
			send_to_char(ch, "%s is now at full mana.\r\n", GET_NAME(victim));

	} else if (!str_cmp("move", arg1) || !str_cmp("movementpoints", arg1)) {
		if ((amount = atoi(arg3)) <= 0) {
			send_to_char(ch, "How many movement points do you wish to transfer?\r\n");
			return; 
		}
		if (GET_MOVE(ch) < amount) {
			send_to_char(ch, "You don't have that many movement points.\r\n");
			return; 
		}
		if (GET_MOVE(victim) >= GET_MAX_MOVE(victim)) {
			send_to_char(ch, "%s is already at maxmove.\r\n", GET_NAME(victim));
			return; 
		}

		if (amount + GET_MOVE(victim) > GET_MAX_MOVE(victim))
			amount = GET_MAX_MOVE(victim) - GET_MOVE(victim);

		if (GET_WIS(ch) < rand_number(0, 40))
			loss = rand_number(0, amount);
		else
			loss = 0;

		if (FIGHTING(victim))
			loss = amount;

		GET_MOVE(ch) -= amount;
		GET_MOVE(victim) += amount-loss;
		percent = (loss * 100) / amount;

		send_to_char(ch, "You transfer %d movement point%s to %s (%d%% "
						 "loss).\r\n",  amount, (amount == 1 ? "" : "s"), 
						 GET_NAME(victim), percent);

		send_to_char(victim, "%s transfers %d movement point%s to you (%d%% "
						 "loss).\r\n", GET_NAME(ch), amount, 
						 (amount == 1 ? "" : "s"), percent);

		if (GET_MOVE(victim) >= GET_MAX_MOVE(victim)) 
			send_to_char(ch, "%s is now at full move.\r\n", GET_NAME(victim));

	} else 
		send_to_char(ch, usage);
}


int in_water(struct char_data *ch)
{
	if (SECT(IN_ROOM(ch)) == SECT_WATER_SWIM)
		return (TRUE);
	
	if (SECT(IN_ROOM(ch)) == SECT_WATER_NOSWIM)
		return (TRUE);
	
	if (SECT(IN_ROOM(ch)) == SECT_UNDERWATER)
		return (TRUE);
	
	return (FALSE);
}


ACMD(do_castout)
{
	struct obj_data *pole;
	int fail;
	
	if (PLR_FLAGGED(ch, PLR_FISHING)) {
		send_to_char(ch, "You are already fishing!\r\n");
		return;
	}
	
	if (!(pole = GET_EQ(ch, WEAR_HOLD)) || (GET_OBJ_TYPE(pole) != ITEM_POLE)) {
		send_to_char(ch, "You need to be holding a fishing pole first.\r\n");
		return;
	}
	
	if (!in_water(ch)) {
		send_to_char(ch, "This is not a good place to fish, you'll want "
						 "to find a better spot.\r\n");
		return;
	}

	fail = rand_number(1, 10);
	if (GET_LEVEL(ch) >= LVL_IMMORT)
		fail = 10;
	
	if (fail <= 3) {
		send_to_char(ch, "You pull your arm back and try to cast out your line, "
						 "but it gets all tangled up.\r\nTry again.\r\n");
		act("$n pulls $s arm back, trying to cast $s fishing line out into the "
			 "water,\r\nbut ends up just a bit tangled.\r\n",
			 FALSE, ch, 0, 0, TO_ROOM);
		return;
	}

	/* Ok, now they've gone through the checks, now set them fishing */
	PLR_FLAGS(ch).PLR_FISHING = 1;
	send_to_char(ch, "You cast your line out into the water, hoping for "
					 "a bite.\r\n");
	act("$n casts $s line out into the water, hoping to catch some food.\r\n",
       FALSE, ch, 0, 0, TO_ROOM);
}


ACMD(do_reelin)
{
	int success, f_num, fish_num;
	struct obj_data *fish;

	if (!PLR_FLAGGED(ch, PLR_FISHING)) {
		send_to_char(ch, "You aren't even fishing!\r\n");
		return;
	}

	if (!PLR_FLAGGED(ch, PLR_FISH_ON)) {
		send_to_char(ch, "You reel in your line, but alas... nothing on the "
						 "end.\r\nBetter luck next time.\r\n");
		PLR_FLAGS(ch).PLR_FISHING = 0;
		act("$n reels $s line in, but with nothing on the end.\r\n",
			 FALSE, ch, 0, 0, TO_ROOM);
		return;
	}

	/* Ok, they are fishing and have a fish on */
	success = rand_number(1, 10);

	PLR_FLAGS(ch).PLR_FISHING = 0;
	PLR_FLAGS(ch).PLR_FISH_ON = 0;

	if (GET_LEVEL(ch) >= LVL_IMMORT)
		success = 10;
	
	if (success <= 6) {
		send_to_char(ch, "You reel in your line, putting up a good fight, but "
						 "you lose him!\r\nTry again?\r\n");
		act("$n reels $s line in, fighting with whatever is on the end, but  "
			 "loses the catch.\r\n", FALSE, ch, 0, 0, TO_ROOM);
		return;
	}

	/* We used object vnums 10030-10050 for our fish that people could
	 * catch. The below numbers reflect that use. If you wish to change
	 * the vnums of the fish, just change the numbers below. You can
	 * see that we seperated the type of fish by freshwater and salt
	 * water.
	 */
	if (SECT(IN_ROOM(ch)) == SECT_WATER_SWIM) {
		fish_num = rand_number(930, 939);
		f_num = real_object(fish_num);
		fish = read_object(f_num, REAL);
		send_to_char(ch, "You reel in %s! Nice catch!\r\n",
						 fish->short_description);
		act("Wow! $n reels in a helluva catch! Looks like $p!\r\n",
			 FALSE, ch, fish, 0, TO_ROOM);
		obj_to_char(fish, ch);
		return;
	} else {
		if (SECT(IN_ROOM(ch)) == SECT_WATER_NOSWIM ||
			 SECT(IN_ROOM(ch)) == SECT_UNDERWATER) {
			fish_num = rand_number(940, 950);
			f_num = real_object(fish_num);
			fish = read_object(f_num, REAL);
			send_to_char(ch, "You reel in %s! Nice catch!\r\n", 
							 fish->short_description);
			obj_to_char(fish, ch);
			return;
		} else
			send_to_char(ch, "You should never see this message, please "
							 "report it.\r\n");
	}
}


/********************************************************************
 ** Written by Christopher M. Ryan 7/26/96
 ** do_bury & do_dig()
 ********************************************************************/
/**********************************************************************
 ** Please note that the way the code is set up all messages that are
 ** to be displayed to the character use send message so you can't add
 ** any variable types to your messages that go to the character.
 **********************************************************************/
const char* msgs[SECT_MAX+1][2] = {
	{"You start to break some floor boards when you dig.\r\n",
	 "$n starts to break some floor boards as $e starts digging.\r\n" },
	{"You wonder if this is a good place after all, with all the gravel.\r\n",
	 "$n breaks a sweat digging up all the gravel here.\r\n" },
	{"You make a nice hole while digging up a lot of dirt.\r\n",
	 "$n digs a hole and goes about $s business.\r\n" },
	{"You seem to be hitting alot of roots when you dig.\r\n",
	 "$n looks like $e is trying to dig up a tree!\r\n" },
	{"You dig up more clay than dirt here.\r\n",
	 "$n seems to be digging up alot of clay.\r\n" },
	{"You start to chip away at the rock here.\r\n",
	 "$n bangs away at the side of the mountain.\r\n" },
	{"You can't dig in the water!\r\n",
	 NULL },
	{"You can't dig in the water!\r\n",
	 NULL },
	{"You can't dig up air!\r\n",
	 NULL },
	{"You can't dig in the water!\r\n",
	 NULL },
	{"You easily dig a hole in the peat moss.\r\n",
	 "$n effortlessly digs a big hole in the peat moss.\r\n" },
	{"You seem to be hitting a lot of roots as you dig.\r\n"
	 "$n looks like $e is trying to dig up a tree!\r\n"  },
	{"You burn your hands trying to dig into the lava.  OUCH!\r\n",
	 NULL },
	{"Your hands get frostbitten as you chip into the ice and snow.\r\n",
	 "$n suffers frostbite as $e digs into the snow and ice.\r\n" },
	{"You sweat brutually as you dig into the sand.\r\n",
	 "$n sweats like a pig as $e digs a hole in the sand.\r\n" },

	/* always keep this as the last message */
	{ "If you see this please tell a god.\r\n", NULL }
};


int dig_ok(struct char_data *ch)
{
	if (world[IN_ROOM(ch)].sector_type == SECT_WATER_SWIM)
		return (FALSE);
	
	if (world[IN_ROOM(ch)].sector_type == SECT_WATER_NOSWIM)
		return (FALSE);
	
	if (world[IN_ROOM(ch)].sector_type == SECT_UNDERWATER)
		return (FALSE);
	
	if (world[IN_ROOM(ch)].sector_type == SECT_FLYING)
		return (FALSE);
	
	if (world[IN_ROOM(ch)].sector_type == SECT_LAVA)
		return (FALSE);
	
	return (TRUE);
}


ACMD(do_bury)
{
	struct obj_data *obj;
	char arg[MAX_INPUT_LENGTH], buf[MAX_INPUT_LENGTH];
	
	half_chop(argument, arg, buf);
	
	if (!*arg) {
		send_to_char(ch, "What do you want to %s?\r\n", CMD_NAME);
		return;
	}
	
	if (!(obj = get_obj_in_list_vis(ch, arg, NULL, ch->carrying))) {
		send_to_char(ch, "You don't have %s %s.\r\n", AN(arg), arg);
		return;
	}
	
	if (subcmd == SCMD_DELAY && GET_LEVEL(ch) < LVL_IMMORT) {
		send_to_char(ch, "You begin digging...\r\n");
		act("$n begins digging...", TRUE, ch, 0, 0, TO_ROOM);
		add_simple_event(ch, cmd, 0, 10 RL_SEC);
		return;
	}

	if (msgs[world[IN_ROOM(ch)].sector_type][0])
		send_to_char(ch, msgs[world[IN_ROOM(ch)].sector_type][0]);

	if (msgs[world[IN_ROOM(ch)].sector_type][1])
		act(msgs[world[IN_ROOM(ch)].sector_type][1], 
			 TRUE, ch, NULL, NULL,TO_ROOM);

	if (dig_ok(ch)) {
		/* set a wait state */
		WAIT_STATE(ch, 10 RL_SEC);

		act("You bury $a $o here.\r\n", TRUE, ch, obj, NULL, TO_CHAR);
		act("$n buries $a $o here.\r\n", TRUE, ch, obj, NULL, TO_ROOM);

		obj_from_char(obj);
		GET_OBJ_EXTRA(obj).ITEM_BURIED = 1;
		obj_to_room(obj, IN_ROOM(ch));
	}
}


ACMD(do_dig)
{
	struct obj_data *obj;
	int found_item = 0;

	if (subcmd == SCMD_DELAY && GET_LEVEL(ch) < LVL_IMMORT) {
		send_to_char(ch, "You begin digging...\r\n");
		act("$n begins digging...", TRUE, ch, 0, 0, TO_ROOM);
		add_simple_event(ch, cmd, 0, 10);
		return;
	}

	if (msgs[world[IN_ROOM(ch)].sector_type][0])
		send_to_char(ch, msgs[world[IN_ROOM(ch)].sector_type][0]);

	if (msgs[world[IN_ROOM(ch)].sector_type][1])
		act(msgs[world[IN_ROOM(ch)].sector_type][1], 
			 TRUE, ch, NULL, NULL,TO_ROOM);

	if (dig_ok(ch)) {
		/* set a wait state */
		WAIT_STATE(ch, 10);

		/* search for an object in the room that has a ITEM_BURIED flag */
		obj = world[IN_ROOM(ch)].contents;

		while (obj) {
			if (IS_BURIED(obj)) {
				/* Remove the buried bit so the player can see it. */
				GET_OBJ_EXTRA(obj).ITEM_BURIED = 0;

				if (CAN_SEE_OBJ(ch, obj)) {
					found_item = 1;     /* player found something */
					act("You found $a $o buried here.\r\n", 
						 TRUE, ch, obj, NULL, TO_CHAR);
					act("$n has found $a $o buried here.\r\n",
						 TRUE, ch, obj, NULL,TO_ROOM);
					obj_from_room(obj);
					obj_to_char(obj, ch);
				} else {
					/* add the bit back becuase the player can't unbury what
					 * what he can't find...  */
					GET_OBJ_EXTRA(obj).ITEM_BURIED = 1;
				}
			}
			/* go to the next obj */
			obj = obj->next;
		}

		if (!found_item)   /* if the player didn't find anything */
			send_to_char(ch, "Sorry! You didn't find anything.\r\n");
	}
}


ACMD(do_sac)
{
   struct obj_data *obj;
	char arg[MAX_INPUT_LENGTH];

   one_argument(argument, arg);

   if (!*arg) {
		if (subcmd == SCMD_SACRIFICE)
			send_to_char(ch, "Sacrifice what?\r\n");
		else if (subcmd == SCMD_LASTRITES)
			send_to_char(ch, "Perform last rites on what?\r\n");
		else if (subcmd == SCMD_BURIAL)
			send_to_char(ch, "Bury what?\r\n");
			
		return;
   }

   if (!(obj = get_obj_in_list_vis(ch, arg, NULL, world[ch->in_room].contents))) {
		send_to_char(ch, "That ain't here!\r\n");
		return;
   }

   if (!IS_CORPSE(obj)) {
		if (subcmd == SCMD_SACRIFICE)
			send_to_char(ch, "You can't sacrifice that!\r\n");
		else if (subcmd == SCMD_LASTRITES)
			send_to_char(ch, "You can't perform last rites on that!\r\n");
		else if (subcmd == SCMD_BURIAL)
			send_to_char(ch, "You can't do a burial on that!\r\n");
	}

	if (subcmd == SCMD_SACRIFICE) {
		act("$n sacrifices $p.", FALSE, ch, obj, 0, TO_ROOM);
		act("You sacrifice $p to your god.", FALSE, ch, obj, 0, TO_CHAR);
		/* Sacrificing is evil... */
		GET_ALIGNMENT(ch) -= 1;
	} else if (subcmd == SCMD_LASTRITES) {
		act("$n performs last rites on $p.", FALSE, ch, obj, 0, TO_ROOM);
		act("You perform last rites on $p.", FALSE, ch, obj, 0, TO_CHAR);
		/* Last rites are good... */
		GET_ALIGNMENT(ch) += 1;
	} else if (subcmd == SCMD_BURIAL) {
		act("$n does a burial for $p.", FALSE, ch, obj, 0, TO_ROOM);
		act("You do a burial for $p.", FALSE, ch, obj, 0, TO_CHAR);
		/* And burials are simply neutral. */
		if (GET_ALIGNMENT(ch) < -0)
			GET_ALIGNMENT(ch) += 1;
		else if (GET_ALIGNMENT(ch) > 0)
			GET_ALIGNMENT(ch) -= 1;
	}
	
	NUM_SACS(ch) += 1;
   extract_obj(obj);
}


ACMD(do_bandage)
{
	struct obj_data *obj;
	struct char_data *vict;
	char arg[MAX_INPUT_LENGTH];
	int goal = GET_SKILL(ch, SKILL_BANDAGE);
	int roll = rand_number(1, 101);

	obj = get_obj_in_list_vis(ch, "bandage", NULL, ch->carrying);
	
	one_argument(argument, arg);

	if (!*arg)
		vict = ch;
	else if (!(vict = get_char_vis(ch, arg, NULL, FIND_CHAR_ROOM)))
		send_to_char(ch, "They don't seem to be here.\r\n");

	if (GET_HIT(vict) >= GET_MAX_HIT(vict)) {
		send_to_char(ch, "%s are already healthy!\r\n", 
						 (vict == ch ? "You" : "They"));
		return; 
	}

	if (GET_LEVEL(ch) < LVL_IMMORT)
		roll = 0;
	
	if (roll > goal) {
		send_to_char(ch, "You fail.\r\n");

		if (obj)
			extract_obj(obj);
	} else {
		if (obj)
			GET_HIT(ch) += goal / 4;
		else
			GET_HIT(ch) += goal / 10;

		if (!obj) {
			send_to_char(ch, "You bandage %s.\r\n",
							 (ch == vict ? "yourself" : GET_NAME(vict)));
			if (ch == vict)
				act("$n bandages $mself.", FALSE, ch, 0, 0, TO_ROOM);
			else
				act("$n bandages $N.", FALSE, ch, 0, vict, TO_ROOM);
		} else {
			if (ch == vict) {
				act("You bandage yourself with $p.", FALSE, ch, obj, 0, TO_CHAR);
				act("$n bandages $mself with $p.", FALSE, ch, obj, 0, TO_ROOM);
			} else {
				act("You bandage $N with $p.", FALSE, ch, obj, vict, TO_CHAR);
				act("$n bandages $N with $p.", FALSE, ch, obj, vict, TO_ROOM);
			}
			extract_obj(obj); 
		}

		if (GET_HIT(vict) >= GET_MAX_HIT(vict)) {
			send_to_char(vict, "You are now at full health.\r\n");
			GET_HIT(vict) = GET_MAX_HIT(vict); 
		}

		if (GET_LEVEL(ch) < LVL_IMMORT)
			WAIT_STATE(ch, PULSE_VIOLENCE * 2);
	}
}


/* This skill is not currently assigned to any class.  The obvious
 * application would be to let vampires use it, or maybe druids.  -ejg */
#define WHISTLE_WOLF       70
#define WHISTLE_TIMBER     71
#define WHISTLE_DIRE       72

ACMD(do_whistle) {
	struct obj_data *obj;
	struct char_data *pet;
	struct follow_type *f;
	int num_followers = 0;
	int goal = GET_SKILL(ch, SKILL_WHISTLE);
	int roll = rand_number(1, 101);
	int penalty = 0, chance;

	if (IS_NPC(ch) || !goal) {
		send_to_char(ch, "You whistle a happy tune.\r\n");
		return;
	}

	if (!OUTSIDE(ch)) {
		send_to_char(ch, "No wolves will answer your call indoors!\r\n");
		return;
	}

	if (GET_MOVE(ch) <= 30 && GET_LEVEL(ch) < LVL_IMMORT) {
		send_to_char(ch, "You are too tired.\r\n");
		return;
	}

	obj = get_obj_in_list_vis(ch, "whistle", NULL, ch->carrying);

	if (!obj) {
      send_to_char(ch, "But you need something to whistle with!\r\n"); 
      return;
	}

	if (GET_LEVEL(ch) < LVL_IMMORT)
		GET_MOVE(ch) -= 30;

	for (f = ch->followers, num_followers = 0; f; f = f->next) {
		if (IN_ROOM(ch) == IN_ROOM(f->follower)) {
			num_followers += 1;

			if (IS_MOB(f->follower))
				penalty += 5;
			else
				penalty += 2;
		}
	}
	
	if (num_followers >= cha_app[(int)GET_CHA(ch)].num_followers) {
		send_to_char(ch, "You can not attract any more followers now.\r\n");
		return;
	}

	/* Better luck at night! */
	if (is_night()) 
		goal += 25;

	/* Worse luck in bad weather */
	if (zone_table[IN_ZONE(ch)].sky == SKY_RAINING) 
		goal -= 10;
	if (zone_table[IN_ZONE(ch)].sky == SKY_LIGHTNING)
		goal -= 25;

	/* Adjust by sector type */
	switch (SECT(IN_ROOM(ch))) {
		case SECT_WATER_SWIM:
		case SECT_WATER_NOSWIM:
		case SECT_UNDERWATER:
		case SECT_FLYING:
		case SECT_INSIDE:
		case SECT_CITY: /* Impossible! */
			goal = -100;
			break;
			
		case SECT_LAVA:
		case SECT_DESERT:
			goal -= 50;
			break;
			
		case SECT_MOUNTAIN:
		case SECT_JUNGLE:
			goal -= 10;
			break;
			
		default: 
		case SECT_FIELD: 
		case SECT_HILLS: /* No modifier */
			break;
			
		case SECT_SWAMP:
		case SECT_TUNDRA:
			goal += 10;
			break;
			
		case SECT_FOREST:
			goal += 25;
			break;
	}

	if (goal > 0)
		chance = rand_number(0, goal + GET_LEVEL(ch));
	else
		chance = 0;

	if (GET_LEVEL(ch) >= LVL_IMMORT)
		chance = 100;

	if (chance < 50) {
		act("Your whistle fails.  No wolves were attracted.", 
			 FALSE, ch, 0, 0, TO_CHAR);
		act("$n whistles a hauting melody.",
			 FALSE, ch, 0, 0, TO_NOTVICT);
		return;
	} else if (chance  < 100)
		pet = read_mobile(WHISTLE_TIMBER, VIRTUAL);
	else if (chance  < 120)
		pet = read_mobile(WHISTLE_DIRE, VIRTUAL);
	else
		pet = read_mobile(WHISTLE_WOLF, VIRTUAL);

	act("Your whistle is answered by $N.", FALSE, ch, 0, pet, TO_CHAR);
	act("$n whistles is answered by $N.", FALSE, ch, 0, pet, TO_ROOM);

	IS_CARRYING_W(pet) = 1000;
	IS_CARRYING_N(pet) = 100;
	AFF_FLAGS(pet).AFF_CHARM = 1;

	char_to_room(pet, IN_ROOM(ch));
	add_follower(pet, ch);

	if (rand_number(1, 10) >= 5) {
		if (chance < 70)
			return;

		if (chance  < 100)
			pet = read_mobile(WHISTLE_TIMBER, VIRTUAL);
		else if (chance  < 120)
			pet = read_mobile(WHISTLE_DIRE, VIRTUAL);
		else
			pet = read_mobile(WHISTLE_WOLF, VIRTUAL);

		act("Your whistle is answered by $N.", FALSE, ch, 0, pet, TO_CHAR);
		act("$ns whistle is answered by $N.", FALSE, ch, 0, pet, TO_ROOM);
		char_to_room(pet, IN_ROOM(ch));
		add_follower(pet, ch);
	}

	if (rand_number(1, 10) >= 8) {
		if (chance < 90) 
			return;

		if (chance  < 100)
			pet = read_mobile(WHISTLE_WOLF, VIRTUAL);
		else if (chance  < 120)
			pet = read_mobile(WHISTLE_DIRE, VIRTUAL);
		if (rand_number(1, 100) <= 50)
			pet = read_mobile(WHISTLE_TIMBER, VIRTUAL);

		act("Your whistle is answered by $N.", FALSE, ch, 0, pet, TO_CHAR);
		act("$ns whistle is answered by $N.", FALSE, ch, 0, pet, TO_ROOM);
		char_to_room(pet, IN_ROOM(ch));
		add_follower(pet, ch);
	} 
}


