/* Wiz commands for affecting and getting info about zones */

#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"

#include "comm.h"
#include "constants.h"
#include "db.h"
#include "interpreter.h"


/* extern functions */
void reset_zone(zone_rnum zone);


/* local functions */
size_t print_zone_to_buf(char *bufptr, size_t left, zone_rnum zone);
void show_connections(struct char_data *ch, char *arg);

ACMD(do_areas);
ACMD(do_roomlink);
ACMD(do_zconnect);
ACMD(do_zreset);


ACMD(do_zreset)
{
	char arg[MAX_INPUT_LENGTH];
	zone_rnum i;
	zone_vnum j;

	one_argument(argument, arg);
	if (!*arg) {
		send_to_char(ch, "You must specify a zone.\r\n");
		return;
	}
	if (*arg == '*') {
		for (i = 0; i <= top_of_zone_table; i++)
			reset_zone(i);
		send_to_char(ch, "Reset world.\r\n");
		mudlog(NRM, MAX(LVL_GRGOD, GET_INVIS_LEV(ch)), TRUE, "(GC) %s reset entire world.", GET_NAME(ch));
		return;
	} else if (*arg == '.')
		i = world[IN_ROOM(ch)].zone;
	else {
		j = atoi(arg);
		for (i = 0; i <= top_of_zone_table; i++)
			if (zone_table[i].number == j)
				break;
	}
	if (i <= top_of_zone_table) {
		reset_zone(i);
		send_to_char(ch, "Reset zone %d (#%d): %s.\r\n", i, zone_table[i].number, zone_table[i].name);
		mudlog(NRM, MAX(LVL_GRGOD, GET_INVIS_LEV(ch)), TRUE, "(GC) %s reset zone %d (%s)", GET_NAME(ch), i, zone_table[i].name);
	} else
		send_to_char(ch, "Invalid zone number.\r\n");
}


/* single zone printing fn used by "show zone" so it's not repeated in the
   code 3 times ... -je, 4/6/93 */

/* FIXME: overflow possible */
size_t print_zone_to_buf(char *bufptr, size_t left, zone_rnum zone)
{
	char buf1[MAX_STRING_LENGTH];
	sprintfield((char*)&zone_table[zone].zflags, zone_bits, buf1, sizeof(buf1));

	return snprintf(bufptr, left,
						 "%3d %-30.30s (levels %d to %d)\r\n"
						 "Flags: %s\r\n"
						 "Age: %3d; Reset: %3d (%d);\r\n"
						 "Range: %5d-%5d; Status: %s\r\n"
						 "Weather: Pressure: %d, Change: %d, Sky: %s\r\n\r\n",
						 zone_table[zone].number, zone_table[zone].name,
						 zone_table[zone].lvl1, zone_table[zone].lvl2,

						 buf1,

						 zone_table[zone].age, zone_table[zone].lifespan,
						 zone_table[zone].reset_mode,

						 zone_table[zone].bot, zone_table[zone].top, 
						 zone_table[zone].status_mode ? "Open" : "Closed",

						 zone_table[zone].pressure, zone_table[zone].change,
						 sky_look[zone_table[zone].sky]);
}

/*
  From: Brian Guilbault <guil9964@nova.gmi.edu>
  Subject: Checking Linked Rooms

  This command should work now. It's still terribly inefficient, but it
  works.  Don't forget to add your calls in interpreter.c and you are all
  set.
*/
ACMD(do_roomlink)          /* Shows all exits to a given room */
{
	char buf[MAX_INPUT_LENGTH], buf2[MAX_INPUT_LENGTH];
	size_t nlen = 0, len = 0;
	int door = 0, i = 0;
	room_vnum room_num = 0;

	one_argument(argument, buf);
	room_num = real_room(atoi(buf));

	if (room_num == NOWHERE) {
		send_to_char(ch, "There is no room with that number.\r\n");
		return;
	} else {
		for (i = 0; i <= top_of_world; i++) {
			for (door = 0; door < NUM_OF_DIRS; door++) {
				if (world[i].dir_option[door] &&
					 world[i].dir_option[door]->to_room == room_num) {

					nlen = snprintf(buf2 + len, sizeof(buf2) - len, 
										 "Exit %s from room %d.\r\n", 
										 dirs[door], world[i].number);

					if (len + nlen >= sizeof(buf2) || nlen < 0)
						break;
					len += nlen;
				}
			}
		}
		if (ch->desc)
			page_string(ch->desc, buf2, TRUE);
	}
}


/*
  From: Angus Mezick <angus@EDGIL.CCMAIL.COMPUSERVE.COM>
  Subject: show connection

  I don't know if i posted this before but here is a nice little util i came
  up for finding what zones connected to a ceretain zone.  I leave it up to
  you to do the show connection part of the command.
*/
void show_connections(struct char_data *ch, char *arg)
{
   zone_vnum zone_num;
   room_vnum start_room;
   char buf[MAX_STRING_LENGTH];
   int j, i, k;
	size_t nlen = 0, len = 0;

   if (!*arg) {
      send_to_char(ch, "USAGE: show connections .\r\n");
      send_to_char(ch, "USAGE: show connections <zone_num>\r\n");
      return;
	} else if (*arg == '.') {
      zone_num = world[IN_ROOM(ch)].zone;
	} else {
      j = atoi(arg);
      for (zone_num = 0; zone_num <= top_of_zone_table; zone_num++)
			if (zone_table[zone_num].number == j)
				break;
	}

   if (zone_num >= 0 && zone_num <= top_of_zone_table) {
      start_room = zone_table[zone_num].number * 100;
      len += snprintf(buf, sizeof(buf),
					"Connections from %-30.30s\r\n"
					"----------------------------------------------------------------------\r\n",
							zone_table[zone_num].name);

      for (i = 0, k = 0; i <= top_of_world; i++) {
			for (j = 0; j < NUM_OF_DIRS; j++) {
				if (world[i].zone == zone_num  &&
					 world[i].dir_option[j]     &&
					 world[i].dir_option[j]->to_room != -1 &&
					 world[world[i].dir_option[j]->to_room].zone != zone_num)
				{
					nlen = snprintf(buf + len, sizeof(buf) - len,
								"%3d: [%5d] %-23.23s -(%-5.5s)-> [%5d] %-23.23s\r\n",
								++k, world[i].number, world[i].name,dirs[j],
								world[world[i].dir_option[j]->to_room].number,
								world[world[i].dir_option[j]->to_room].name);

					if (len + nlen >= sizeof(buf) || nlen < 0)
						break;
					len += nlen;
				}
			}
		}
      len += snprintf(buf + len, sizeof(buf) - len,
					"\r\nConnections to %-30.30s\r\n"
					"----------------------------------------------------------------------\r\n",
				  zone_table[zone_num].name);

      for (i = 0, k = 0; i <= top_of_world; i++) {
			for (j = 0; j < NUM_OF_DIRS; j++) {
				if (world[i].zone != zone_num  &&
					 world[i].dir_option[j]     &&
					 world[i].dir_option[j]->to_room != -1 &&
					 world[world[i].dir_option[j]->to_room].zone == zone_num) {
					nlen = snprintf(buf + len, sizeof(buf) - len,
							  "%3d: [%5d] %-23.23s -(%-5.5s)-> [%5d] %-23.23s\r\n",
							  ++k, world[i].number, world[i].name,dirs[j],
							  world[world[i].dir_option[j]->to_room].number,
							  world[world[i].dir_option[j]->to_room].name);

					if (len + nlen >= sizeof(buf) || nlen < 0)
						break;
					len += nlen;
				}
			}
		}

      if (ch->desc)
			page_string(ch->desc, buf, TRUE);
	}
}


ACMD(do_zconnect)
{
	show_connections(ch, argument);
}



int display_zone_to_buf(char *bufptr, int zone, int level, size_t buf_size, int all_data)
{
	char buf1[MAX_STRING_LENGTH];
	size_t nlen = 0, len = 0;
	sprintfield((char*)&zone_table[zone].zflags, zone_bits, buf1, sizeof(buf1));

   nlen = snprintf(bufptr + len, buf_size - len,
						 " %-30.30s (%2d to %2d)\t%s\r\n", 
						 zone_table[zone].name, zone_table[zone].lvl1,
						 zone_table[zone].lvl2,
						 zone_table[zone].status_mode ? "Open" : "Closed");
	if (len + nlen >= buf_size || nlen < 0)
		return (len += nlen);
	len += nlen;
	
	if (level >= LVL_IMMORT && all_data)
	{
		nlen = snprintf(bufptr + len, buf_size - len, "\tZONE FLAGS: %s\r\n", buf1);
		if (len + nlen >= buf_size || nlen < 0)
			return (len += nlen);
		len += nlen;
	}
	
	if (level >= LVL_GOD && all_data)
	{
		nlen = snprintf(bufptr + len, buf_size - len,
					"\tVNUM:     vnum: [%d], bottom room: [%d], top room: [%d]\r\n",
					zone_table[zone].number, 
					zone_table[zone].bot, zone_table[zone].top);
		if (len + nlen >= buf_size || nlen < 0)
			return (len += nlen);
		len += nlen;
	}
	
	if (level >= LVL_GRGOD && all_data)
	{
		nlen = snprintf(bufptr + len, buf_size - len,
					"\tRESET:    lifespan: [%d], age: [%d], reset mode: [%d]\r\n",
					zone_table[zone].lifespan, zone_table[zone].age,
					zone_table[zone].reset_mode);
		if (len + nlen >= buf_size || nlen < 0)
			return (len += nlen);
		len += nlen;
	}
	
	if (level >= LVL_IMPL && all_data)
	{
		nlen = snprintf(bufptr + len, buf_size - len,
					"\tWEATHER:  Pressure: [%d], Change: [%d], Sky: %s [%d]\r\n",
					zone_table[zone].pressure, zone_table[zone].change,
					sky_look[zone_table[zone].sky], zone_table[zone].sky);
		if (len + nlen >= buf_size || nlen < 0)
			return (len += nlen);
		len += nlen;
	}

	return (len);
}

 
ACMD(do_areas)
{
   zone_rnum i;
	int all_data = FALSE;
	size_t nlen = 0, len = 0;
	char buf[MAX_STRING_LENGTH];
	char arg[MAX_INPUT_LENGTH];
   *buf = '\0';
 
	if (GET_LEVEL(ch) >= LVL_IMMORT) {
		one_argument(argument, arg);
		if (!stricmp(arg, "details"))
			all_data = TRUE;
	} 
 
	for (i = 0; i <= top_of_zone_table; i++) {
		nlen = display_zone_to_buf(buf + len, i, GET_LEVEL(ch), 
											sizeof(buf) - len, all_data);

		if (len + nlen >= sizeof(buf) || nlen < 0)
			break;
		len += nlen;
	}

	page_string(ch->desc, buf, TRUE);
}
 

#define ZCMD(zone,command) zone_table[zone].cmd[command]

/* Check the previous command in the list to determine whether
 * it was used in the same room or not 
 */
int check_same_room(int zone, int subcmd, room_rnum rnum)
{
	if (subcmd <= 0) /* Looks like we're already at the bottom, tough luck */
   	return (FALSE);

	if (ZCMD(zone, (subcmd-1)).if_flag) {  /* Need to go back another command */
   	if (check_same_room(zone, (subcmd - 1), rnum) != TRUE)
      	return (FALSE); /* Checking prev command found no match to the rnum */
      else
      	return (TRUE);  /* Found it! */
	} else {
		switch (ZCMD(zone, (subcmd-1)).command){
			case 'M':   /* Mobile */
			case 'O':	/* Object */
	      	if (ZCMD(zone, (subcmd-1)).arg3 == rnum)
            	return (TRUE);

	      	break;

			case 'R':	/* Room */
			case 'D':	/* Door */
	      	if (ZCMD(zone, (subcmd-1)).arg1 == rnum)
            	return (TRUE);

      		break;

			default:
				break;
	   }
	}
	return (FALSE);
}

/* Called from vstat - show the zone reset information for
 * the specified room
 */
void do_zstat_room(struct char_data *ch, zone_rnum rnum)
{
	room_vnum rvnum = GET_ROOM_VNUM(rnum);
	int zone = world[rnum].zone;
	int subcmd = 0, counter = 0, found;
	size_t nlen = 0, len = 0;
	char output[MAX_STRING_LENGTH];
	char buf1[MAX_STRING_LENGTH], buf2[MAX_STRING_LENGTH];

	sprintfield((char*)&zone_table[zone].zflags, zone_bits, buf1, sizeof(buf1));

	send_to_char(ch,
					 "Zone number : %d [room: %4d]\r\n"
					 "Zone name   : %s\r\n"
					 "Zone Flags  : %s\r\n",
					 zone_table[zone].number, rvnum, 
					 zone_table[zone].name ? zone_table[zone].name : "<NONE!>",
					 buf1);

	if (!(ZCMD(zone, subcmd).command)){
		send_to_char(ch, "No zone loading information.\r\n");
		return;
	}

	snprintf(output, sizeof(output), 
				"--------------------------------------------------\r\n");
	while ((ZCMD(zone, subcmd).command) && (ZCMD(zone, subcmd).command != 'S')) {
		found = FALSE;
		/* Translate what the command means. */

		/* If this is a "then do this" command, we need to check that
		 * the previous command was issued in this room */
		if (ZCMD(zone, subcmd).if_flag == TRUE){
			if (check_same_room(zone, subcmd, rnum) == FALSE){
				subcmd++;
				continue; /* not this room */
			}
		}

		switch (ZCMD(zone, subcmd).command) {
			case 'M':
				if (ZCMD(zone, subcmd).arg3 != rnum)
					break; /* not this room */
				nlen = snprintf(buf2 + len, sizeof(buf2) - len,
							"%sLoad %s [%d], Max : %d\r\n",
							ZCMD(zone, subcmd).if_flag ? " then " : "-",
							mob_proto[ZCMD(zone, subcmd).arg1].player.short_descr,
							mob_index[ZCMD(zone, subcmd).arg1].vnum, 
							ZCMD(zone, subcmd).arg2);
				found = TRUE;
				break;
			case 'G':
				nlen = snprintf(buf2 + len, sizeof(buf2) - len,
							"%sGive it %s [%d], Max : %d\r\n",
							ZCMD(zone, subcmd).if_flag ? " then " : "-",
							obj_proto[ZCMD(zone, subcmd).arg1].short_description,
							obj_index[ZCMD(zone, subcmd).arg1].vnum,
							ZCMD(zone, subcmd).arg2);
				found = TRUE;
				break;
			case 'O':
				if (ZCMD(zone, subcmd).arg3 != rnum)
					break; /* not this room */
				nlen = snprintf(buf2 + len, sizeof(buf2) - len,
							"%sLoad %s [%d], Max : %d\r\n",
							ZCMD(zone, subcmd).if_flag ? " then " : "-",
							obj_proto[ZCMD(zone, subcmd).arg1].short_description,
							obj_index[ZCMD(zone, subcmd).arg1].vnum,
							ZCMD(zone, subcmd).arg2);
				found = TRUE;
				break;
			case 'E':
				nlen = snprintf(buf2 + len, sizeof(buf2) - len,
							"%sEquip with %s [%d], %s, Max : %d\r\n",
							ZCMD(zone, subcmd).if_flag ? " then " : "-",
							obj_proto[ZCMD(zone, subcmd).arg1].short_description,
							obj_index[ZCMD(zone, subcmd).arg1].vnum,
							equipment_types[ZCMD(zone, subcmd).arg3],
							ZCMD(zone, subcmd).arg2);
				found = TRUE;
				break;
			case 'P':
				nlen = snprintf(buf2 + len, sizeof(buf2) - len,
							"%sPut %s [%d] in %s [%d], Max : %d\r\n",
							ZCMD(zone, subcmd).if_flag ? " then " : "-",
							obj_proto[ZCMD(zone, subcmd).arg1].short_description,
							obj_index[ZCMD(zone, subcmd).arg1].vnum,
							obj_proto[ZCMD(zone, subcmd).arg3].short_description,
							obj_index[ZCMD(zone, subcmd).arg3].vnum,
							ZCMD(zone, subcmd).arg2);
				found = TRUE;
				break;
			case 'R':
				if (ZCMD(zone, subcmd).arg1 != rnum)
					break; // not this room
				nlen = snprintf(buf2 + len, sizeof(buf2) - len,
							"%sRemove %s [%d] from room.\r\n",
							ZCMD(zone, subcmd).if_flag ? " then " : "-",
							obj_proto[ZCMD(zone, subcmd).arg2].short_description,
							obj_index[ZCMD(zone, subcmd).arg2].vnum);
				found = TRUE;
				break;
			case 'D':
				if (ZCMD(zone, subcmd).arg1 != rnum)
					break; /* not this room */
				nlen = snprintf(buf2 + len, sizeof(buf2) - len,
							"%sSet door %s as %s.\r\n",
							ZCMD(zone, subcmd).if_flag ? " then " : "-",
							dirs[ZCMD(zone, subcmd).arg2],
							ZCMD(zone, subcmd).arg3 ? 
									 ((ZCMD(zone, subcmd).arg3 == 1) ?
									  ((ZCMD(zone, subcmd).arg3 == 2) ?
										"hidden" : "closed") : "locked") : "open");
				found = TRUE;
				break;
			default: /* Catch all */
				nlen = snprintf(buf2 + len, sizeof(buf2) - len,
									 "<Unknown Command>");
				found = TRUE;
				break;
		}

      if (len + nlen >= sizeof(buf2) || nlen < 0)
			break;
      len += nlen;

		subcmd++;
	}
	if (*buf2)
		snprintf(buf1, sizeof(buf1), "%s", buf2);

	snprintf(output + strlen(output), 
				sizeof(output) - strlen(output) - len,
				"%s\r\n", *buf1 ? buf1 : "");
	send_to_char(ch, output);
}
