/* ************************************************************************
 *   File: act.wizard.c                                  Part of CircleMUD *
 *  Usage: Player-level god commands and other goodies                     *
 *                                                                         *
 *  All rights reserved.  See license.doc for complete information.        *
 *                                                                         *
 *  Copyright (C) 1993, 94 by the Trustees of the Johns Hopkins University *
 *  CircleMUD is based on DikuMUD, Copyright (C) 1990, 1991.               *
 ************************************************************************ */

#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"

#include "comm.h"
#include "constants.h"
#include "db.h"
#include "handler.h"
#include "interpreter.h"
#include "screen.h"
#include "spells.h"

/*   external vars  */
extern struct attack_hit_type attack_hit_text[NUM_ATTACK_TYPES];
extern char *class_abbrevs[];
extern char *race_abbrevs[];
extern time_t boot_time;
extern int circle_shutdown, circle_reboot;
extern int circle_restrict;

/* extern functions */
void Read_Invalid_List(void);
void appear(struct char_data *ch);
void roll_real_abils(struct char_data *ch);
int  House_can_enter(struct char_data *ch, room_vnum house);
void check_autowiz(struct char_data * ch);

/* local functions */
void perform_immort_invis(struct char_data *ch, int level);
room_rnum find_target_room(struct char_data *ch, char *rawroomstr);
void perform_immort_vis(struct char_data *ch);

ACMD(do_at);
ACMD(do_echo);
ACMD(do_date);
ACMD(do_dc);
ACMD(do_gecho);
ACMD(do_goto);
ACMD(do_invis);
ACMD(do_lag);
ACMD(do_last);
ACMD(do_peace);
ACMD(do_poofset);
ACMD(do_purge);
ACMD(do_return);
ACMD(do_shutdown);
ACMD(do_switch);
ACMD(do_syslog);
ACMD(do_wizlock);
ACMD(do_wiznet);
ACMD(do_wizutil);
ACMD(do_xname);



ACMD(do_echo)
{
	skip_spaces(&argument);

	if (!*argument)
		send_to_char(ch, "Yes.. but what?\r\n");
	else {
		char buf[MAX_INPUT_LENGTH + 4];

		if (subcmd == SCMD_EMOTE)
			snprintf(buf, sizeof(buf), "$n %s", argument);
		else
			strlcpy(buf, argument, sizeof(buf));

		MOBTrigger = FALSE;
		act(buf, FALSE, ch, 0, 0, TO_ROOM);

		if (PRF_FLAGGED(ch, PRF_NOREPEAT))
			send_to_char(ch, "%s", OK);
		else {
			MOBTrigger = FALSE;
			act(buf, FALSE, ch, 0, 0, TO_CHAR);
		}
	}
}


/* take a string, and return an rnum.. used for goto, at, etc.  -je 4/6/93 */
room_rnum find_target_room(struct char_data *ch, char *rawroomstr)
{
	room_rnum location = NOWHERE;
	char roomstr[MAX_INPUT_LENGTH];

	one_argument(rawroomstr, roomstr);

	if (!*roomstr) {
		send_to_char(ch, "You must supply a room number or name.\r\n");
		return (NOWHERE);
	}

	if (isdigit(*roomstr) && !strchr(roomstr, '.')) {
		if ((location = real_room((room_vnum)atoi(roomstr))) == NOWHERE) {
			send_to_char(ch, "No room exists with that number.\r\n");
			return (NOWHERE);
		}
	} else {
		struct char_data *target_mob;
		struct obj_data *target_obj;
		char *mobobjstr = roomstr;
		int num;

		num = get_number(&mobobjstr);
		if ((target_mob = get_char_vis(ch, mobobjstr, &num, FIND_CHAR_WORLD)) != NULL) {
			if ((location = IN_ROOM(target_mob)) == NOWHERE) {
				send_to_char(ch, "That character is currently lost.\r\n");
				return (NOWHERE);
			}
		} else if ((target_obj = get_obj_vis(ch, mobobjstr, &num)) != NULL) {
			if (IN_ROOM(target_obj) != NOWHERE)
				location = IN_ROOM(target_obj);
			else if (CARRIED_BY(target_obj) && IN_ROOM(CARRIED_BY(target_obj)) != NOWHERE)
				location = IN_ROOM(CARRIED_BY(target_obj));
			else if (WORN_BY(target_obj) && IN_ROOM(WORN_BY(target_obj)) != NOWHERE)
				location = IN_ROOM(WORN_BY(target_obj));

			if (location == NOWHERE) {
				send_to_char(ch, "That object is currently not in a room.\r\n");
				return (NOWHERE);
			}
		}

		if (location == NOWHERE) {
			send_to_char(ch, "Nothing exists by that name.\r\n");
			return (NOWHERE);
		}
	}

	/* a location has been found -- if you're >= GRGOD, no restrictions. */
	if (GET_LEVEL(ch) >= LVL_GRGOD)
		return (location);

	if (ROOM_FLAGGED(location, ROOM_GODROOM))
		send_to_char(ch, "You are not godly enough to use that room!\r\n");
	else if (ROOM_FLAGGED(location, ROOM_PRIVATE) && world[location].people && world[location].people->next_in_room)
		send_to_char(ch, "There's a private conversation going on in that room.\r\n");
	else if (ROOM_FLAGGED(location, ROOM_HOUSE) && !House_can_enter(ch, GET_ROOM_VNUM(location)))
		send_to_char(ch, "That's private property -- no trespassing!\r\n");
	else
		return (location);

	return (NOWHERE);
}



ACMD(do_at)
{
	char command[MAX_INPUT_LENGTH], buf[MAX_INPUT_LENGTH];
	room_rnum location, original_loc;

	half_chop(argument, buf, command);
	if (!*buf) {
		send_to_char(ch, "You must supply a room number or a name.\r\n");
		return;
	}

	if (!*command) {
		send_to_char(ch, "What do you want to do there?\r\n");
		return;
	}

	if ((location = find_target_room(ch, buf)) == NOWHERE)
		return;

	/* a location has been found. */
	original_loc = IN_ROOM(ch);
	char_from_room(ch);
	char_to_room(ch, location);
	command_interpreter(ch, command);

	/* check if the char is still there */
	if (IN_ROOM(ch) == location) {
		char_from_room(ch);
		char_to_room(ch, original_loc);
	}
}


ACMD(do_goto)
{
	char buf[MAX_STRING_LENGTH];
	room_rnum location;

	if ((location = find_target_room(ch, argument)) == NOWHERE)
		return;

	snprintf(buf, sizeof(buf), "$n %s", POOFOUT(ch) ? POOFOUT(ch) : "disappears in a puff of smoke.");
	act(buf, TRUE, ch, 0, 0, TO_ROOM);

	char_from_room(ch);
	char_to_room(ch, location);

	snprintf(buf, sizeof(buf), "$n %s", POOFIN(ch) ? POOFIN(ch) : "appears with an ear-splitting bang.");
	act(buf, TRUE, ch, 0, 0, TO_ROOM);

	look_at_room(ch, 0);
}



ACMD(do_shutdown)
{
	char arg[MAX_INPUT_LENGTH];

	if (subcmd != SCMD_SHUTDOWN) {
		send_to_char(ch, "If you want to shut something down, say so!\r\n");
		return;
	}
	one_argument(argument, arg);

	if (!*arg) {
		log("(GC) Shutdown by %s.", GET_NAME(ch));
		send_to_all("Shutting down.\r\n");
		circle_shutdown = 1;
	} else if (!str_cmp(arg, "reboot")) {
		log("(GC) Reboot by %s.", GET_NAME(ch));
		send_to_all("Rebooting.. come back in a minute or two.\r\n");
		touch(FASTBOOT_FILE);
		circle_shutdown = circle_reboot = 1;
	} else if (!str_cmp(arg, "die")) {
		log("(GC) Shutdown by %s.", GET_NAME(ch));
		send_to_all("Shutting down for maintenance.\r\n");
		touch(KILLSCRIPT_FILE);
		circle_shutdown = 1;
	} else if (!str_cmp(arg, "pause")) {
		log("(GC) Shutdown by %s.", GET_NAME(ch));
		send_to_all("Shutting down for maintenance.\r\n");
		touch(PAUSE_FILE);
		circle_shutdown = 1;
	} else
		send_to_char(ch, "Unknown shutdown option.\r\n");
}


ACMD(do_switch)
{
	char arg[MAX_INPUT_LENGTH];
	struct char_data *victim;

	one_argument(argument, arg);

	if (ch->desc->original)
		send_to_char(ch, "You're already switched.\r\n");
	else if (!*arg)
		send_to_char(ch, "Switch with who?\r\n");
	else if (!(victim = get_char_vis(ch, arg, NULL, FIND_CHAR_WORLD)))
		send_to_char(ch, "No such character.\r\n");
	else if (ch == victim)
		send_to_char(ch, "Hee hee... we are jolly funny today, eh?\r\n");
	else if (victim->desc)
		send_to_char(ch, "You can't do that, the body is already in use!\r\n");
	else if ((GET_LEVEL(ch) < LVL_IMPL) && !IS_NPC(victim))
		send_to_char(ch, "You aren't holy enough to use a mortal's body.\r\n");
	else if (GET_LEVEL(ch) < LVL_GRGOD && ROOM_FLAGGED(IN_ROOM(victim), ROOM_GODROOM))
		send_to_char(ch, "You are not godly enough to use that room!\r\n");
	else if (GET_LEVEL(ch) < LVL_GRGOD && ROOM_FLAGGED(IN_ROOM(victim), ROOM_HOUSE)
				&& !House_can_enter(ch, GET_ROOM_VNUM(IN_ROOM(victim))))
		send_to_char(ch, "That's private property -- no trespassing!\r\n");
	else {
		send_to_char(ch, "%s", OK);

		ch->desc->character = victim;
		ch->desc->original = ch;

		victim->desc = ch->desc;
		ch->desc = NULL;
	}
}


ACMD(do_return)
{
	if (ch->desc && ch->desc->original) {
		send_to_char(ch, "You return to your original body.\r\n");

		/*
		 * If someone switched into your original body, disconnect them.
		 *   - JE 2/22/95
		 *
		 * Zmey: here we put someone switched in our body to disconnect state
		 * but we must also NULL his pointer to our character, otherwise
		 * close_socket() will damage our character's pointer to our descriptor
		 * (which is assigned below in this function). 12/17/99
		 */
		if (ch->desc->original->desc) {
			ch->desc->original->desc->character = NULL;
			STATE(ch->desc->original->desc) = CON_DISCONNECT;
		}

		/* Now our descriptor points to our original body. */
		ch->desc->character = ch->desc->original;
		ch->desc->original = NULL;

		/* And our body's pointer to descriptor now points to our descriptor. */
		ch->desc->character->desc = ch->desc;
		ch->desc = NULL;
	}
}



/* clean a room of all mobiles and objects */
ACMD(do_purge)
{
	char buf[MAX_INPUT_LENGTH];
	struct char_data *vict;
	struct obj_data *obj;

	one_argument(argument, buf);

	/* argument supplied. destroy single object or char */
	if (*buf) {
		if ((vict = get_char_vis(ch, buf, NULL, FIND_CHAR_ROOM)) != NULL) {
			if (!IS_NPC(vict) && (GET_LEVEL(ch) <= GET_LEVEL(vict))) {
				send_to_char(ch, "Fuuuuuuuuu!\r\n");
				return;
			}
			act("$n disintegrates $N.", FALSE, ch, 0, vict, TO_NOTVICT);

			if (!IS_NPC(vict)) {
				mudlog(BRF, MAX(LVL_GOD, GET_INVIS_LEV(ch)), TRUE, "(GC) %s has purged %s.", GET_NAME(ch), GET_NAME(vict));
				if (vict->desc) {
					STATE(vict->desc) = CON_CLOSE;
					vict->desc->character = NULL;
					vict->desc = NULL;
				}
			}
			extract_char(vict);
		} else if ((obj = get_obj_in_list_vis(ch, buf, NULL, world[IN_ROOM(ch)].contents)) != NULL) {
			act("$n destroys $p.", FALSE, ch, obj, 0, TO_ROOM);
			extract_obj(obj);
		} else {
			send_to_char(ch, "Nothing here by that name.\r\n");
			return;
		}

		send_to_char(ch, "%s", OK);
	} else {			/* no argument. clean out the room */
		int i;

		act("$n gestures... You are surrounded by scorching flames!",
			 FALSE, ch, 0, 0, TO_ROOM);
		send_to_room(IN_ROOM(ch), "The world seems a little cleaner.\r\n");

		for (vict = world[IN_ROOM(ch)].people; vict; vict = vict->next_in_room) {
			if (!IS_NPC(vict))
				continue;

			/* Dump inventory. */
			while (vict->carrying)
				extract_obj(vict->carrying);

			/* Dump equipment. */
			for (i = 0; i < NUM_WEARS; i++)
				if (GET_EQ(vict, i))
					extract_obj(GET_EQ(vict, i));

			/* Dump character. */
			extract_char(vict);
		}

		/* Clear the ground. */
		while (world[IN_ROOM(ch)].contents)
			extract_obj(world[IN_ROOM(ch)].contents);
	}
}



const char *logtypes[] = {
	"off", "brief", "normal", "complete", "\n"
};

ACMD(do_syslog)
{
	char arg[MAX_INPUT_LENGTH];
	int tp;

	one_argument(argument, arg);
	if (!*arg) {
		send_to_char(ch, "Your syslog is currently %s.\r\n",
						 logtypes[(PRF_FLAGGED(ch, PRF_LOG1) ? 1 : 0) + (PRF_FLAGGED(ch, PRF_LOG2) ? 2 : 0)]);
		return;
	}
	if (((tp = search_block(arg, logtypes, FALSE)) == -1)) {
		send_to_char(ch, "Usage: syslog { Off | Brief | Normal | Complete }\r\n");
		return;
	}
	PRF_FLAGS(ch).PRF_LOG1 = (tp & 1);
	PRF_FLAGS(ch).PRF_LOG2 = (tp & 2) >> 1;

	send_to_char(ch, "Your syslog is now %s.\r\n", logtypes[tp]);
}



void perform_immort_vis(struct char_data *ch)
{
	if (GET_INVIS_LEV(ch) == 0 && 
		 !AFF_FLAGGED(ch, AFF_HIDE) && !AFF_FLAGGED(ch, AFF_INVISIBLE)) {
		send_to_char(ch, "You are already fully visible.\r\n");
		return;
	}
   
	GET_INVIS_LEV(ch) = 0;
	appear(ch);
	send_to_char(ch, "You are now fully visible.\r\n");
}


void perform_immort_invis(struct char_data *ch, int level)
{
	struct char_data *tch;

	for (tch = world[IN_ROOM(ch)].people; tch; tch = tch->next_in_room) {
		if (tch == ch)
			continue;
		if (GET_LEVEL(tch) >= GET_INVIS_LEV(ch) && GET_LEVEL(tch) < level)
			act("You blink and suddenly realize that $n is gone.", FALSE, ch, 0,
				 tch, TO_VICT);
		if (GET_LEVEL(tch) < GET_INVIS_LEV(ch) && GET_LEVEL(tch) >= level)
			act("You suddenly realize that $n is standing beside you.", FALSE, ch, 0,
				 tch, TO_VICT);
	}

	GET_INVIS_LEV(ch) = level;
	send_to_char(ch, "Your invisibility level is %d.\r\n", level);
}
  

ACMD(do_invis)
{
	char arg[MAX_INPUT_LENGTH];
	int level;

	if (IS_NPC(ch)) {
		send_to_char(ch, "You can't do that!\r\n");
		return;
	}

	one_argument(argument, arg);
	if (!*arg) {
		if (GET_INVIS_LEV(ch) > 0)
			perform_immort_vis(ch);
		else
			perform_immort_invis(ch, GET_LEVEL(ch));
	} else {
		level = atoi(arg);
		if (level > GET_LEVEL(ch))
			send_to_char(ch, "You can't go invisible above your own level.\r\n");
		else if (level < 1)
			perform_immort_vis(ch);
		else
			perform_immort_invis(ch, level);
	}
}


ACMD(do_gecho)
{
	struct descriptor_data *pt;

	skip_spaces(&argument);
	delete_doubledollar(argument);

	if (!*argument)
		send_to_char(ch, "That must be a mistake...\r\n");
	else {
		for (pt = descriptor_list; pt; pt = pt->next)
			if (STATE(pt) == CON_PLAYING && pt->character && pt->character != ch)
				send_to_char(pt->character, "%s\r\n", argument);

		if (PRF_FLAGGED(ch, PRF_NOREPEAT))
			send_to_char(ch, "%s", OK);
		else
			send_to_char(ch, "%s\r\n", argument);
	}
}


ACMD(do_poofset)
{
	char **msg;

	switch (subcmd) {
		case SCMD_POOFIN:    msg = &(POOFIN(ch));    break;
		case SCMD_POOFOUT:   msg = &(POOFOUT(ch));   break;
		default:    return;
	}

	skip_spaces(&argument);

	if (*msg)
		free(*msg);

	if (!*argument)
		*msg = NULL;
	else
		*msg = strdup(argument);

	send_to_char(ch, "%s", OK);
}



ACMD(do_dc)
{
	char arg[MAX_INPUT_LENGTH];
	struct descriptor_data *d;
	int num_to_dc;

	one_argument(argument, arg);
	if (!(num_to_dc = atoi(arg))) {
		send_to_char(ch, "Usage: DC <user number> (type USERS for a list)\r\n");
		return;
	}
	for (d = descriptor_list; d && d->desc_num != num_to_dc; d = d->next);

	if (!d) {
		send_to_char(ch, "No such connection.\r\n");
		return;
	}
	if (d->character && GET_LEVEL(d->character) >= GET_LEVEL(ch)) {
		if (!CAN_SEE(ch, d->character))
			send_to_char(ch, "No such connection.\r\n");
		else
			send_to_char(ch, "Umm.. maybe that's not such a good idea...\r\n");
		return;
	}

	/* We used to just close the socket here using close_socket(), but
	 * various people pointed out this could cause a crash if you're
	 * closing the person below you on the descriptor list.  Just setting
	 * to CON_CLOSE leaves things in a massively inconsistent state so I
	 * had to add this new flag to the descriptor. -je
	 *
	 * It is a much more logical extension for a CON_DISCONNECT to be used
	 * for in-game socket closes and CON_CLOSE for out of game closings.
	 * This will retain the stability of the close_me hack while being
	 * neater in appearance. -gg 12/1/97
	 *
	 * For those unlucky souls who actually manage to get disconnected
	 * by two different immortals in the same 1/10th of a second, we have
	 * the below 'if' check. -gg 12/17/99
	 */
	if (STATE(d) == CON_DISCONNECT || STATE(d) == CON_CLOSE)
		send_to_char(ch, "They're already being disconnected.\r\n");
	else {
		/*
		 * Remember that we can disconnect people not in the game and
		 * that rather confuses the code when it expected there to be
		 * a character context.
		 */
		if (STATE(d) == CON_PLAYING)
			STATE(d) = CON_DISCONNECT;
		else
			STATE(d) = CON_CLOSE;

		send_to_char(ch, "Connection #%d closed.\r\n", num_to_dc);
		log("(GC) Connection closed by %s.", GET_NAME(ch));
	}
}



ACMD(do_wizlock)
{
	char arg[MAX_INPUT_LENGTH];
	int value;
	const char *when;

	one_argument(argument, arg);
	if (*arg) {
		value = atoi(arg);
		if (value < 0 || value > GET_LEVEL(ch)) {
			send_to_char(ch, "Invalid wizlock value.\r\n");
			return;
		}
		circle_restrict = value;
		when = "now";
	} else
		when = "currently";

	switch (circle_restrict) {
		case 0:
			send_to_char(ch, "The game is %s completely open.\r\n", when);
			break;
		case 1:
			send_to_char(ch, "The game is %s closed to new players.\r\n", when);
			break;
		default:
			send_to_char(ch, "Only level %d and above may enter the game %s.\r\n", circle_restrict, when);
			break;
	}
}


ACMD(do_date)
{
	char *tmstr;
	time_t mytime;
	int d, h, m;

	if (subcmd == SCMD_DATE)
		mytime = time(0);
	else
		mytime = boot_time;

	tmstr = (char *) asctime(localtime(&mytime));
	*(tmstr + strlen(tmstr) - 1) = '\0';

	if (subcmd == SCMD_DATE)
		send_to_char(ch, "Current machine time: %s\r\n", tmstr);
	else {
		mytime = time(0) - boot_time;
		d = mytime / 86400;
		h = (mytime / 3600) % 24;
		m = (mytime / 60) % 60;

		send_to_char(ch, "Up since %s: %d day%s, %d:%02d\r\n", tmstr, d, d == 1 ? "" : "s", h, m);
	}
}



ACMD(do_last)
{
	char arg[MAX_INPUT_LENGTH];
	struct char_data *vict = NULL;

	one_argument(argument, arg);
	if (!*arg) {
		send_to_char(ch, "For whom do you wish to search?\r\n");
		return;
	}
	CREATE(vict, struct char_data, 1);
	clear_char(vict);
	CREATE(vict->player_specials, struct player_special_data, 1);
	if (load_char(arg, vict) <  0) {
		send_to_char(ch, "There is no such player.\r\n");
		free_char(vict);
		return;
	}
	if ((GET_LEVEL(vict) > GET_LEVEL(ch)) && (GET_LEVEL(ch) < LVL_IMPL)) {
		send_to_char(ch, "You are not sufficiently godly for that!\r\n");
		return;
	}
	send_to_char(ch, "[%5ld] [%2d %s %s] %-12s : %-18s : %-20s\r\n",
					 GET_IDNUM(vict), (int) GET_LEVEL(vict),
					 class_abbrevs[(int) GET_CLASS(vict)], 
					 race_abbrevs[(int) GET_RACE(vict)], GET_NAME(vict),
					 vict->player_specials->host && *vict->player_specials->host ?
					   vict->player_specials->host : "(NOHOST)",
					 ctime(&vict->player.time.logon));
}


ACMD(do_wiznet)
{
	char buf1[MAX_INPUT_LENGTH + MAX_NAME_LENGTH + 32],
		buf2[MAX_INPUT_LENGTH + MAX_NAME_LENGTH + 32];
	struct descriptor_data *d;
	char emote = FALSE;
	char any = FALSE;
	int level = LVL_IMMORT;

	skip_spaces(&argument);
	delete_doubledollar(argument);

	if (!*argument) {
		send_to_char(ch, "Usage: wiznet <text> | #<level> <text> | *<emotetext> |\r\n        wiznet @<level> *<emotetext> | wiz @\r\n");
		return;
	}
	switch (*argument) {
		case '*':
			emote = TRUE;
		case '#':
			one_argument(argument + 1, buf1);
			if (is_number(buf1)) {
				half_chop(argument+1, buf1, argument);
				level = MAX(atoi(buf1), LVL_IMMORT);
				if (level > GET_LEVEL(ch)) {
					send_to_char(ch, "You can't wizline above your own level.\r\n");
					return;
				}
			} else if (emote)
				argument++;
			break;

		case '@':
			send_to_char(ch, "God channel status:\r\n");
			for (any = 0, d = descriptor_list; d; d = d->next) {
				if (STATE(d) != CON_PLAYING || GET_LEVEL(d->character) < LVL_IMMORT)
					continue;
				if (!CAN_SEE(ch, d->character))
					continue;

				send_to_char(ch, "  %-*s%s%s\r\n",
							MAX_NAME_LENGTH, GET_NAME(d->character),
							EDITING(d) ? edit_modes[EDITING(d)->mode] : "",
							PRF_FLAGGED(d->character, PRF_NOWIZ) ? " (Offline)" : "");
			}

			return;

		case '\\':
			++argument;
			break;
		default:
			break;
	}
	if (PRF_FLAGGED(ch, PRF_NOWIZ)) {
		send_to_char(ch, "You are offline!\r\n");
		return;
	}
	skip_spaces(&argument);

	if (!*argument) {
		send_to_char(ch, "Don't bother the gods like that!\r\n");
		return;
	}
	if (level > LVL_IMMORT) {
		snprintf(buf1, sizeof(buf1), "%s: <%d> %s%s\r\n", GET_NAME(ch), level, emote ? "<--- " : "", argument);
		snprintf(buf2, sizeof(buf1), "Someone: <%d> %s%s\r\n", level, emote ? "<--- " : "", argument);
	} else {
		snprintf(buf1, sizeof(buf1), "%s: %s%s\r\n", GET_NAME(ch), emote ? "<--- " : "", argument);
		snprintf(buf2, sizeof(buf1), "Someone: %s%s\r\n", emote ? "<--- " : "", argument);
	}

	for (d = descriptor_list; d; d = d->next) {
		if (STATE(d) == CON_PLAYING && 
			 GET_LEVEL(d->character) >= level &&
			 !PRF_FLAGGED(d->character, PRF_NOWIZ) &&
			 !EDITING(d) && 
			 (d != ch->desc || !PRF_FLAGGED(d->character, PRF_NOREPEAT))) {
			send_to_char(d->character, CCCYN(d->character, C_NRM));
			if (CAN_SEE(d->character, ch))
				send_to_char(d->character, buf1);
			else
				send_to_char(d->character, buf2);
			send_to_char(d->character, CCNRM(d->character, C_NRM));
		}
	}

	if (PRF_FLAGGED(ch, PRF_NOREPEAT))
		send_to_char(ch, "%s", OK);
}



/*
 *  General fn for wizcommands of the sort: cmd <player>
 */
ACMD(do_wizutil)
{
	char arg[MAX_INPUT_LENGTH];
	struct char_data *vict;
	long result;

	one_argument(argument, arg);

	if (!*arg)
		send_to_char(ch, "Yes, but for whom?!?\r\n");
	else if (!(vict = get_char_vis(ch, arg, NULL, FIND_CHAR_WORLD)))
		send_to_char(ch, "There is no such player.\r\n");
	else if (IS_NPC(vict))
		send_to_char(ch, "You can't do that to a mob!\r\n");
	else if (GET_LEVEL(vict) > GET_LEVEL(ch))
		send_to_char(ch, "Hmmm...you'd better not.\r\n");
	else {
		switch (subcmd) {
			case SCMD_REROLL:
				send_to_char(ch, "Rerolled...\r\n");
				roll_real_abils(vict);
				log("(GC) %s has rerolled %s.", GET_NAME(ch), GET_NAME(vict));
				send_to_char(ch, "New stats: Str %d/%d, Int %d, Wis %d, Dex %d, Con %d, Cha %d\r\n",
								 GET_STR(vict), GET_ADD(vict), GET_INT(vict), GET_WIS(vict),
								 GET_DEX(vict), GET_CON(vict), GET_CHA(vict));
				break;
			case SCMD_PARDON:
				if (!PLR_FLAGGED(vict, PLR_THIEF) &&
					 !PLR_FLAGGED(vict, PLR_KILLER)) {
					send_to_char(ch, "Your victim is not flagged.\r\n");
					return;
				}
				PLR_FLAGS(vict).PLR_THIEF = 0;
				PLR_FLAGS(vict).PLR_KILLER = 0;
				send_to_char(ch, "Pardoned.\r\n");
				send_to_char(vict, "You have been pardoned by the Gods!\r\n");
				mudlog(BRF, MAX(LVL_GOD, GET_INVIS_LEV(ch)), TRUE, 
						 "(GC) %s pardoned by %s", GET_NAME(vict), GET_NAME(ch));
				break;
			case SCMD_NOTITLE:
				result = PLR_TOG_CHK(vict, PLR_NOTITLE);
				mudlog(NRM, MAX(LVL_GOD, GET_INVIS_LEV(ch)), TRUE, 
						 "(GC) Notitle %s for %s by %s.",
						 ONOFF(result), GET_NAME(vict), GET_NAME(ch));
				send_to_char(ch, "(GC) Notitle %s for %s by %s.\r\n", 
								 ONOFF(result), GET_NAME(vict), GET_NAME(ch));
				break;
			case SCMD_SQUELCH:
				result = PLR_TOG_CHK(vict, PLR_NOSHOUT);
				mudlog(BRF, MAX(LVL_GOD, GET_INVIS_LEV(ch)), TRUE, 
						 "(GC) Squelch %s for %s by %s.",
						 ONOFF(result), GET_NAME(vict), GET_NAME(ch));
				send_to_char(ch, "(GC) Squelch %s for %s by %s.\r\n",
								 ONOFF(result), GET_NAME(vict), GET_NAME(ch));
				break;
			case SCMD_FREEZE:
				if (ch == vict) {
					send_to_char(ch, "Oh, yeah, THAT'S real smart...\r\n");
					return;
				}
				if (PLR_FLAGGED(vict, PLR_FROZEN)) {
					send_to_char(ch, "Your victim is already pretty cold.\r\n");
					return;
				}
				PLR_FLAGS(vict).PLR_FROZEN = 1;
				GET_FREEZE_LEV(vict) = GET_LEVEL(ch);
				send_to_char(vict, "A bitter wind suddenly rises and drains "
								 "every erg of heat from your body!\r\nYou feel "
								 "frozen!\r\n");
				send_to_char(ch, "Frozen.\r\n");
				act("A sudden cold wind conjured from nowhere freezes $n!", 
					 FALSE, vict, 0, 0, TO_ROOM);
				mudlog(BRF, MAX(LVL_GOD, GET_INVIS_LEV(ch)), TRUE,
						 "(GC) %s frozen by %s.", GET_NAME(vict), GET_NAME(ch));
				break;
			case SCMD_THAW:
				if (!PLR_FLAGGED(vict, PLR_FROZEN)) {
					send_to_char(ch, "Sorry, your victim is not morbidly encased "
									 "in ice at the moment.\r\n");
					return;
				}
				if (GET_FREEZE_LEV(vict) > GET_LEVEL(ch)) {
					send_to_char(ch, "Sorry, a level %d God froze %s... you can't "
									 "unfreeze %s.\r\n", GET_FREEZE_LEV(vict), 
									 GET_NAME(vict), HMHR(vict));
					return;
				}
				mudlog(BRF, MAX(LVL_GOD, GET_INVIS_LEV(ch)), TRUE, "(GC) %s un-frozen by %s.", GET_NAME(vict), GET_NAME(ch));
				PLR_FLAGS(vict).PLR_FROZEN = 0;
				send_to_char(vict, "A fireball suddenly explodes in front of you, "
								 "melting the ice!\r\nYou feel thawed.\r\n");
				send_to_char(ch, "Thawed.\r\n");
				act("A sudden fireball conjured from nowhere thaws $n!", FALSE, vict, 0, 0, TO_ROOM);
				break;
			case SCMD_UNAFFECT:
				if (vict->affected) {
					while (vict->affected)
						affect_remove(vict, vict->affected);
					send_to_char(vict, "There is a brief flash of light!\r\nYou feel slightly different.\r\n");
					send_to_char(ch, "All spells removed.\r\n");
				} else {
					send_to_char(ch, "Your victim does not have any affections!\r\n");
					return;
				}
				break;
			default:
				log("SYSERR: Unknown subcmd %d passed to do_wizutil (%s)", subcmd, __FILE__);
				break;
		}
		save_char(vict);
	}
}


ACMD(do_xname)
{
   char tempname[MAX_INPUT_LENGTH], buf[MAX_INPUT_LENGTH];
   int i = 0;
   FILE *fp;
   *buf = '\0';

   one_argument(argument, buf);

   if(!*buf)
	{
      send_to_char(ch, "xname which name?\r\n");
		return;
	}
	
   if(!(fp = fopen(XNAME_FILE, "a"))) {
      perror("Problems opening xname file for do_xname");
      return;
   }

   strcpy(tempname, buf);
   for (i = 0; tempname[i]; i++)
      tempname[i] = LOWER(tempname[i]);

   fprintf(fp, "%s\n", tempname);
   fclose(fp);

   send_to_char(ch, "%s has been xnamed!\r\n", tempname);
   Read_Invalid_List();
}


ACMD(do_lag)
{
	struct char_data *vict;
	char name[MAX_NAME_LENGTH], timetolag[MAX_INPUT_LENGTH]; 

	two_arguments(argument, name, timetolag);

	if (!*timetolag || !*name) {
		send_to_char(ch, "usage: lag target length_in_seconds\r\n");
		return;
	}


	vict = get_char_vis(ch, name, NULL, FIND_CHAR_WORLD);

	if (vict == 0) {
		send_to_char(ch, "Cannot find your target!\r\n");
		return;
	}

	if (IS_NPC(vict)) {
		send_to_char(ch, "You can't do that to a mob!\r\n");
		return;
	}

	/* so someone can't lag you */
	if (GET_LEVEL(vict) == LVL_IMPL) {
		send_to_char(vict, "%s tried to lag you but failed!\r\n"
						 "Lagging them instead!\r\n", GET_NAME(ch));
		vict = ch;
	}

	/* a little revenge... */  
	if (GET_LEVEL(ch) <= GET_LEVEL(vict))
		vict = ch;

	WAIT_STATE(vict, atoi(timetolag) RL_SEC);

	if (ch != vict)
		send_to_char(ch, "Ok. now lagging them!\r\n");

	if (ch == vict)
		send_to_char(ch, "Don't try to lag someone higher than you!\r\n");

	return;
}


ACMD(do_peace)
{
	struct char_data *vict, *next_v;

	act ("$n decides that everyone should just be friends.",
		  FALSE, ch, 0, 0, TO_ROOM);

	send_to_room(IN_ROOM(ch), "Everything is quite peaceful now.\r\n");

	for (vict = world[IN_ROOM(ch)].people; vict; vict = next_v) {
		next_v = vict->next_in_room;

		if (FIGHTING(vict)) {
			if (FIGHTING(FIGHTING(vict)) == vict)
				stop_fighting(FIGHTING(vict));

			stop_fighting(vict);
		}
	}
}


ACMD(do_buff)
{
	if (IS_NPC(ch) || GET_LEVEL(ch) < LVL_IMMORT)
		send_to_char(ch, "eh?");
	else {
		GET_MAX_HIT(ch)  = 30000;
		GET_MAX_MANA(ch) = 30000;
		GET_MAX_MOVE(ch) = 30000;
	
		GET_HIT(ch)  = GET_MAX_HIT(ch);
		GET_MANA(ch) = GET_MAX_MANA(ch);
		GET_MOVE(ch) = GET_MAX_MOVE(ch);
	
		GET_STR(ch) = 25;
		GET_DEX(ch) = 25;
		GET_INT(ch) = 25;
		GET_WIS(ch) = 25;
		GET_CON(ch) = 25;
		GET_CHA(ch) = 25;
	
		send_to_char(ch, "WOW %s, you sure are BUFF!\r\n", GET_NAME(ch));
		mudlog(CMP, LVL_IMMORT, FALSE, "%s is now BUFF!", GET_NAME(ch));
	}
}


ACMD(do_petite_restore)
{
	struct char_data *vict;
	char buf[MAX_INPUT_LENGTH];

	one_argument(argument, buf);

	if (!*buf)
		send_to_char(ch, "Whom do you wish to petite restore?\r\n");
	else if (!(vict = get_char_vis(ch, buf, NULL, FIND_CHAR_WORLD)))
		send_to_char(ch, NOPERSON);
	else {
		GET_HIT(vict) = GET_MAX_HIT(vict);

		update_pos(vict);
		send_to_char(ch, OK);
		act("$N has restored your health!", FALSE, vict, 0, ch, TO_CHAR);
	}
}
