#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"

#include "coins.h"
#include "comm.h"
#include "constants.h"
#include "db.h"
#include "handler.h"
#include "mobprog.h"


/* external functions */
void mprog_read_programs(FILE * fp, struct index_data * pMobIndex);


/* local functions */
void parse_simple_mob(FILE *mob_f, int i, int nr);
void interpret_espec(const char *keyword, const char *value, int i, int nr);
void parse_espec(char *buf, int i, int nr);
void parse_enhanced_mob(FILE *mob_f, int i, int nr);
void parse_mobile(FILE *mob_f, int nr);
int vnum_mobile(char *searchname, struct char_data *ch);
struct char_data *read_mobile(mob_vnum nr, int type);
mob_rnum real_mobile(mob_vnum vnum);


void parse_simple_mob(FILE *mob_f, int i, int nr)
{
	int j, t[10];
	char line[READ_SIZE];

	mob_proto[i].real_abils.str = 11;
	mob_proto[i].real_abils.intel = 11;
	mob_proto[i].real_abils.wis = 11;
	mob_proto[i].real_abils.dex = 11;
	mob_proto[i].real_abils.con = 11;
	mob_proto[i].real_abils.cha = 11;

	if (!get_line(mob_f, line)) {
		log("SYSERR: Format error in mob #%d, file ended after S flag!", nr);
		exit(1);
	}

	if (sscanf(line, " %d %d %d %dd%d+%d %dd%d+%d ",
				  t, t + 1, t + 2, t + 3, t + 4, t + 5, t + 6, t + 7, t + 8) != 9) {
		log("SYSERR: Format error in mob #%d, first line after S flag\n"
			 "...expecting line of form '# # # #d#+# #d#+#'", nr);
		exit(1);
	}

	GET_LEVEL(mob_proto + i) = t[0];
	GET_HITROLL(mob_proto + i) = 20 - t[1];
	GET_AC(mob_proto + i) = 10 * t[2];

	/* max hit = 0 is a flag that H, M, V is xdy+z */
	GET_MAX_HIT(mob_proto + i) = 0;
	GET_HIT(mob_proto + i) = t[3];
	GET_MANA(mob_proto + i) = t[4];
	GET_MOVE(mob_proto + i) = t[5];

	GET_MAX_MANA(mob_proto + i) = 10;
	GET_MAX_MOVE(mob_proto + i) = 50;

	mob_proto[i].mob_specials.damnodice = t[6];
	mob_proto[i].mob_specials.damsizedice = t[7];
	GET_DAMROLL(mob_proto + i) = t[8];

	if (!get_line(mob_f, line)) {
      log("SYSERR: Format error in mob #%d, second line after S flag\n"
			 "...expecting line of form '# #', but file ended!", nr);
      exit(1);
	}

	if (sscanf(line, " %d %d ", t, t + 1) != 2) {
		log("SYSERR: Format error in mob #%d, second line after S flag\n"
			 "...expecting line of form '# #'", nr);
		exit(1);
	}

	convert_copper(mob_proto + i, t[0]);
	GET_SHARES(mob_proto + i) = 0;
	GET_EXP(mob_proto + i) = t[1];

	if (!get_line(mob_f, line)) {
		log("SYSERR: Format error in last line of mob #%d\n"
			 "...expecting line of form '# # #', but file ended!", nr);
		exit(1);
	}

	if (sscanf(line, " %d %d %d ", t, t + 1, t + 2) != 3) {
		log("SYSERR: Format error in last line of mob #%d\n"
			 "...expecting line of form '# # #'", nr);
		exit(1);
	}

	GET_POS(mob_proto + i) = t[0];
	GET_DEFAULT_POS(mob_proto + i) = t[1];
	GET_SEX(mob_proto + i) = t[2];

	GET_CLASS(mob_proto + i) = CLASS_OTHER;
	GET_RACE(mob_proto + i) = RACE_HUMAN;
	GET_WEIGHT(mob_proto + i) = 200;
	GET_HEIGHT(mob_proto + i) = 198;

	mob_proto[i].mob_specials.newitem = 0;

	/*
	 * these are now save applies; base save numbers for MOBs are now from
	 * the warrior save table.
	 */
	for (j = 0; j < 5; j++)
		GET_SAVE(mob_proto + i, j) = 0;
}


/*
 * interpret_espec is the function that takes espec keywords and values
 * and assigns the correct value to the mob as appropriate.  Adding new
 * e-specs is absurdly easy -- just add a new CASE statement to this
 * function!  No other changes need to be made anywhere in the code.
 *
 * CASE		: Requires a parameter through 'value'.
 * BOOL_CASE	: Being specified at all is its value.
 */

#define CASE(test)	\
	if (value && !matched && !str_cmp(keyword, test) && (matched = TRUE))

#define BOOL_CASE(test)	\
	if (!value && !matched && !str_cmp(keyword, test) && (matched = TRUE))

#define RANGE(low, high)	\
	(num_arg = MAX((low), MIN((high), (num_arg))))

void interpret_espec(const char *keyword, const char *value, int i, int nr)
{
	int num_arg = 0, matched = FALSE;

	/*
	 * If there isn't a colon, there is no value.  While Boolean options are
	 * possible, we don't actually have any.  Feel free to make some.
	 */
	if (value)
		num_arg = atoi(value);

	CASE("BareHandAttack") {
		RANGE(0, 99);
		mob_proto[i].mob_specials.attack_type = num_arg;
	}

	CASE("NumAttacks") {
		RANGE (1, 10);  /* Upper limit so we don't get too absurd :p   */
		mob_proto[i].mob_specials.num_attacks = num_arg;
	}
	
	CASE("Str") {
		RANGE(3, 25);
		mob_proto[i].real_abils.str = num_arg;
	}

	CASE("StrAdd") {
		RANGE(0, 100);
		mob_proto[i].real_abils.str_add = num_arg;    
	}

	CASE("Int") {
		RANGE(3, 25);
		mob_proto[i].real_abils.intel = num_arg;
	}

	CASE("Wis") {
		RANGE(3, 25);
		mob_proto[i].real_abils.wis = num_arg;
	}

	CASE("Dex") {
		RANGE(3, 25);
		mob_proto[i].real_abils.dex = num_arg;
	}

	CASE("Con") {
		RANGE(3, 25);
		mob_proto[i].real_abils.con = num_arg;
	}

	CASE("Cha") {
		RANGE(3, 25);
		mob_proto[i].real_abils.cha = num_arg;
	}

	if (!matched) {
		log("SYSERR: Warning: unrecognized espec keyword %s in mob #%d",
			 keyword, nr);
	}    
}

#undef CASE
#undef BOOL_CASE
#undef RANGE

void parse_espec(char *buf, int i, int nr)
{
	char *ptr;

	if ((ptr = strchr(buf, ':')) != NULL) {
		*(ptr++) = '\0';
		while (isspace(*ptr))
			ptr++;
	}
	interpret_espec(buf, ptr, i, nr);
}


void parse_enhanced_mob(FILE *mob_f, int i, int nr)
{
	char line[READ_SIZE];

	parse_simple_mob(mob_f, i, nr);

	while (get_line(mob_f, line)) {
		if (!strcmp(line, "E"))	/* end of the enhanced section */
			return;
		else if (*line == '#') {	/* we've hit the next mob, maybe? */
			log("SYSERR: Unterminated E section in mob #%d", nr);
			exit(1);
		} else
			parse_espec(line, i, nr);
	}

	log("SYSERR: Unexpected end of file reached after mob #%d", nr);
	exit(1);
}


void parse_mobile(FILE *mob_f, int nr)
{
	static int i = 0;
	int j, t[10];
	char line[READ_SIZE], *tmpptr, letter;
	char f1[128], f2[128], buf2[128];

	mob_index[i].vnum = nr;
	mob_index[i].number = 0;
	mob_index[i].func = NULL;

	clear_char(mob_proto + i);

	/*
	 * Mobiles should NEVER use anything in the 'player_specials' structure.
	 * The only reason we have every mob in the game share this copy of the
	 * structure is to save newbie coders from themselves. -gg 2/25/98
	 */
	mob_proto[i].player_specials = &dummy_mob;
	snprintf(buf2, sizeof(buf2), "mob vnum %d", nr);

	/***** String data *****/
	mob_proto[i].player.name = fread_string(mob_f, buf2);
	tmpptr = mob_proto[i].player.short_descr = fread_string(mob_f, buf2);
	if (tmpptr && *tmpptr)
		if (!str_cmp(fname(tmpptr), "a") || !str_cmp(fname(tmpptr), "an") ||
			 !str_cmp(fname(tmpptr), "the"))
			*tmpptr = LOWER(*tmpptr);
	mob_proto[i].player.long_descr = fread_string(mob_f, buf2);
	mob_proto[i].player.description = fread_string(mob_f, buf2);
	GET_TITLE(mob_proto + i) = NULL;
	mob_proto[i].mob_specials.num_attacks = 1;

	/* *** Numeric data *** */
	if (!get_line(mob_f, line)) {
		log("SYSERR: Format error after string section of mob #%d\n"
			 "...expecting line of form '# # # {S | E}', but file ended!", nr);
		exit(1);
	}

#ifdef CIRCLE_ACORN	/* Ugh. */
	if (sscanf(line, "%s %s %d %s", f1, f2, t + 2, &letter) != 4)
#else
		if (sscanf(line, "%s %s %d %c", f1, f2, t + 2, &letter) != 4)
#endif
		{
			log("SYSERR: Format error after string section of mob #%d\n"
				 "...expecting line of form '# # # {S | E}'", nr);
			exit(1);
		}

	ASCIICONV(MOB_FLAGS(mob_proto + i), f1);
	MOB_FLAGS(mob_proto + i).MOB_ISNPC = 1;

	if (MOB_FLAGGED(mob_proto + i, MOB_NOTDEADYET)) {
		/* Rather bad to load mobiles with this bit already set. */
		log("SYSERR: Mob #%d has reserved bit MOB_NOTDEADYET set.", nr);
		MOB_FLAGS(mob_proto + i).MOB_NOTDEADYET = 0;
	}
/*
	check_bitvector_names(MOB_FLAGS(mob_proto + i), action_bits_count, buf2, "mobile");
*/
	ASCIICONV(AFF_FLAGS(mob_proto + i), f2);
/*
	check_bitvector_names(AFF_FLAGS(mob_proto + i), affected_bits_count, buf2, "mobile affect");
*/
	GET_ALIGNMENT(mob_proto + i) = t[2];

	/* AGGR_TO_ALIGN is ignored if the mob is AGGRESSIVE. */
	if (MOB_FLAGGED(mob_proto + i, MOB_AGGRESSIVE) &&
		 (MOB_FLAGGED(mob_proto + i, MOB_AGGR_GOOD) ||
		  MOB_FLAGGED(mob_proto + i, MOB_AGGR_EVIL) ||
		  MOB_FLAGGED(mob_proto + i, MOB_AGGR_NEUTRAL)))
		log("SYSERR: Mob #%d both Aggressive and Aggressive_to_Alignment.", nr);

	switch (UPPER(letter)) {
		case 'S':	/* Simple monsters */
			parse_simple_mob(mob_f, i, nr);
			break;
		case 'E':	/* Circle3 Enhanced monsters */
			parse_enhanced_mob(mob_f, i, nr);
			break;
			/* add new mob types here.. */
		default:
			log("SYSERR: Unsupported mob type '%c' in mob #%d", letter, nr);
			exit(1);
	}

	mob_proto[i].aff_abils = mob_proto[i].real_abils;

	for (j = 0; j < NUM_WEARS; j++)
		mob_proto[i].equipment[j] = NULL;

	mob_proto[i].nr = i;
	mob_proto[i].desc = NULL;

	letter = fread_letter(mob_f);
	if (letter == '>') {
		ungetc(letter, mob_f);
		(void) mprog_read_programs(mob_f, &mob_index[i]);
	} else
		ungetc(letter, mob_f);
 
	top_of_mobt = i++;
}


int vnum_mobile(char *searchname, struct char_data *ch)
{
	int nr, found = 0;
	size_t nlen = 0, len = 0;
	char buf[MAX_STRING_LENGTH];
	buf[0] = '\0';

	for (nr = 0; nr <= top_of_mobt; nr++) {
		if (isname(searchname, mob_proto[nr].player.name)) {
			nlen = snprintf(buf + len, sizeof(buf) - len, "%3d. [%5d] %s\r\n",
								 ++found, mob_index[nr].vnum, 
								 mob_proto[nr].player.short_descr);

			if (len + nlen >= sizeof(buf) || nlen < 0)
				break;
			len += nlen;
		}
	}
	if (found && ch->desc)
		page_string(ch->desc, buf, TRUE);
	
	return (found);
}



/* create a new mobile from a prototype */
struct char_data *read_mobile(mob_vnum nr, int type) /* and mob_rnum */
{
	mob_rnum i;
	struct char_data *mob;

	if (type == VIRTUAL) {
		if ((i = real_mobile(nr)) == NOBODY) {
			log("WARNING: Mobile vnum %d does not exist in database.", nr);
			return (NULL);
		}
	} else
		i = nr;

	CREATE(mob, struct char_data, 1);
	clear_char(mob);
	*mob = mob_proto[i];
	mob->next = character_list;
	character_list = mob;

	if (!mob->points.max_hit) {
		mob->points.max_hit = dice(mob->points.hit, mob->points.mana) +
			mob->points.move;
	} else
		mob->points.max_hit = rand_number(mob->points.hit, mob->points.mana);

	mob->points.hit = mob->points.max_hit;
	mob->points.mana = mob->points.max_mana;
	mob->points.move = mob->points.max_move;

	mob->player.time.birth = time(0);
	mob->player.time.played = 0;
	mob->player.time.logon = time(0);

	mob_index[i].number++;

	return (mob);
}


/* returns the real number of the monster with given virtual number */
mob_rnum real_mobile(mob_vnum vnum)
{
	mob_rnum bot, top, mid;

	bot = 0;
	top = top_of_mobt;

	/* perform binary search on mob-table */
	for (;;) {
		mid = (bot + top) / 2;

		if ((mob_index + mid)->vnum == vnum)
			return (mid);
		if (bot >= top)
			return (NOBODY);
		if ((mob_index + mid)->vnum > vnum)
			top = mid - 1;
		else
			bot = mid + 1;
	}
}


/* MOBProg functions */
 
/* This routine transfers between alpha and numeric forms of the
 *  mob_prog bitvector types. This allows the use of the words in the
 *  mob/script files.
 */
int mprog_name_to_type (char *name)
{
	if (!str_cmp(name, "in_file_prog"  ))    return (IN_FILE_PROG);
	if (!str_cmp(name, "act_prog"      ))    return (ACT_PROG);
	if (!str_cmp(name, "speech_prog"   ))    return (SPEECH_PROG);
	if (!str_cmp(name, "rand_prog"     ))    return (RAND_PROG);
	if (!str_cmp(name, "fight_prog"    ))    return (FIGHT_PROG);
	if (!str_cmp(name, "hitprcnt_prog" ))    return (HITPRCNT_PROG);
	if (!str_cmp(name, "death_prog"    ))    return (DEATH_PROG);
	if (!str_cmp(name, "entry_prog"    ))    return (ENTRY_PROG);
	if (!str_cmp(name, "greet_prog"    ))    return (GREET_PROG);
	if (!str_cmp(name, "all_greet_prog"))    return (ALL_GREET_PROG);
	if (!str_cmp(name, "give_prog"     ))    return (GIVE_PROG);
	if (!str_cmp(name, "bribe_prog"    ))    return (BRIBE_PROG);
	if (!str_cmp(name, "talk_prog"     ))    return (TALK_PROG); 
 
	return (ERROR_PROG);
}
 
/*
 * Read a number from a file.
 */
int fread_number(FILE *fp)
{
	int number;
	bool sign;
	char c;
 
	do {
		c = getc(fp);
	} while (isspace(c));
 
	number = 0;
 
	sign   = FALSE;
	if (c == '+') 
		c = getc(fp);
	else if (c == '-') {
		sign = TRUE;
		c = getc(fp);
	}
 
	if (!isdigit(c)) {
		log("Fread_number: bad format.", 0);
		exit(1);
	}
 
	while (isdigit(c)) {
		number = number * 10 + c - '0';
		c      = getc(fp);
	}
 
	if (sign)
		number = 0 - number;
 
	if (c == '|')
		number += fread_number(fp);
	else if (c != ' ')
		ungetc(c, fp);
 
	return (number);
}

 
/*
 * Read to end of line (for comments).
 */
void fread_to_eol(FILE *fp)
{
	char c;
 
	do {
		c = getc(fp);
	} while (c != '\n' && c != '\r' && c != '~');
 
	do {
		c = getc(fp);
	} while (c == '\n' || c == '\r' && c != '~');
 
	ungetc(c, fp);
	return;
}

 
/*
 * Read one word (into static buffer).
 */
char *fread_word(FILE *fp)
{
	static char word[MAX_INPUT_LENGTH];
	char *pword;
	char cEnd;
 
	do {
		cEnd = getc(fp);
	} while (isspace(cEnd));
 
	if (cEnd == '\'' || cEnd == '"')
		pword   = word;
	else {
		word[0] = cEnd;
		pword   = word+1;
		cEnd    = ' ';
	}
 
	for (; pword < word + MAX_INPUT_LENGTH; pword++) {
		*pword = getc(fp);
		if (cEnd == ' ' ? isspace(*pword) || *pword == '~' : *pword == cEnd) {
			if (cEnd == ' ' || cEnd == '~')
				ungetc(*pword, fp);
			*pword = '\0';
			return (word);
		}
	}
 
	log("SYSERR: Fread_word: word too long.");
	exit(1);

	return (NULL);
}
 
 
/* This routine reads in scripts of MOBprograms from a file */
MPROG_DATA* mprog_file_read(char *f, MPROG_DATA *mprg,
		struct index_data *pMobIndex)
{
   char MOBProgfile[MAX_INPUT_LENGTH], buf2[MAX_STRING_LENGTH];
   char letter;
   int  done = FALSE;
   FILE *progfile;
   MPROG_DATA *mprg2;
 
   snprintf(MOBProgfile, sizeof(MOBProgfile), "%s%s", PRG_PREFIX, f);
 
   progfile = fopen(MOBProgfile, "r");
   if (!progfile) {
      log("Mob: %d couldnt open mobprog file", pMobIndex->vnum);
      exit(1);
   }
 
   mprg2 = mprg;
   switch (letter = fread_letter(progfile)) {
		case '>':
			break;
		case '|':
			log("empty mobprog file.", 0);
			exit(1);
			break;
		default:
			log("in mobprog file syntax error.", 0);
			exit(1);
			break;
   }
 
   while (!done) {
		mprg2->type = mprog_name_to_type(fread_word(progfile));
		switch (mprg2->type) {
			case ERROR_PROG:
				log("mobprog file type error", 0);
				exit(1);
				break;
			case IN_FILE_PROG:
				log("mprog file contains a call to file.", 0);
				exit(1);
				break;
			default:
				snprintf(buf2, sizeof(buf2), "Error in file %s", f);
				pMobIndex->progtypes = pMobIndex->progtypes | mprg2->type;
				mprg2->arglist       = fread_string(progfile,buf2);
				mprg2->comlist       = fread_string(progfile,buf2);
				switch (letter = fread_letter(progfile)) {
					case '>':
						CREATE(mprg2->next, MPROG_DATA, 1);
						mprg2       = mprg2->next;
						mprg2->next = NULL;
						break;
					case '|':
					case -1:
						done = TRUE;
						break;
					default:
						log("in mobprog file %s syntax error.", f);
						exit(1);
						break;
				}
				break;
		}
   }
   fclose(progfile);
   return (mprg2);
}

 
struct index_data *get_obj_index(int vnum)
{
   int nr;

   for (nr = 0; nr <= top_of_objt; nr++)
		if(obj_index[nr].vnum == vnum) 
			return (&obj_index[nr]);

   return (NULL);
}
 

struct index_data *get_mob_index(int vnum)
{
   int nr;

   for (nr = 0; nr <= top_of_mobt; nr++)
		if (mob_index[nr].vnum == vnum) 
			return (&mob_index[nr]);

   return (NULL);
}
 

/* This procedure is responsible for reading any in_file MOBprograms. */
void mprog_read_programs(FILE *fp, struct index_data *pMobIndex)
{
   MPROG_DATA *mprg;
   char letter;
	char buf2[MAX_STRING_LENGTH];
   int  done = FALSE;
 
   if ((letter = fread_letter(fp)) != '>') {
		log("Load_mobiles: vnum %d MOBPROG char", pMobIndex->vnum);
		exit(1);
   }
	CREATE(pMobIndex->mobprogs, MPROG_DATA, 1);
   mprg = pMobIndex->mobprogs;
 
   while (!done) {
		mprg->type = mprog_name_to_type(fread_word(fp));
		switch (mprg->type) {
			case ERROR_PROG:
				log("Load_mobiles: vnum %d MOBPROG type.", pMobIndex->vnum);
				exit(1);
				break;
			case IN_FILE_PROG:
				snprintf(buf2, sizeof(buf2), "Mobprog for mob #%d",
							pMobIndex->vnum);
				mprg = mprog_file_read(fread_word(fp), mprg,pMobIndex);
				fread_to_eol(fp);   /* need to strip off that silly ~*/
				switch (letter = fread_letter(fp)) {
					case '>':
						CREATE(mprg->next, MPROG_DATA, 1);
						mprg       = mprg->next;
						mprg->next = NULL;
						break;
					case '|':
						mprg->next = NULL;
						fread_to_eol(fp);
						done = TRUE;
						break;
					default:
						log("Load_mobiles: vnum %d bad MOBPROG.", pMobIndex->vnum);
						exit(1);
						break;
				}
				break;
			default:
				snprintf(buf2, sizeof(buf2), "Mobprog for mob #%d", 
							pMobIndex->vnum);
				pMobIndex->progtypes = pMobIndex->progtypes | mprg->type;
				mprg->arglist        = fread_string(fp, buf2);
				mprg->comlist        = fread_string(fp, buf2);
				switch (letter = fread_letter(fp)) {
					case '>':
						CREATE(mprg->next, MPROG_DATA, 1);
						mprg       = mprg->next;
						mprg->next = NULL;
						break;
					case '|':
						mprg->next = NULL;
						fread_to_eol(fp);
						done = TRUE;
						break;
					default:
						log("Load_mobiles: vnum %d bad MOBPROG.", pMobIndex->vnum);
						exit(1);
						break;
				}
				break;
		}
   }
}
