#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"

#include "comm.h"
#include "constants.h"
#include "db.h"
#include "handler.h"
#include "interpreter.h"
#include "spells.h"


/* extern vars */
extern int scheck;
extern const char *unused_spellname;	/* spell_parser.c */


/* local functions */
char *parse_object(FILE *obj_f, int nr);
int vnum_object(char *searchname, struct char_data *ch);
int vnum_weapon(int attacktype, struct char_data * ch);
struct obj_data *create_obj(void);
struct obj_data *read_object(obj_vnum nr, int type);
void free_obj(struct obj_data *obj);
void clear_object(struct obj_data *obj);
obj_rnum real_object(obj_vnum vnum);
int check_object(struct obj_data *obj);
int check_object_spell_number(struct obj_data *obj, int val);
int check_object_level(struct obj_data *obj, int val);


/* read all objects from obj file; generate index and prototypes */
char *parse_object(FILE *obj_f, int nr)
{
	static int i = 0;
	static char line[READ_SIZE];
	int t[10], j, retval, wsplnum;
	char *tmpptr;
	char f1[READ_SIZE], f2[READ_SIZE], buf2[128];
	struct extra_descr_data *new_descr;
						 
	obj_index[i].vnum = nr;
	obj_index[i].number = 0;
	obj_index[i].func = NULL;
									 
	clear_object(obj_proto + i);
	obj_proto[i].item_number = i;
	obj_proto[i].sitting_here = NULL;
											 
	snprintf(buf2, sizeof(buf2), "object #%d", nr);
												 
	/* *** string data *** */
	if ((obj_proto[i].name = fread_string(obj_f, buf2)) == NULL) {
		log("SYSERR: Null obj name or format error at or near %s", buf2);
		exit(1);
	}
	tmpptr = obj_proto[i].short_description = fread_string(obj_f, buf2);
	if (tmpptr && *tmpptr)
		if (!str_cmp(fname(tmpptr), "a") || !str_cmp(fname(tmpptr), "an") ||
			 !str_cmp(fname(tmpptr), "the"))
			*tmpptr = LOWER(*tmpptr);
	tmpptr = obj_proto[i].description = fread_string(obj_f, buf2);
	if (tmpptr && *tmpptr)
		CAP(tmpptr);
	obj_proto[i].action_description = fread_string(obj_f, buf2);
	/* *** numeric data *** */
	if (!get_line(obj_f, line)) {
		log("SYSERR: Expecting first numeric line of %s, but file ended!", buf2);
		exit(1);
	}
	if ((retval = sscanf(line, " %d %s %s", t, f1, f2)) != 3) {
		log("SYSERR: Format error in first numeric line (expecting 3 args, got %d), %s", retval, buf2);
		exit(1);
	}
	/* Object flags checked in check_object(). */
	GET_OBJ_TYPE(obj_proto + i) = t[0];
	ASCIICONV(obj_proto[i].obj_flags.extra_flags, f1);
	ASCIICONV(obj_proto[i].obj_flags.wear_flags, f2);

	if (!get_line(obj_f, line)) {
		log("SYSERR: Expecting second numeric line of %s, but file ended!", buf2);
		exit(1);
	}
	if ((retval = sscanf(line, "%d %d %d %d", t, t + 1, t + 2, t + 3)) != 4) {
		log("SYSERR: Format error in second numeric line (expecting 4 args, "
			 "got %d), %s", retval, buf2);
		exit(1);
	}
	GET_OBJ_VAL(obj_proto + i, 0) = t[0];
	GET_OBJ_VAL(obj_proto + i, 1) = t[1];
	GET_OBJ_VAL(obj_proto + i, 2) = t[2];
	GET_OBJ_VAL(obj_proto + i, 3) = t[3];
	if (!get_line(obj_f, line)) {
		log("SYSERR: Expecting third numeric line of %s, but file ended!", buf2);
		exit(1);
	}
	if ((retval = sscanf(line, "%d %d %d", t, t + 1, t + 2)) != 3) {
		log("SYSERR: Format error in third numeric line (expecting 3 args, "
			 "got %d), %s", retval, buf2);
		exit(1);
	}
	GET_OBJ_WEIGHT(obj_proto + i) = t[0];
	GET_OBJ_COST(obj_proto + i) = t[1];
	GET_OBJ_RENT(obj_proto + i) = t[2];
	/* check to make sure that weight of containers exceeds curr. quantity */
	if (GET_OBJ_TYPE(obj_proto + i) == ITEM_DRINKCON || 
		 GET_OBJ_TYPE(obj_proto + i) == ITEM_FOUNTAIN) {
		if (GET_OBJ_WEIGHT(obj_proto + i) < GET_OBJ_VAL(obj_proto + i, 1))
			GET_OBJ_WEIGHT(obj_proto + i) = GET_OBJ_VAL(obj_proto + i, 1) + 5;
	}
	/* *** extra descriptions and affect fields *** */
	for (j = 0; j < MAX_OBJ_AFFECT; j++) {
		obj_proto[i].affected[j].location = APPLY_NONE;
		obj_proto[i].affected[j].modifier = 0;
	}
	strcat(buf2, ", after numeric constants\n" /* strcat: OK (for 'buf2 >= 87') */
			 "...expecting 'E', 'A', '$', or next object number");
	j = 0;
	for (;;) {
		if (!get_line(obj_f, line)) {
			log("SYSERR: Format error in %s", buf2);
			exit(1);
		}
		switch (*line) {
			case 'E':
				CREATE(new_descr, struct extra_descr_data, 1);
				new_descr->keyword = fread_string(obj_f, buf2);
				new_descr->description = fread_string(obj_f, buf2);
				new_descr->next = obj_proto[i].ex_description;
				obj_proto[i].ex_description = new_descr;
				break;
			case 'T': /* Object Timers */
				if (!get_line(obj_f, line)) {
					log("SYSERR: Format error in 'T' field, %s\n"
						 "...expecting numeric constant but file ended!", buf2);
					exit(1);
				}

				if ((retval = sscanf(line, "%d", t)) != 1) {
					log("SYSERR: Format error in 'T' field, %s\n"
						 "...expecting 1 numeric argument, got %d\n"
						 "...offending line: '%s'", buf2, retval, line);
					exit(1);
				}
				obj_proto[i].obj_flags.timer = t[0];
				break;
			case 'A': /* Object Affects */
				if (j >= MAX_OBJ_AFFECT) {
					log("SYSERR: Too many A fields (%d max), %s", MAX_OBJ_AFFECT, buf2);
					exit(1);
				}
				if (!get_line(obj_f, line)) {
					log("SYSERR: Format error in 'A' field, %s\n"
						 "...expecting 2 numeric constants but file ended!", buf2);
					exit(1);
				}
																																							 
				if ((retval = sscanf(line, " %d %d ", t, t + 1)) != 2) {
					log("SYSERR: Format error in 'A' field, %s\n"
						 "...expecting 2 numeric arguments, got %d\n"
						 "...offending line: '%s'", buf2, retval, line);
					exit(1);
				}
				obj_proto[i].affected[j].location = t[0];
				obj_proto[i].affected[j].modifier = t[1];
				j++;
				break;
         case 'C':
				/*
				** affect player flags (see AFF_XXX flags in structs.h )
				** This allows players flags to be affected by an item,
				** for example create a ring which gives the player
				** permanent sense life when worn.
				**
				** Note: more 'C' fields are allowed because all found 'C'
				** (set char bits) fields are added to the current
				** values. (nice for builders who don't want those large
				** bitvector numbers in the obj file).
				*/
				if (!get_line(obj_f, line)) {
					log("SYSERR: Format error in 'C' field, %s\n"
						 "...expecting numeric constant but file ended!", buf2);
					exit(1);
				}
				
				if ((retval = sscanf(line, "%d", t)) != 1) {
					log("SYSERR: Format error in 'C' field, %s\n"
						 "...expecting 1 numeric argument, got %d\n"
						 "...offending line: '%s'", buf2, retval, line);
					exit(1);
				}
				
				snprintf(f1, sizeof(f1), "%d\0", t[0]);
				ASCIICONV(obj_proto[i].obj_flags.bitvector, f1);
				break;
			case 'S':  /* Weapon Spells */
				if (wsplnum > MAX_WEAPON_SPELLS) {
					log("SYSERR: Too many S fields (%d max), %s", MAX_WEAPON_SPELLS, buf2);
					exit(1);
				}
				if (!get_line(obj_f, line)) {
					log("SYSERR: Format error in 'S' field, %s.  Expecting "
						 "numeric constants, but file ended!",buf2);
					exit(1);
				}

				if((retval = sscanf(line, " %d %d %d ", t, t+1, t+2)) != 3) {
					log("SYSERR: Format error in 'S' field, %s\n"
						 "...expecting 3 numeric argumentss, got %d\n"
						 "...offending line: '%s'", buf2, retval, line);
					exit(1);
				}

				obj_proto[i].has_spells = TRUE;
				obj_proto[i].wpn_spells[wsplnum].spellnum = t[0];
				obj_proto[i].wpn_spells[wsplnum].level    = t[1];
				obj_proto[i].wpn_spells[wsplnum].percent  = t[2];
				wsplnum += 1;
				break;
			case '$':
			case '#':
				check_object(obj_proto + i);
				top_of_objt = i++;
				return (line);
			default:
				log("SYSERR: Format error in (%c): %s", *line, buf2);
				exit(1);
		}
	}
}


int vnum_object(char *searchname, struct char_data *ch)
{
	int nr, found = 0;
	size_t nlen = 0, len = 0;
	char buf[MAX_STRING_LENGTH];
	buf[0] = '\0';
			 
	for (nr = 0; nr <= top_of_objt; nr++) {
		if (isname(searchname, obj_proto[nr].name)) {
			nlen = snprintf(buf + len, sizeof(buf) - len, "%3d. [%5d] %s\r\n", 
								 ++found, obj_index[nr].vnum, 
								 obj_proto[nr].short_description);

			if (len + nlen >= sizeof(buf) || nlen < 0)
				break;
			len += nlen;
		}
	}
	if (found && ch->desc)
		page_string(ch->desc, buf, TRUE);
																									
	return (found);
}


int vnum_weapon(int attacktype, struct char_data * ch)
{
	int nr, found = 0;
	size_t nlen = 0, len = 0;
	char buf[MAX_STRING_LENGTH];
	buf[0] = '\0';

	for (nr = 0; nr <= top_of_objt; nr++) {
		if (obj_proto[nr].obj_flags.type_flag == ITEM_WEAPON && 
			 obj_proto[nr].obj_flags.value[3] == attacktype) {
			nlen = snprintf(buf + len, sizeof(buf) - len, "%3d. [%5d] %s\r\n",
								 ++found, obj_index[nr].vnum,
								 obj_proto[nr].short_description);

			if (len + nlen >= sizeof(buf) || nlen < 0)
				break;
			len += nlen;
		}
	}
	if (found && ch->desc)
		page_string(ch->desc, buf, TRUE);

	return (found);
}


bool can_wear(struct obj_data *obj, int part)
{
	if (ISFIELDSET(obj->obj_flags.wear_flags, part))
		return (TRUE);

	return (FALSE);
}

void vwear_object(struct can_be_worn_on wearpos, struct char_data * ch)
{
	int nr, found = 0;
	size_t nlen = 0, len = 0;
	char buf[MAX_STRING_LENGTH] = "";

	for (nr = 0; nr <= top_of_objt; nr++) {
		if (ISFIELDSET((&obj_proto[nr])->obj_flags.wear_flags, wearpos)) {
			nlen = snprintf(buf + len, sizeof(buf) - len, "%3d. [%5d] %s\r\n",
								 ++found, obj_index[nr].vnum, 
								 obj_proto[nr].short_description);

			if (len + nlen >= sizeof(buf) || nlen < 0)
				break;
			len += nlen;
		}
	}
	if (ch->desc)
		page_string(ch->desc, buf, TRUE);
}

void vwear_obj(int type, struct char_data * ch)
{
	int nr, found =0;
	size_t nlen = 0, len = 0;
	char buf[MAX_STRING_LENGTH] = "";

	for (nr = 0; nr <= top_of_objt; nr++) {
		if ((&obj_proto[nr])->obj_flags.type_flag == type) {
			nlen = snprintf(buf + len, sizeof(buf) - len, "%3d. [%5d] %s\r\n",
								 ++found, obj_index[nr].vnum, 
								 obj_proto[nr].short_description);

			if (len + nlen >= sizeof(buf) || nlen < 0)
				break;
			len += nlen;
		}
	}
	if (ch->desc)
		page_string(ch->desc, buf, TRUE);
}


/* create an object, and add it to the object list */
struct obj_data *create_obj(void)
{
	struct obj_data *obj;
	 
	CREATE(obj, struct obj_data, 1);
	clear_object(obj);
	obj->next = object_list;
	object_list = obj;
					 
	return (obj);
}


/* create a new object from a prototype */
struct obj_data *read_object(obj_vnum nr, int type) /* and obj_rnum */
{
	struct obj_data *obj;
	obj_rnum i = type == VIRTUAL ? real_object(nr) : nr;
		 
	if (i == NOTHING || i > top_of_objt) {
		log("Object (%c) %d does not exist in database.", type == VIRTUAL ? 'V' : 'R', nr);
		return (NULL);
	}
															  
	CREATE(obj, struct obj_data, 1);
	clear_object(obj);
	*obj = obj_proto[i];
	obj->next = object_list;
	object_list = obj;
																				  
	obj_index[i].number++;
																					  
	return (obj);
}



/* release memory allocated for an obj struct */
void free_obj(struct obj_data *obj)
{
	int nr;
	 
	if ((nr = GET_OBJ_RNUM(obj)) == NOTHING) {
		if (obj->name)
			free(obj->name);
		if (obj->description)
			free(obj->description);
		if (obj->short_description)
			free(obj->short_description);
		if (obj->action_description)
			free(obj->action_description);
		if (obj->ex_description)
			free_extra_descriptions(obj->ex_description);
	} else {
		if (obj->name && obj->name != obj_proto[nr].name)
			free(obj->name);
		if (obj->description && obj->description != obj_proto[nr].description)
			free(obj->description);
		if (obj->short_description && obj->short_description != obj_proto[nr].short_description)
			free(obj->short_description);
		if (obj->action_description && obj->action_description != obj_proto[nr].action_description)
			free(obj->action_description);
		if (obj->ex_description && obj->ex_description != obj_proto[nr].ex_description)
			free_extra_descriptions(obj->ex_description);
	}
																
	free(obj);
}


void clear_object(struct obj_data *obj)
{
	memset((char *) obj, 0, sizeof(struct obj_data));
	 
	obj->item_number = NOTHING;
	IN_ROOM(obj) = NOWHERE;
	WORN_ON(obj) = NOWHERE;
}


/* returns the real number of the object with given virtual number */
obj_rnum real_object(obj_vnum vnum)
{
	obj_rnum bot, top, mid;
	 
	bot = 0;
	top = top_of_objt;
			 
	/* perform binary search on obj-table */
	for (;;) {
		mid = (bot + top) / 2;
	 
		if ((obj_index + mid)->vnum == vnum)
			return (mid);
		if (bot >= top)
			return (NOTHING);
		if ((obj_index + mid)->vnum > vnum)
			top = mid - 1;
		else
			bot = mid + 1;
	}
}


/*
 * Extend later to include more checks.
 *
 * TODO: Add checks for unknown bitvectors.
 */
int check_object(struct obj_data *obj)
{
	char objname[MAX_INPUT_LENGTH + 32];
	int error = FALSE;
		 
	if (GET_OBJ_WEIGHT(obj) < 0 && (error = TRUE))
		log("SYSERR: Object #%d (%s) has negative weight (%d).",
			 GET_OBJ_VNUM(obj), obj->short_description, GET_OBJ_WEIGHT(obj));
																		  
	if (GET_OBJ_RENT(obj) < 0 && (error = TRUE))
		log("SYSERR: Object #%d (%s) has negative cost/day (%d).",
			 GET_OBJ_VNUM(obj), obj->short_description, GET_OBJ_RENT(obj));
																																		 
	snprintf(objname, sizeof(objname), "Object #%d (%s)", GET_OBJ_VNUM(obj), obj->short_description);
/*
	error |= check_bitvector_names(GET_OBJ_WEAR(obj), wear_bits_count, objname, "object wear");
	error |= check_bitvector_names(GET_OBJ_EXTRA(obj), extra_bits_count, objname, "object extra");
	error |= check_bitvector_names(GET_OBJ_AFFECT(obj), affected_bits_count, objname, "object affect");
*/
	switch (GET_OBJ_TYPE(obj)) {
		case ITEM_DRINKCON:
		{
			char onealias[MAX_INPUT_LENGTH], *space = strrchr(obj->name, ' ');
	 
			strlcpy(onealias, space ? space + 1 : obj->name, sizeof(onealias));
			if (search_block(onealias, drinknames, TRUE) < 0 && (error = TRUE))
				log("SYSERR: Object #%d (%s) doesn't have drink type as last "
					 "alias. (%s)", GET_OBJ_VNUM(obj), obj->short_description, 
					 obj->name);
		}
		/* Fall through. */
		case ITEM_FOUNTAIN:
			if (GET_OBJ_VAL(obj, 1) > GET_OBJ_VAL(obj, 0) && (error = TRUE))
				log("SYSERR: Object #%d (%s) contains (%d) more than "
					 "maximum (%d).", GET_OBJ_VNUM(obj), obj->short_description,
					 GET_OBJ_VAL(obj, 1), GET_OBJ_VAL(obj, 0));
			break;
		case ITEM_SCROLL:
		case ITEM_POTION:
			error |= check_object_level(obj, 0);
			error |= check_object_spell_number(obj, 1);
			error |= check_object_spell_number(obj, 2);
			error |= check_object_spell_number(obj, 3);
			break;
		case ITEM_WAND:
		case ITEM_STAFF:
			error |= check_object_level(obj, 0);
			error |= check_object_spell_number(obj, 3);
			if (GET_OBJ_VAL(obj, 2) > GET_OBJ_VAL(obj, 1) && (error = TRUE))
				log("SYSERR: Object #%d (%s) has more charges (%d) than "
					 "maximum (%d).", GET_OBJ_VNUM(obj), obj->short_description,
					 GET_OBJ_VAL(obj, 2), GET_OBJ_VAL(obj, 1));
			break;
		case ITEM_CHAIR:
			if (GET_OBJ_VAL(obj, 1) > GET_OBJ_VAL(obj, 0) && (error = TRUE))
				log("SYSERR: Object #%d (%s) contains (%d) more than "
					 "maximum (%d).", GET_OBJ_VNUM(obj), obj->short_description,
					 GET_OBJ_VAL(obj, 1), GET_OBJ_VAL(obj, 0));     
			break;
	}
	return (error);
}


int check_object_spell_number(struct obj_data *obj, int val)
{
	int error = FALSE;
	const char *spellname;
		 
	if (GET_OBJ_VAL(obj, val) == -1)	/* i.e.: no spell */
		return (error);
																					  
	/*
	 * Check for negative spells, spells beyond the top define, and any
	 * spell which is actually a skill.
	 */
	if (GET_OBJ_VAL(obj, val) < 0)
		error = TRUE;
	if (GET_OBJ_VAL(obj, val) > TOP_SPELL_DEFINE)
		error = TRUE;
	if (GET_OBJ_VAL(obj, val) > MAX_SPELLS && GET_OBJ_VAL(obj, val) <= MAX_SKILLS)
		error = TRUE;
	if (error)
		log("SYSERR: Object #%d (%s) has out of range spell #%d.",
			 GET_OBJ_VNUM(obj), obj->short_description, GET_OBJ_VAL(obj, val));
	/*
	 * This bug has been fixed, but if you don't like the special behavior...
	 */
#if 0
	if (GET_OBJ_TYPE(obj) == ITEM_STAFF &&
		 HAS_SPELL_ROUTINE(GET_OBJ_VAL(obj, val), MAG_AREAS | MAG_MASSES))
		log("... '%s' (#%d) uses %s spell '%s'.",
			 obj->short_description,	GET_OBJ_VNUM(obj),
			 HAS_SPELL_ROUTINE(GET_OBJ_VAL(obj, val), MAG_AREAS) ? "area" : "mass",
			 skill_name(GET_OBJ_VAL(obj, val)));
#endif
	if (scheck)		/* Spell names don't exist in syntax check mode. */
		return (error);
	/* Now check for unnamed spells. */
	spellname = skill_name(GET_OBJ_VAL(obj, val));
	if ((spellname == unused_spellname || !str_cmp("UNDEFINED", spellname)) && (error = TRUE))
		log("SYSERR: Object #%d (%s) uses '%s' spell #%d.",
			 GET_OBJ_VNUM(obj), obj->short_description, spellname,
			 GET_OBJ_VAL(obj, val));
	return (error);
}


int check_object_level(struct obj_data *obj, int val)
{
	int error = FALSE;
	 
	if ((GET_OBJ_VAL(obj, val) < 0 || GET_OBJ_VAL(obj, val) > LVL_IMPL) &&
		 (error = TRUE)) {
		log("SYSERR: Object #%d (%s) has out of range level #%d.",
			 GET_OBJ_VNUM(obj), obj->short_description, GET_OBJ_VAL(obj, val));
	}
	
	return (error);
}


int my_obj_save_to_disk(FILE *fp, struct obj_data *obj, int locate)
{
	int counter2;
	struct extra_descr_data *ex_desc;
	char buf1[MAX_STRING_LENGTH +1];

	if (obj->action_description) {
		strcpy(buf1, obj->action_description);
		strip_string(buf1);
	} else
		*buf1 = 0;

	fprintf(fp, "#%d\n%d %d %d %d %d %d\n", GET_OBJ_VNUM(obj), locate,
			  GET_OBJ_VAL(obj, 0), GET_OBJ_VAL(obj, 1),
			  GET_OBJ_VAL(obj, 2), GET_OBJ_VAL(obj, 3), GET_OBJ_EXTRA(obj));

	if (!(IS_OBJ_STAT(obj,ITEM_UNIQUE_SAVE)))
		return (1);

	fprintf(fp, "XAP\n%s~\n%s~\n%s~\n%s~\n%d %d %d %d %d\n",
			  obj->name ? obj->name : "undefined",
			  obj->short_description ? obj->short_description : "undefined",
			  obj->description ? obj->description : "undefined",
			  buf1, GET_OBJ_TYPE(obj), GET_OBJ_WEAR(obj), GET_OBJ_WEIGHT(obj),
			  GET_OBJ_COST(obj), GET_OBJ_RENT(obj));

	/* Do we have affects? */
	for (counter2 = 0; counter2 < MAX_OBJ_AFFECT; counter2++) {
		if (obj->affected[counter2].modifier)
			fprintf(fp, "A\n%d %d\n", obj->affected[counter2].location,
					  obj->affected[counter2].modifier);
	}
	
	/* Do we have extra descriptions? */
	if (obj->ex_description) { /* Yep, save them too */
		for (ex_desc = obj->ex_description; ex_desc; ex_desc = ex_desc->next) {
			/* Sanity check to prevent nasty protection faults */
			if (!*ex_desc->keyword || !*ex_desc->description)
            continue;

			strcpy(buf1, ex_desc->description);
			strip_string(buf1);
			fprintf(fp, "E\n%s~\n%s~\n", ex_desc->keyword, buf1);
		}
	}
	return (1);
}

/* This procedure removes the '\r\n' from a string so that it may be
	saved to a file.  Use it only on buffers, not on the orginal
	strings. */

void strip_string(char *buffer)
{
	register char *ptr, *str;

	ptr = buffer;
	str = ptr;

	while ((*str = *ptr)) {
		str++;
		ptr++;
		if (*ptr == '\r')
			ptr++;
	}
}

int read_xap_objects(FILE *fl, struct char_data *ch) {
	struct obj_data *temp = NULL;
	struct extra_descr_data *new_descr;
	char line[MAX_STRING_LENGTH], buf2[MAX_STRING_LENGTH];
	char f1[READ_SIZE];
	int t[10], nr, danger, j, k, zwei;
	int num_of_objs = 0, retval, wsplnum;

	if(!feof(fl))
		get_line(fl, line);

	while (!feof(fl) && line != NULL) {
		/* first, we get the number. Not too hard. */
		if(*line == '#') {
			num_of_objs++;
			if (sscanf(line, "#%d", &nr) != 1) {
				continue;
			}
			/* we have the number, check it, load obj. */
			if (nr == -1) {  /* then its unique */
				temp = create_obj();
				temp->item_number = NOTHING;
				GET_OBJ_EXTRA(temp).ITEM_UNIQUE_SAVE;
			} else if ( nr < 0) 
				continue;
			else {
				temp = read_object(nr, VIRTUAL);
				if (nr >= 99999 || !temp) {
					continue;
				}
			}
			/* now we read locate. - this is for autoeq, will be 0 elsewise */
			/* only the rest of the vals, and extra flags will be read */
			get_line(fl,line);
			sscanf(line,"%d %d %d %d %d %s", t, t + 1, t+2, t + 3, t + 4, f1);
			GET_OBJ_VAL(temp,0) = t[1];
			GET_OBJ_VAL(temp,1) = t[2];
			GET_OBJ_VAL(temp,2) = t[3];
			GET_OBJ_VAL(temp,3) = t[4];
			ASCIICONV(temp->obj_flags.extra_flags, f1);

			get_line(fl,line);
			/* read line check for xap. */
			if(!str_cmp("XAP",line)) {  /* then this is a Xap Obj, requires
														 special care */
				if ((temp->name = fread_string(fl, buf2)) == NULL)
					temp->name = "undefined";

				if ((temp->short_description = fread_string(fl, buf2)) == NULL)
					temp->short_description = "undefined";

				if ((temp->description = fread_string(fl, buf2)) == NULL)
					temp->description = "undefined";

				if ((temp->action_description = fread_string(fl, buf2)) == NULL)
					temp->action_description=0;

				if (!get_line(fl, line) ||
					 (sscanf(line, "%d %s %d %d %d", t, f1, t+2, t+3, t+4) != 5)) {
					fprintf(stderr, "Format error in first numeric line (expecting _x_ args)");
					return (0);
				}
				temp->obj_flags.type_flag = t[0];
				ASCIICONV(temp->obj_flags.wear_flags, f1);
				temp->obj_flags.weight = t[2];
				temp->obj_flags.cost = t[3];
				temp->obj_flags.cost_per_day = t[4];

				/* buf2 is error codes pretty much */
				strcat(buf2, ", after numeric constants (expecting E/#xxx)");

				/* we're clearing these for good luck */

				for (j = 0; j < MAX_OBJ_AFFECT; j++) {
					temp->affected[j].location = APPLY_NONE;
					temp->affected[j].modifier = 0;
				}

				if (temp->ex_description)
					temp->ex_description = NULL;
				
				get_line(fl,line);
				for (k = j = zwei = 0; !zwei && !feof(fl);) {
					switch (*line) {
						case 'E':
							CREATE(new_descr, struct extra_descr_data, 1);
							new_descr->keyword = fread_string(fl, buf2);
							new_descr->description = fread_string(fl, buf2);
							new_descr->next = temp->ex_description;
							temp->ex_description = new_descr;
							get_line(fl,line);
							break;
						case 'T': /* Object Timers */
							if (!get_line(fl, line)) {
								log("XAPERR: Format error in 'T' field, %s\n"
									 "...expecting numeric constant but file ended!",
									 temp->name);
								danger = 1;
							}

							if ((retval = sscanf(line, "%d", t)) != 1) {
								log("XAPERR: Format error in 'T' field, %s\n"
									 "...expecting 1 numeric argument, got %d\n"
									 "...offending line: '%s'",
									 temp->name, retval, line);
								danger = 1;
							}
							temp->obj_flags.timer = t[0];
							get_line(fl,line);
							break;
						case 'A':
							if (j >= MAX_OBJ_AFFECT) {
								log("XAPERR: Too many A fields (%d max), %s", 
									 MAX_OBJ_AFFECT, buf2);
								danger = 1;
							}
							if (!get_line(fl, line)) {
								log("SYSERR: Format error in 'A' field, %s\n"
									 "...expecting 2 numeric constants but file ended!",
									 temp->name);
								danger = 1;
							}

							if ((retval = sscanf(line, " %d %d ", t, t + 1)) != 2) {
								log("SYSERR: Format error in 'A' field, %s\n"
									 "...expecting 2 numeric arguments, got %d\n"
									 "...offending line: '%s'", 
									 temp->name, retval, line);
								danger = 1;
							}
							temp->affected[j].location = t[0];
							temp->affected[j].modifier = t[1];  
							j++;
							get_line(fl,line);
							break;
						case 'C':
							if (!get_line(fl, line)) {
								log("SYSERR: Format error in 'C' field, %s\n"
									 "...expecting numeric constant but file ended!", 
									 temp->name);
								danger = 1;
							}
				
							if ((retval = sscanf(line, "%d", t)) != 1) {
								log("SYSERR: Format error in 'C' field, %s\n"
									 "...expecting 1 numeric argument, got %d\n"
									 "...offending line: '%s'", 
									 temp->name, retval, line);
								danger = 1;
							}
				
							snprintf(f1, sizeof(f1), "%d\0", t[0]);
							ASCIICONV(temp->obj_flags.bitvector, f1);
							get_line(fl,line);
							break;
						case 'S':  /* Weapon Spells */
							if (wsplnum > MAX_WEAPON_SPELLS) {
								log("SYSERR: Too many S fields (%d max), %s", 
									 MAX_WEAPON_SPELLS, temp->name);
								danger = 1;
							}
							if (!get_line(fl, line)) {
								log("SYSERR: Format error in 'S' field, %s.  "
									 "Expecting numeric constants, but file ended!",
									 temp->name);
								danger = 1;
							}

							if ((retval = sscanf(line, " %d %d %d ", t, t+1, t+2)) != 3) {
								log("SYSERR: Format error in 'S' field, %s\n"
									 "...expecting 3 numeric argumentss, got %d\n"
									 "...offending line: '%s'", 
									 temp->name, retval, line);
								danger = 1;
							}

							temp->has_spells = TRUE;
							temp->wpn_spells[wsplnum].spellnum = t[0];
							temp->wpn_spells[wsplnum].level    = t[1];
							temp->wpn_spells[wsplnum].percent  = t[2];
							wsplnum += 1;
							get_line(fl,line);
							break;
						case '$':
						case '#':
							zwei = 1;
							break;
						default:
							zwei = 1;
							break;
					}
				}      /* exit our for loop */
				get_line(fl,line);
			}   /* exit our xap loop */

			if (temp)
				obj_to_char(temp, ch);
		}
	}  /* exit our while loop */
	return (num_of_objs);
}

