/**************************************************************************
 * The Mythran Mud Economy Snippet Version 3 - updated for CircleMUD use!
 *
 * Copyrights and rules for using the economy system:
 *
 *      The Mythran Mud Economy system was written by The Maniac, it was
 *      loosly based on the rather simple 'Ack!'s banking system'
 *
 *      If you use this code you must follow these rules.
 *              -Keep all the credits in the code.
 *              -Mail Maniac (v942346@si.hhs.nl) to say you use the code
 *              -Send a bug report, if you find 'it'
 *              -Credit me somewhere in your mud.
 *              -Follow the envy/merc/diku license
 *              -If you want to: send me some of your code
 *
 * All my snippets can be found on http://www.hhs.nl/~v942346/snippets.html
 * Check it often because it's growing rapidly  -- Maniac --
 *
 **************************************************************************
 *
 * Version 3 represents a port to CircleMUD done by Edward Glamkowski.
 * Earlier version were written for use with Envy MUD - please contact
 * the Maniac with any questions concerning Envy MUD or the use of this
 * code in that MUD base.  If you are using CircleMUD and have any 
 * comments, questions or concerns, feel free to contact me at:
 * eglamkowski@angelfire.com
 *
 * Please follow the above usage rules, and please also include me in
 * your credits! =)
 *
 **************************************************************************/

#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"

#include "coins.h"
#include "comm.h"
#include "db.h"
#include "economy.h"
#include "handler.h"
#include "interpreter.h"


int share_value = SHARE_VALUE;      /* External share_value by Maniac */


/* Local functions */
ACMD(do_bank);
ACMD(do_update);
void bank_update(void);
void load_bank(void);

/* Extern variables */
extern struct room_data *world;
extern struct time_info_data time_info;


SPECIAL(bank_teller)
{
	/* The Mythran mud economy system (bank and trading)
	 *
	 * based on:
	 * Simple banking system. by -- Stephen --
	 *
	 * The following changes and additions where
	 * made by the Maniac from Mythran Mud
	 * (v942346@si.hhs.nl)
	 *
	 * History:
	 * 18/05/96:     Added the transfer option, enables chars to transfer
	 *               money from their account to other players' accounts
	 * 18/05/96:     Big bug detected, can deposit/withdraw/transfer
	 *               negative amounts (nice way to steal is
	 *               bank transfer -(lots of dogh)
	 *               Fixed it (thought this was better... -= Maniac =-)
	 * 21/06/96:     Fixed a bug in transfer (transfer to MOBS)
	 *               Moved balance from ch->balance to ch->pcdata->balance
	 * 21/06/96:     Started on the invest option, so players can invest
	 *               money in shares, using buy, sell and check
	 *               Finished version 1.0 releasing it monday 24/06/96
	 * 24/06/96:     Mythran Mud Economy System V1.0 released by Maniac
	 *
	 *********************************************************************
	 * Further changes by Edward Glamkowski (eglamkowski@angelfire.com):
	 *
	 * 09/05/99:     
	 * 1. Ported to circle mud.
	 *               
	 * 2. Moved all bank code to this file (bank_update, load_bank) and created
	 *    an economy.h with detailed notes on relevent variables.  Both invest
	 *    and transfer options work, though the invest code is boring (need to
	 *    tie it in to the amount of business the shops do!) and the transfer
	 *    option is a really bad hack, code-wise.
	 *
	 * 3. Converted to be a SPECIAL() instead of an ACMD().  Lots of clean-up
	 *    to account for this.  Be sure to assign it to a mob to make use of it 
	 *    (an example mob, #2, is provided).  Note that this proc can safely
	 *    co-exist with the obj bank proc.
	 *
	 * 4. No longer uses do_say(), but send_to_char() in all cases.
	 *
	 * 5. Added a 'shares' option, to query how many shares you have, but it
	 *    does not tell you about net worth.
	 *
	 * 6. Rewrote load_bank() to report errors and recover from them.
	 */
	struct char_data *mob = (struct char_data *)me;
	struct char_data *victim = NULL;
	int amount = 0, type, transfer_amount;
	char buf1[MAX_INPUT_LENGTH] = "";
	char buf2[MAX_INPUT_LENGTH] = "";
	char buf3[MAX_INPUT_LENGTH] = "";
	char buf4[MAX_INPUT_LENGTH] = "";
	char bal[MAX_STRING_LENGTH], currency[MAX_INPUT_LENGTH];

	if (cmd != find_command("bank"))
		/* Pass control back to the main interpreter. -ejg */
		return (FALSE);

	if (mob == NULL) {
		/* Should never happen, but just in case... -ejg */
		send_to_char(ch, "You can't do that here.\r\n");
		log("SYSERR: NULL me pointer in bank_teller()!");
		return (FALSE);
	}

	if (IS_NPC(ch)) {
		send_to_char(ch, "Banking Services are only available to players!\r\n");
		return (TRUE);
	}

	if (!argument || !*argument) {
		send_to_char(ch, "Bank Options:\r\n\r\n");
		send_to_char(ch, "Bank balance  : Displays your balance.\r\n");
		send_to_char(ch, "Bank deposit  : Deposit money into your account.\r\n");
		send_to_char(ch, "Bank withdraw : Withdraw money from your account.\r\n");
#ifdef BANK_TRANSFER
		send_to_char(ch, "Bank transfer : Transfer money to another player's account.\r\n");
#endif
#ifdef BANK_INVEST
		send_to_char(ch, "Bank buy #    : Buy # shares\r\n");
		send_to_char(ch, "Bank sell #   : Sell # shares\r\n");
		send_to_char(ch, "Bank shares   : Check how many shares you own\r\n");
		send_to_char(ch, "Bank check    : Check the current rates of the shares.\r\n");
#endif
		return (TRUE);
	}

	/* Yuck - this is a horrible hack, but we may need a third argument
	 * if we are doing a transfer option... -ejg */
	four_arguments(argument, buf1, buf2, buf3, buf4);


	/* Now work out what to do... */
	if (!str_cmp(buf1, "balance")) {
		send_to_char(ch, "%s tells you, 'Your current balance is:'\r\n", 
						 CAP(GET_NAME(mob)));
		send_to_char(ch, list_from_copper(ch, GET_BANK_GOLD(ch), bal, UNCONDENSED_LIST));
		send_to_char(ch, "\r\n");
		return (TRUE);
	}

	if (!str_cmp(buf1, "deposit")) {
		if (!*buf2 || !is_number(buf2)) {
			send_to_char(ch, "How much do you want to deposit?\r\n");
			return (TRUE);
		}

		amount = atoi(buf2);
		strlcpy(currency, buf3, sizeof(currency));
		if (!strcmp(currency, "platinum") || is_abbrev(currency, "platinum"))
			type = PLAT_COINS;
		else if (!strcmp(currency, "gold") || is_abbrev(currency, "gold"))
			type = GOLD_COINS;
		else if (!strcmp(currency, "silver") || is_abbrev(currency, "silver"))
			type = SILVER_COINS;
		else if (!strcmp(currency, "copper") || is_abbrev(currency, "copper"))
			type = COPPER_COINS;
		else {
			send_to_char(ch, "Deposit what kind of coins?\r\n");
			return (FALSE);
		}

		if (GET_MONEY(ch, type) < amount) {
			send_to_char(ch, "%s asks you, 'How can you deposit %d %s when "
							 "you only have %d?'\r\n", CAP(GET_NAME(mob)), amount,
							 coin_types[type], GET_MONEY(ch, type));
			return (TRUE);
		}
		GET_MONEY(ch, type) -= amount;
		GET_BANK_GOLD(ch) += convert_to_copper(amount, type);
		send_to_char(ch, 
						 "%s tells you, 'You deposit %d %s coins.\r\n"
						 "Your new balance is %s.'\r\n", 
						 CAP(GET_NAME(mob)), amount, coin_types[type], 
						 list_from_copper(ch, GET_BANK_GOLD(ch), bal, UNCONDENSED_LIST));
		return (TRUE);
	} /* deposit */
    
#ifdef BANK_TRANSFER
	if (!str_cmp(buf1, "transfer")) {
		victim = get_char_vis(ch, buf2, NULL, FIND_CHAR_WORLD);
		/* Blah - needs to be able to transfer to anybody, including
		 * players you can't see or players not currently logged on.  
		 * -ejg */

		if (victim == NULL) {
			send_to_char(ch, "%s asks you, 'Transfer money to WHOSE "
							 "account?!'\r\n", CAP(GET_NAME(mob)));
			return (TRUE);
		}

		if (IS_NPC(victim)) {
			send_to_char(ch, "%s tells you, 'You can't transfer money to "
							 "them!\r\n", CAP(GET_NAME(mob)));
			return (TRUE);
		}

		if (!*buf3 || !is_number(buf3)) {
			send_to_char(ch, "How much do you want to transfer?\r\n");
			return (TRUE);
		}

		amount = atoi(buf3);
		strlcpy(currency, buf4, sizeof(currency));
		if (!strcmp(currency, "platinum") || is_abbrev(currency, "platinum"))
			type = PLAT_COINS;
		else if (!strcmp(currency, "gold") || is_abbrev(currency, "gold"))
			type = GOLD_COINS;
		else if (!strcmp(currency, "silver") || is_abbrev(currency, "silver"))
			type = SILVER_COINS;
		else if (!strcmp(currency, "copper") || is_abbrev(currency, "copper"))
			type = COPPER_COINS;
		else {
			send_to_char(ch, "Transfer what kind of coins?\r\n");
			return (FALSE);
		}

		transfer_amount = convert_to_copper(amount, type);
		
		if (transfer_amount > GET_BANK_GOLD(ch)) {
			send_to_char(ch, "%s tells you, 'You do not have that much to be"
							 "transferred!\r\n", CAP(GET_NAME(mob)));
			return (TRUE);
		}
		GET_BANK_GOLD(ch) -= transfer_amount;
		GET_BANK_GOLD(victim) += transfer_amount;
		send_to_char(ch, "%s tells you, 'You transfer %d %s to %s.  Your "
						 "new balance is %s.'\r\n", CAP(GET_NAME(mob)), amount,
						 coin_types[type], GET_NAME(victim), 
						 list_from_copper(ch, GET_BANK_GOLD(ch), bal, UNCONDENSED_LIST));
		send_to_char(victim, "[BANK] %s has transferred %d %s to your "
						 "account.\r\n",  GET_NAME(ch), amount, coin_types[type]);
		return (TRUE);
	}
#endif

	if (!str_cmp(buf1, "withdraw")) {
		if (!*buf2 || !is_number(buf2)) {
			send_to_char(ch, "How much do you want to withdraw?\r\n");
			return (TRUE);
		}

		amount = atoi(buf2);
		strlcpy(currency, buf3, sizeof(currency));
		if (!strcmp(currency, "platinum") || is_abbrev(currency, "platinum"))
			type = PLAT_COINS;
		else if (!strcmp(currency, "gold") || is_abbrev(currency, "gold"))
			type = GOLD_COINS;
		else if (!strcmp(currency, "silver") || is_abbrev(currency, "silver"))
			type = SILVER_COINS;
		else if (!strcmp(currency, "copper") || is_abbrev(currency, "copper"))
			type = COPPER_COINS;
		else {
			send_to_char(ch, "Deposit what kind of coins?\r\n");
			return (FALSE);
		}

		transfer_amount = convert_to_copper(amount, type);
		if (GET_BANK_GOLD(ch) < amount) {
			send_to_char(ch, "%s asks you, 'How can you withdraw %d %s "
							 "when your balance is only %d?'\r\n",
							 CAP(GET_NAME(mob)), amount, coin_types[type], 
							 list_from_copper(ch, GET_BANK_GOLD(ch), bal, UNCONDENSED_LIST));
			return (TRUE);
		}
		GET_BANK_GOLD(ch) -= transfer_amount;
		GET_MONEY(ch, type) += amount;

		send_to_char(ch, 
						 "%s tells you, 'You withdraw %d %s coins.\r\n"
						 "Your new balance is %s.'\r\n", 
						 CAP(GET_NAME(mob)), amount, coin_types[type], 
						 list_from_copper(ch, GET_BANK_GOLD(ch), bal, UNCONDENSED_LIST));
		return (TRUE);
	}

#ifdef BANK_INVEST
	if (!str_cmp(buf1, "buy")) {
		if (!*buf2 || !is_number(buf2)) {
			send_to_char(ch, "%s asks you, 'How many shares do you wish to "
							 "buy?'\r\n", CAP(GET_NAME(mob)));
			return (TRUE);
		}
		amount = atoi(buf2);
		
		if ((amount * share_value) > GET_BANK_GOLD(ch)) {
			send_to_char(ch, "%s tells you, '%d shares will cost you %d copper, "
							 "get more money!'\r\n", CAP(GET_NAME(mob)), amount,
							 (amount * share_value));
			return (TRUE);
		}

		GET_BANK_GOLD(ch) -= (amount * share_value);
		GET_SHARES(ch) += amount;
		send_to_char(ch, "%s tells you, 'You buy %d shares for %d copper,\r\n"
						 "you now have %d shares.'\r\n", CAP(GET_NAME(mob)), 
						 amount, (amount * share_value), GET_SHARES(ch));
		return (TRUE);
	}

	if (!str_cmp(buf1, "sell")) {
		if (!*buf2 || !is_number(buf2)) {
			send_to_char(ch, "%s asks you, 'How many shares do you wish to "
							 "sell?'\r\n", CAP(GET_NAME(mob)));
			return (TRUE);
		}
		amount = atoi(buf2);

		if (amount > GET_SHARES(ch)) {
			send_to_char(ch, "%s tells you, 'You only have %d "
							 "shares!'\r\n", CAP(GET_NAME(mob)), GET_SHARES(ch));
			return (TRUE);
		}

		GET_BANK_GOLD(ch) += (amount * share_value);
		GET_SHARES(ch) -= amount;
		send_to_char(ch, "%s informs you, 'You sell %d shares for %d "
						 "copper,\r\nyou now have %d shares.\r\n", 
						 CAP(GET_NAME(mob)), amount, (amount * share_value),
						 GET_SHARES(ch));
		return (TRUE);
	}

	if (!str_cmp(buf1, "shares")) {
		send_to_char(ch, "%s informs you, 'You have %d shares.'\r\n", 
						 CAP(GET_NAME(mob)), GET_SHARES(ch));
		return (TRUE);
	}

	if (!str_cmp(buf1, "check")) {
		send_to_char(ch, "%s informs you, 'The current shareprice is "
						 "%d copper.'\r\n", CAP(GET_NAME(mob)), share_value);

		if (GET_SHARES(ch)) {
			send_to_char(ch, "%s informs you further, 'You currently have %d "
							 "shares.\r\nAt %d a share, your net worth is %d "
							 "copper.'\r\n", CAP(GET_NAME(mob)), GET_SHARES(ch), 
							 share_value,  (GET_SHARES(ch) * share_value));
		}
		return (TRUE);
	}
#endif

	return (FALSE);
}


ACMD(do_update)
{
	char arg[MAX_INPUT_LENGTH];
	
	one_argument(argument, arg);

	if (!arg || !*arg) {
		send_to_char(ch, "Current options to update: \r\n\r\n");
		send_to_char(ch, "bank: Update the stock market ticker.\r\n");
		return;
	}
  
	if (!str_cmp(arg, "bank")) {
		bank_update();
		send_to_char(ch, "Ok...bank updated.\r\n");
	}
	/* Add new update options here */
}

		  
/*
 * Update the bank system
 * (C) 1996 The Maniac from Mythran Mud
 *
 * This updates the shares value (I hope)
 *
 * (It does, but not in an interesting way!  
 *  To the IMP reading this: change it! -ejg)
 */
void bank_update(void)
{
	int   value = 0;
	FILE *fp;

	if ((time_info.hours > 17))
		return;         /* Bank is closed, no market... */
  
	value = rand_number(0, 200);
	value -= 100;
	value /= 10;
  
	share_value += value;
  
	if (!(fp = fopen(BANK_FILE, "w"))) {
		log("bank_update:  Couldn't open file %s", BANK_FILE);
		return;
	}

	fprintf(fp, "SHARE_VALUE %d\r\n", share_value);
	fclose(fp);
}


/*
 * Load the bank information (economy info)
 * By Maniac from Mythran Mud
 *
 * Totally re-written by Ed Glamkowski, now includes error checking
 * and recovery.
 */
void load_bank(void)
{
	FILE *fp;
	char word[256];
	int  number = 0;

	if (!(fp = fopen(BANK_FILE, "r")))
		return;

	if (fscanf(fp, "%s %d", word, &number) != 2) {
		log("SYSERR:  Failed to load bank, using default value");
		number = SHARE_VALUE;
	}

	if (word == NULL || str_cmp(word, "SHARE_VALUE")) {
		log("SYSERR:  Error in bank file (encountered %s).  "
			 "Using default value", word);
		number = SHARE_VALUE;
	}

	share_value = number;
}
