#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"

#include "comm.h"
#include "db.h"
#include "event.h"
#include "handler.h"


int in_event_handler = 0;
struct complex_event_data *pending_complex_events = NULL;
struct complex_event_data *prev_complex_event = NULL;


void complex_event_activity(void) 
{
  struct complex_event_data *temp, *prox;

  in_event_handler = 1;
  prev_complex_event = NULL;

  for (temp = pending_complex_events; temp; temp = prox) {
    prox = temp->next;
    temp->ticks_to_go--;

    if (temp->ticks_to_go == 0) { /* run the event */
      if (!temp->func)
        log("SYSERR: Attempting to run a NULL event. Ignoring");
      else
        (temp->func)(temp->causer, temp->victim, (long)temp->info);

      /* remove the event from the list. */
      if (!prev_complex_event)
        pending_complex_events = prox;
      else
        prev_complex_event->next = prox;

      free(temp);
    } else if (temp->ticks_to_go == -1) {
      if (!prev_complex_event)
        pending_complex_events = prox;
      else
        prev_complex_event->next = prox;

      free(temp);
    } else  
      prev_complex_event = temp;
  }
  in_event_handler = 0;
}


void add_complex_event(int delay, EVENT(*func), void *causer, void *victim, void *info)
{
  struct complex_event_data *new;

  CREATE(new, struct complex_event_data, 1);

  new->ticks_to_go = delay;
  new->func   = func;
  new->causer = causer;
  new->victim = victim;
  new->info   = info;

  new->next = pending_complex_events;
  pending_complex_events = new;

  if (in_event_handler && !prev_complex_event)
    prev_complex_event = pending_complex_events;
}


void kill_complex_events(void *pointer)
{
  struct complex_event_data *temp, *prox;
  struct complex_event_data *previous;

  if (in_event_handler)
    log("SYSERR: Trying to remove events inside the handler. Attempting "
		  "to continue.");

  previous = NULL;

  for (temp = pending_complex_events; temp; temp = prox) {
    prox = temp->next;
    if (temp->causer == pointer || 
		  temp->victim == pointer ||
        (void *)(temp->func) == pointer)
      temp->ticks_to_go = 0;
  }
}


/* ************************************ *
 *  Pre-defined events.(Just examples)  *
 * ************************************ */
#if 0
extern struct room_data *world;

void death_cry(struct char_data *ch);

/* Using this example will require extensive changes elsewhere. */

EVENT(teleport_event)
{
  int destin = 0;

  /* Is the character still in the same room? */
  if (GET_LEVEL(VICTIM_CH) < LVL_GOD && IN_ROOM(VICTIM_CH) == info) {

    /* Tell the other players in the room about the teleport. */
    act(world[IN_ROOM(VICTIM_CH)].teleport->message_you, 
		  TRUE, VICTIM_CH, NULL, VICTIM_CH, TO_VICT);
    act(world[IN_ROOM(VICTIM_CH)].teleport->message_oth,
		  FALSE, VICTIM_CH, NULL, VICTIM_CH, TO_NOTVICT);

    /* Perform the teleport. */
    if ((destin = real_room(world[info].teleport->to_room)) != NOWHERE) {
      char_from_room(VICTIM_CH);
      char_to_room(VICTIM_CH, destin);
      look_at_room(VICTIM_CH, 1);

      if (ROOM_FLAGGED(IN_ROOM(VICTIM_CH), ROOM_DEATH) && 
			 GET_LEVEL(VICTIM_CH) < LVL_IMMORT) {
        log_death_trap(VICTIM_CH);
        death_cry(VICTIM_CH);
        extract_char(VICTIM_CH);
      }
    } else
      log("SYSERR: Trying to teleport to unexisting room. Ignoring.");
  };
};
#endif
