/* ***********************************************************************\
 *  _____ _            ____                  _       _                     *
 * |_   _| |__   ___  |  _ \  ___  _ __ ___ (_)_ __ (_) ___  _ __          *
 *   | | | '_ \ / _ \ | | | |/ _ \| '_ ` _ \| | '_ \| |/ _ \| '_ \         *
 *   | | | | | |  __/ | |_| | (_) | | | | | | | | | | | (_) | | | |        *
 *   |_| |_| |_|\___| |____/ \___/|_| |_| |_|_|_| |_|_|\___/|_| |_|        *
 *                                                                         *
 *  File:  GAMBLE.C                                 Based on CircleMUD 3.0 *
 *  Usage: Implementation of casino games                                  *
 *  Programmer(s): Original code by the EnvyMUD Development Team           *
 *                 All Modifications by Sean Mountcastle (Glasgian)        *
 \*********************************************************************** */

#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"

#include "coins.h"
#include "comm.h"
#include "db.h"
#include "handler.h"
#include "interpreter.h"
#include "screen.h"
#include "spells.h"


/* New for TD 6/10/95 */
void play_slots(struct char_data *ch)
{
	int num1, num2, num3, win = 0;
	char *slot_msg[] = {
		"*YOU SHOULDN'T SEE THIS*",
		"a mithril bar",              /* 1 */
		"a golden dragon",
		"a Dwarven hammer",
		"a temple",
		"an Elven bow",               /* 5 */
		"a red brick",
		"a refuse pile",
		"a waybread",
		"a Gnomish bell",
		"a beggar",                   /* 10 */
	};

	if (convert_all_to_copper(ch) < 5) {
		send_to_char(ch, "You do not have enough money to play the slots!\r\n");
		return;
	} else
		make_change(ch, 5, BUYING);

	act("$N pulls on the crank of the Gnomish slot machine.",
		 FALSE, ch, 0, 0, TO_ROOM);
	send_to_char(ch, "You pull on the crank of the Gnomish slot machine.\r\n");

	/* very simple roll 3 random numbers from 1 to 10 */
	num1 = rand_number(1, 10);
	num2 = rand_number(1, 10);
	num3 = rand_number(1, 10);

	if (num1 == num2 && num2 == num3) {
		/* all 3 are equal! Woohoo! */
		if (num1 == 1)
			win += 1000;
		else if (num1 == 2)
			win += 250;
		else if (num1 == 3)
			win += 100;
		else if (num1 == 4)
			win += 50;
		else if (num1 == 5)
			win += 25;
		else if (num1 == 10)
			win += 5;
	}

	send_to_char(ch, "You got %s, %s, %s, ", 
					 slot_msg[num1], slot_msg[num2], slot_msg[num3]);

	if (win > 1)
		send_to_char(ch, "you win %d copper pieces!\r\n", win);
	else
		send_to_char(ch, "you lose.\r\n");

	add_money_to_char(ch, win, COPPER_COINS);
}


void play_high_dice(struct char_data *ch, struct char_data *dealer, int bet)
{
	int die1, die2, die3, die4;
	char buf[MAX_STRING_LENGTH];

	if (convert_all_to_copper(ch) < bet) {
		act("$n says, 'I'm sorry sir but you don't have that much money.'",
			 FALSE, dealer, 0, ch, TO_VICT);
		return;
	} else if (bet > 5000) {
		act("$n says, 'I'm sorry sir but the limit at this table is 5000 "
			 "copper pieces.'", FALSE, dealer, 0, ch, TO_VICT);
		return;
	} else {
		make_change(ch, bet, BUYING);
		act("$N places $S bet on the table.", FALSE, dealer, 0, ch, TO_NOTVICT);
		send_to_char(ch, "You place your bet on the table.\r\n");
	}

	/* dealer rolls two dice */
	die1 = rand_number(1, 6);
	die2 = rand_number(1, 6);

	snprintf(buf, sizeof(buf),
				"$n rolls the dice, $e gets %d, and %d, for a total of %d.",
				die1, die2, (die1 + die2));
	act(buf, FALSE, dealer, 0, ch, TO_ROOM);
	/* now its the players turn */
	die3 = rand_number(1, 6);
	die4 = rand_number(1, 6);

	snprintf(buf, sizeof(buf),
				"$N rolls the dice, $E gets %d, and %d, for a total of %d.",
				die3, die4, (die3 + die4));
	act(buf, FALSE, dealer, 0, ch, TO_NOTVICT);
	send_to_char(ch, "You roll the dice, and get %d, and %d, for a total "
					 "of %d.\r\n", die3, die4, (die3 + die4));

	if ((die1+die2) >= (die3+die4)) {
		snprintf(buf, sizeof(buf), "The house wins %d coins from $N.", bet);
		act(buf, FALSE, dealer, 0, ch, TO_NOTVICT);

		send_to_char(ch, "The house wins %d coins from you.\r\n", bet);
	} else {
		snprintf(buf, sizeof(buf), "$N wins %d copper coins!", bet*2);
		act(buf, FALSE, dealer, 0, ch, TO_NOTVICT);

		send_to_char(ch, "You win %d copper coins!\r\n", bet*2);

		add_money_to_char(ch, bet*2, COPPER_COINS);
	}
}


void play_triples(struct char_data *ch, struct char_data *dealer, char *guess, int bet)
{
	int die1, die2, die3, total = 0;
	char buf[MAX_STRING_LENGTH];

	if (convert_all_to_copper(ch) < bet) {
		act("$n says, 'I'm sorry sir but you don't have that much money.'",
			 FALSE, dealer, 0, ch, TO_VICT);
		return;
	} else if (bet > 5000) {
		act("$n says, 'I'm sorry sir but the limit at this table is 5000 "
			 "copper pieces.'", FALSE, dealer, 0, ch, TO_VICT);
		return;
	} else {
		make_change(ch, bet, BUYING);
		act("$N places $S bet on the table.", FALSE, dealer, 0, ch, TO_NOTVICT);
		send_to_char(ch, "You place your bet on the table.\r\n");
	}

	if (!*guess) {
		act("$n tells you, 'You need to specify upper, lower, or triple.'",
			 FALSE, dealer, 0, ch, TO_VICT);
		act("$n hands your bet back to you.", FALSE, dealer, 0, ch, TO_VICT);
		add_money_to_char(ch, bet, COPPER_COINS);
		return;
	}

	if (!(!str_cmp(guess, "upper") || 
			!str_cmp(guess, "lower") || 
			!str_cmp(guess, "triple"))) {
		act("$n tells you, 'You need to specify upper, lower, or triple.'",
			 FALSE, dealer, 0, ch, TO_VICT);
		act("$n hands your bet back to you.", FALSE, dealer, 0, ch, TO_VICT);
		add_money_to_char(ch, bet, COPPER_COINS);
		return;
	}

	die1 = rand_number(1, 6);
	die2 = rand_number(1, 6);
	die3 = rand_number(1, 6);

	total = die1 + die2 + die3;

	snprintf(buf, sizeof(buf), "$N rolls %d, %d, and %d", die1, die2, die3);
	if (die1 == die2 && die2 == die3)
		strcat(buf, ", $E scored a triple!");
	else
		strcat(buf, ".");
	act(buf, FALSE, dealer, 0, ch, TO_NOTVICT);
	send_to_char(ch, "You roll %d, %d, and %d, for a total of %d.\r\n", 
					 die1, die2, die3, total);

	if ((die1 == die2 && die2 == die3) && !str_cmp(guess, "triple")) {
		/* scored a triple! player wins 3x the bet */
		act("$n says, 'Congratulations $N, you win.'", 
			 FALSE, dealer, 0, ch, TO_ROOM);
		snprintf(buf, sizeof(buf), "$n hands you %d copper pieces.", (bet*3));
		act(buf, FALSE, dealer, 0, ch, TO_VICT);
		act("$n hands $N some copper pieces.", 
			 FALSE, dealer, 0, ch, TO_NOTVICT);
		add_money_to_char(ch, bet*3, COPPER_COINS);
	} else if ((total <= 9 && !str_cmp(guess, "lower")) ||
				  (total > 9  && !str_cmp(guess, "upper"))) {
		act("$n says, 'Congratulations $N, you win.'", 
			 FALSE, dealer, 0, ch, TO_ROOM);
		snprintf(buf, sizeof(buf), "$n hands you %d copper pieces.", (bet*2));
		act(buf, FALSE, dealer, 0, ch, TO_VICT);
		act("$n hands $N some copper pieces.", 
			 FALSE, dealer, 0, ch, TO_NOTVICT);
		add_money_to_char(ch, bet*2, COPPER_COINS);
	} else {
		act("$n says, 'Sorry $N, better luck next time.'", 
			 FALSE, dealer, 0, ch, TO_ROOM);
		act("$n greedily counts $s new winnings.", 
			 FALSE, dealer, 0, ch, TO_ROOM);
	}
}


void play_seven(struct char_data *ch, struct char_data *dealer, char *guess, int bet)
{
	int die1, die2, total = 0;
	char buf[MAX_STRING_LENGTH];

	if (convert_all_to_copper(ch) < bet) {
		act("$n says, 'I'm sorry sir but you don't have that much money.'",
			 FALSE, dealer, 0, ch, TO_VICT);
		return;
	} else if (bet > 5000) {
		act("$n says, 'I'm sorry sir but the limit at this table is 5000 "
			 "copper pieces.'", FALSE, dealer, 0, ch, TO_VICT);
		return;
	} else {
		make_change(ch, bet, BUYING);
		act("$N places $S bet on the table.", FALSE, dealer, 0, ch, TO_NOTVICT);
		send_to_char(ch, "You place your bet on the table.\r\n");
	}

	if (!*guess) {
		act("$n tells you, 'You need to specify under, over, or seven.'",
			 FALSE, dealer, 0, ch, TO_VICT);
		act("$n hands your bet back to you.", FALSE, dealer, 0, ch, TO_VICT);
		add_money_to_char(ch, bet, COPPER_COINS);
		return;
	}
	if (!(!str_cmp(guess, "under") || \
			!str_cmp(guess, "over")  || \
			!str_cmp(guess, "seven"))) {
		act("$n tells you, 'You need to specify under, over, or seven.'",
			 FALSE, dealer, 0, ch, TO_VICT);
		act("$n hands your bet back to you.", FALSE, dealer, 0, ch, TO_VICT);
		add_money_to_char(ch, bet, COPPER_COINS);
		return;
	}

	act("$n says, 'Roll the dice $N and tempt lady luck.'",
		 FALSE, dealer, 0, ch, TO_ROOM);

	die1 = rand_number(1, 6);
	die2 = rand_number(1, 6);
	total = die1 + die2;

	snprintf(buf, sizeof(buf), 
				"$N rolls the dice, getting a %d and a %d.", die1, die2);
	act(buf, FALSE, dealer, 0, ch, TO_NOTVICT);
	send_to_char(ch, "You roll the dice, they come up %d and %d.\r\n", die1, die2);

	if ((total < 7 && !str_cmp(guess, "under")) || \
		 (total > 7 && !str_cmp(guess, "over"))) {
		/* player wins 2x $s money */
		act("$n says, 'Congratulations $N, you win!'", FALSE, dealer, 0, ch, TO_ROOM);
		act("$n hands $N some copper pieces.", FALSE, dealer, 0, ch, TO_NOTVICT);
		snprintf(buf, sizeof(buf), "$n hands you %d copper pieces.", (bet*2));
		act(buf, FALSE, dealer, 0, ch, TO_VICT);
		add_money_to_char(ch, bet*2, COPPER_COINS);
	} else if (total == 7 && !str_cmp(guess, "seven")) {
		/* player wins 5x $s money */
		act("$n says, 'Congratulations $N, you win!'", FALSE, dealer, 0, ch, TO_ROOM);
		act("$n hands $N some copper pieces.", FALSE, dealer, 0, ch, TO_NOTVICT);
		snprintf(buf, sizeof(buf), "$n hands you %d copper pieces.", (bet*5));
		act(buf, FALSE, dealer, 0, ch, TO_VICT);
		add_money_to_char(ch, bet*5, COPPER_COINS);
	} else {
		/* player loses */
		act("$n says, 'Sorry $N, you lose.'", FALSE, dealer, 0, ch, TO_ROOM);
		act("$n takes $N's bet from the table.", FALSE, dealer, 0, ch, TO_NOTVICT);
		act("$n takes your bet from the table.", FALSE, dealer, 0, ch, TO_VICT);
	}
}


void play_craps(struct char_data *ch, struct char_data *dealer, int bet)
{
	int  die1, die2, mark = 0, last = 0;
	int won = FALSE, firstime = TRUE;
	char buf[MAX_STRING_LENGTH];

	if (convert_all_to_copper(ch) < bet) {
		act("$n says, 'I'm sorry sir but you don't have that much money.'",
			 FALSE, dealer, 0, ch, TO_VICT);
		return;
	} else if (bet > 10000) {
		act("$n says, 'I'm sorry sir but the limit at this table is 10000 "
			 "copper pieces.'", FALSE, dealer, 0, ch, TO_VICT);
		return;
	} else {
		make_change(ch, bet, BUYING);
		act("$N places $S bet on the table.", FALSE, dealer, 0, ch, TO_NOTVICT);
		send_to_char(ch, "You place your bet on the table.\r\n");
	}

	act("$n hands $N the dice and says, 'roll 'em'", 
		 FALSE, dealer, 0, ch, TO_NOTVICT);
	act("$n hands you the dice and says, 'roll 'em'", 
		 FALSE, dealer, 0, ch, TO_VICT);

	while (won != TRUE) {
		die1 = rand_number(1, 6);
		die2 = rand_number(1, 6);
		mark = die1 + die2;

		snprintf(buf, sizeof(buf), "$n says, '$N rolls %d and %d, totalling %d.",
					die1, die2, mark);
		act(buf, FALSE, dealer, 0, ch, TO_ROOM);

		if ((mark == 7  || mark == 11) && firstime) {
			/* win on first roll of the dice! 3x bet */
			act("$n says, 'Congratulations $N, you win!'", 
				 FALSE, dealer, 0, ch, TO_ROOM);
			act("$n hands $N some copper pieces.", 
				 FALSE, dealer, 0, ch, TO_NOTVICT);
			snprintf(buf, sizeof(buf), "$n hands you %d copper pieces.", (bet*3));
			act(buf, FALSE, dealer, 0, ch, TO_VICT);
			add_money_to_char(ch, bet*3, COPPER_COINS);
			won = TRUE;
		} else if (mark == 3 || mark == 12) {
			/* player loses on first roll */
			act("$n says, 'Sorry $N, you lose.'", 
				 FALSE, dealer, 0, ch, TO_ROOM);
			act("$n takes $N's bet from the table.",
				 FALSE, dealer, 0, ch, TO_NOTVICT);
			act("$n takes your bet from the table.",
				 FALSE, dealer, 0, ch, TO_VICT);
			won = TRUE;
		} else if ((last == mark) && !firstime) {
			/* player makes $s mark! 2x bet */
			act("$n says, 'Congratulations $N, you win!'", 
				 FALSE, dealer, 0, ch, TO_ROOM);
			act("$n hands $N some copper pieces.", 
				 FALSE, dealer, 0, ch, TO_NOTVICT);
			snprintf(buf, sizeof(buf), "$n hands you %d copper pieces.", (bet*2));
			act(buf, FALSE, dealer, 0, ch, TO_VICT);
			add_money_to_char(ch, bet*2, COPPER_COINS);
			won = TRUE;
		} else {
			snprintf(buf, sizeof(buf), 
						"$n says, '$N's mark is %d.  Roll 'em again $N!'", mark);
			act(buf, FALSE, dealer, 0, ch, TO_ROOM);
			firstime = FALSE;
			last = mark;
			won = FALSE;
		}
	}
}


void play_blackjack(struct char_data *ch, struct char_data *dealer, char *guess, int bet)
{
	send_to_char(ch, "This is currently unavailable\r\n");
	return;
}
